<?php
    
namespace HCM\FPA\ObrigacaoLegal\Rais;

use HCM\Geral\Util\MsgCalculo;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use HCM\FPA\ObrigacaoLegal\Rais\Tipo00;

class QuerysRais {
    public function __construct($entityManager, $hcmParameters) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
     public function getValoresMensaisRais($nrvinculom, $nrorg, $datainicial, $datafinal, $grupoincide, $nrestrutlegal=null) {
        
        $params = array(
            'grupoincide' => $grupoincide,
            'datainicial' => $datainicial,
            'datafinal' => $datafinal,
            'datainicial' => $datainicial,
            'nrvinculom' => $nrvinculom,
            'nrorg' => $nrorg
        );
        
        if ($nrestrutlegal) {
            $condestrutlegal = 'AND vh.nrestrutlegal = :nrestrutlegal';
            $params['nrestrutlegal'] = $nrestrutlegal;
        }else{
            $condestrutlegal = '';
        }
        $sql = $this->entityManager->getConnection()->prepare("
SELECT nritem,
    NVL(janeiro,0)   as janeiro,
    NVL(fevereiro,0) as fevereiro,
    NVL(marco,0)     as marco,
    NVL(abril,0)     as abril,
    NVL(maio,0)      as maio,
    NVL(junho,0)     as junho,
    NVL(julho,0)     as julho,
    NVL(agosto,0)    as agosto,
    NVL(setembro,0)  as setembro,
    NVL(outubro,0)   as outubro,
    NVL(novembro,0)  as novembro,
    NVL(dezembro,0)  as dezembro
FROM(
    SELECT
        fpa_grpitem.nritem,
        fpa_incidevento.idincidevento,
        decode(fpa_grpitem.nritem, 1, vreventofolha, 2, vrreffolha, 0) valoritem,
        to_char(case when fpa_calculofolha.dtocorrencia > vm.dtrescisaovinc then vm.dtrescisaovinc else fpa_calculofolha.dtocorrencia end, 'mm') dtocorrencia
    FROM fpa_incidevento
    INNER JOIN fpa_grpitem
        ON fpa_incidevento.nrtipoincide = fpa_grpitem.nrtipoincide
    INNER JOIN fpa_itecalcfolha
        ON fpa_itecalcfolha.nreventom = fpa_incidevento.nreventom
    INNER JOIN gpe_vinculom vm
        ON vm.nrorg = fpa_itecalcfolha.nrorg
        AND vm.nrvinculom = fpa_itecalcfolha.nrvinculom
    INNER JOIN gpe_vinculoh vh
        ON vh.nrorg = fpa_itecalcfolha.nrorg
        AND vh.nrvinculom = fpa_itecalcfolha.nrvinculom
    INNER JOIN fpa_calculofolha
        ON fpa_itecalcfolha.nrcalculofolha = fpa_calculofolha.nrcalculofolha
        AND fpa_itecalcfolha.nrorg = fpa_calculofolha.nrorg
    WHERE fpa_grpitem.nrgrpincide = :grupoincide
        AND fpa_calculofolha.dtocorrencia between to_date(:datainicial) AND to_date(:datafinal)
        ".$condestrutlegal."
        AND vh.dtmescompetenc = ( SELECT max(dtmescompetenc) FROM gpe_vinculoh
                                   WHERE nrorg = vh.nrorg
                                     AND nrvinculom = vh.nrvinculom
                                     AND dtmescompetenc <= fpa_calculofolha.dtocorrencia
                                     AND dtmescompetenc >= trunc(to_date(:datainicial),'year') )
        AND (fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,1,1) OR 
             fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,2,1))  
        AND fpa_calculofolha.nrtpmodalidcal = 1
        AND fpa_itecalcfolha.nrvinculom = :nrvinculom
        AND fpa_itecalcfolha.nrorg = :nrorg
        AND (fpa_incidevento.nrorg = :nrorg OR fpa_incidevento.nrorg = 0)
) pivot (
    SUM(decode(idincidevento, 'n', 0, '+', valoritem, valoritem*-1))
    for dtocorrencia in (
        '01'    as  janeiro,
        '02'    as  fevereiro,
        '03'    as  marco,
        '04'    as  abril,
        '05'    as  maio,
        '06'    as  junho,
        '07'    as  julho,
        '08'    as  agosto,
        '09'    as  setembro,
        '10'    as  outubro,
        '11'    as  novembro,
        '12'    as  dezembro))
");

        
        $sql->execute($params);
        $result = $sql->fetchAll();
        $return = array();
        foreach($result as $item){
          $return[$item['NRITEM']] = $item;  
        };
        return $return;
        
    }
    
    public function getValoresTotaisAnoAntRais($nrvinculom, $nrorg, $dataInicial, $dataFinal, $grupoIncide)
    {
        $sql = $this->entityManager->getConnection()->prepare("
    SELECT
        fpa_grpitem.nritem,
        SUM(decode(fpa_incidevento.idincidevento, 'n', 0, '+', fpa_itecalcfolha.vreventofolha, fpa_itecalcfolha.vreventofolha*-1)) valor_total
      from fpa_incidevento
      join fpa_grpitem on fpa_incidevento.nrtipoincide = fpa_grpitem.nrtipoincide
      join fpa_itecalcfolha on fpa_itecalcfolha.nreventom = fpa_incidevento.nreventom
      join fpa_calculofolha on fpa_itecalcfolha.nrcalculofolha = fpa_calculofolha.nrcalculofolha
                           and fpa_itecalcfolha.nrorg = fpa_calculofolha.nrorg
     where fpa_grpitem.nrgrpincide = :grupoincide
       and fpa_calculofolha.dtocorrencia between :datainicial and :datafinal
       and (fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,2) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,7,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,8,1))
       and fpa_calculofolha.nrtpmodalidcal = 1
       and fpa_itecalcfolha.nrvinculom = :nrvinculom
       and fpa_itecalcfolha.nrorg = :nrorg
       and fpa_itecalcfolha.vreventofolha > 0
       and (fpa_incidevento.nrorg = :nrorg or fpa_incidevento.nrorg = 0)
     group by
        fpa_grpitem.nritem
");

        $params = array(
            'grupoincide' => $grupoIncide,
            'datainicial' => $dataInicial,
            'datafinal' => $dataFinal,
            'nrvinculom' => $nrvinculom,
            'nrorg' => $nrorg
        );

        $sql->execute($params);
        $result = $sql->fetchAll();
        return $result;
    }
    
    public function getvalorestotaisraisorg20($nrvinculom, $nrorg, $datainicial, $datafinal, $grupoincide, $dtrescisaovinc, $nrestrutlegal=null, $nrestrutgeren=null)
    {
        $params = array(
        "datainicial" => $datainicial,
        "datafinal" => $datafinal,
        "grupoincide" => $grupoincide,
        "nrvinculom" => $nrvinculom,
        "nrorg" => $nrorg,
        "mesrescisaovinc" => null

        );
        if ($dtrescisaovinc) {
            $datarescisao = dateutil::getdatadestring($dtrescisaovinc);
            $datainicio = dateutil::getdatadestring($datainicial);
            $datafim = dateutil::getdatadestring($datafinal);
            
            if (($datarescisao > $datainicio && $datarescisao < $datafim) ||$datainicio->format('d/m/y') === $datarescisao->format('d/m/y')||$datarescisao->format('d/m/y') === $datafim->format('d/m/y')) {
                $mesrescisaovinc = substr($dtrescisaovinc, 3, 2);
                $params["mesrescisaovinc"] = $mesrescisaovinc;
            }
            
        }
        if ($nrestrutlegal) {
            $condestrutlegal = 'and vh.nrestrutlegal = :nrestrutlegal';
            $params['nrestrutlegal'] = $nrestrutlegal;
        }else{
            $condestrutlegal = '';
        }

        $sql = $this->entityManager->getConnection()->prepare("
select 
nritem,
  min(case when nritem = 3 and referencia < inicio then 99 else referencia end) referencia,
  count(*) quantmeses,
  sum(valor_total) valor_total
from (
    select
        fpa_grpitem.nritem,
        to_number(to_char(to_date(:datainicial), 'mm')) inicio,
        to_number(to_char(eh.dtextincao, 'mm')) extincao,
        case
          when fpa_grpitem.nritem = 9 then fpa_itecalcfolha.vrreffolha
          when fpa_grpitem.nritem = 4 and fpa_itecalcfolha.nreventom = 120 then 12
          else to_number(to_char(fpa_itecalcfolha.dtocorrencia, 'mm'))
        end referencia,
        sum(decode(fpa_incidevento.idincidevento, 'n', 0, '+', fpa_itecalcfolha.vreventofolha, fpa_itecalcfolha.vreventofolha*-1)) valor_total
      from fpa_incidevento
      join fpa_grpitem on fpa_incidevento.nrtipoincide = fpa_grpitem.nrtipoincide
      join fpa_itecalcfolha on fpa_itecalcfolha.nreventom = 
                               case
                                 when fpa_incidevento.nrtipoincide = 102 and fpa_incidevento.nreventom = 120 and fpa_itecalcfolha.nrorg = 20
                                 then 1500
                                 else fpa_incidevento.nreventom
                               end
      join gpe_vinculoh vh on vh.nrorg = fpa_itecalcfolha.nrorg
                          and vh.nrvinculom = fpa_itecalcfolha.nrvinculom
      join estruturah eh on eh.nrorg = vh.nrorg
                        and eh.nrestruturam = vh.nrestrutlegal
                        and eh.dtmescompetenc = ( select max(dtmescompetenc) from estruturah
                                                   where nrorg = eh.nrorg
                                                     and nrestruturam = eh.nrestruturam
                                                     and dtmescompetenc <= :datafinal )
      join fpa_calculofolha on fpa_itecalcfolha.nrcalculofolha = fpa_calculofolha.nrcalculofolha
                           and fpa_itecalcfolha.nrorg = fpa_calculofolha.nrorg
     where fpa_grpitem.nrgrpincide = :grupoincide
       and fpa_calculofolha.dtocorrencia between (case when fpa_grpitem.nritem in (3,4) and vh.nrorg = 20 then trunc(to_date(:datainicial),'year') else to_date(:datainicial) end) and :datafinal
        $condestrutlegal
        and vh.dtmescompetenc = ( select max(dtmescompetenc) from gpe_vinculoh
                                   where nrorg = vh.nrorg
                                     and nrvinculom = vh.nrvinculom
                                     and dtmescompetenc <= (case when fpa_grpitem.nritem in (3,4) and vh.nrorg = 20 then to_date(:datafinal) else fpa_calculofolha.dtocorrencia end)
                                     and dtmescompetenc >= (case when fpa_grpitem.nritem in (3,4) and vh.nrorg = 20 then trunc(to_date(:datainicial),'year') else to_date(:datainicial) end) )
       and (fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,3,decode(fpa_incidevento.nreventom,760,4,1))    
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,3,4) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,4,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,4,4) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,2) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,6,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,7,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,8,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,9,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,10,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,11,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,12,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,13,1))
       and fpa_calculofolha.nrtpmodalidcal = 1
       and fpa_itecalcfolha.nrvinculom = :nrvinculom
       and fpa_itecalcfolha.nrorg = :nrorg
       and fpa_itecalcfolha.vreventofolha > 0
       and (fpa_incidevento.nrorg = :nrorg or fpa_incidevento.nrorg = 0)
       and ((fpa_grpitem.nritem = 4 and :mesrescisaovinc is not null and fpa_itecalcfolha.nreventom not in (110, 125)) or
            (fpa_grpitem.nritem = 4 and :mesrescisaovinc is null and fpa_itecalcfolha.nreventom <> 600) or
            fpa_grpitem.nritem <> 4)
     group by
        fpa_grpitem.nritem,
        to_number(to_char(to_date(:datainicial), 'mm')),
        to_number(to_char(eh.dtextincao, 'mm')),
        case
          when fpa_grpitem.nritem = 9 then fpa_itecalcfolha.vrreffolha
          when fpa_grpitem.nritem = 4 and fpa_itecalcfolha.nreventom = 120 then 12
          else to_number(to_char(fpa_itecalcfolha.dtocorrencia, 'mm'))
        end
)
where ((nritem = 3 and referencia < 12 and (extincao is null or extincao >= 11 or extincao >= to_number(:mesrescisaovinc))) or (nritem = 4 and (referencia = 12 or referencia >= :mesrescisaovinc)) or (nritem <> 3 and nritem <> 4))
and valor_total > 0
group by nritem
");

        $sql->execute($params);
        $result = $sql->fetchAll(); 
        return $result;
    }
    
    public function getValoresTotaisRais($nrvinculom, $nrorg, $datainicial, $datafinal, $grupoincide, $dtrescisaovinc, $nrestrutlegal=null, $nrestrutgeren=null)
    {
        $mesrescisaovinc = substr($dtrescisaovinc,3,2);
        $condestrutlegal = $nrestrutlegal?('and vh.nrestrutlegal = '.$nrestrutlegal):'';
        
        $params = array(
        ':grupoincide' => $grupoincide,
        ':datainicial' => $datainicial,
        ':datafinal' => $datafinal,
        ':nrvinculom' => $nrvinculom,
        ':nrorg' => $nrorg,
        ':mesrescisaovinc' => $mesrescisaovinc
        );
        
        $sql = $this->entityManager->getConnection()->prepare("
        select nritem,
  min(referencia) referencia,
  count(*) quantmeses,
  sum(valor_total) valor_total
from (
    select
        fpa_grpitem.nritem,
        case
          when fpa_grpitem.nritem = 9 then fpa_itecalcfolha.vrreffolha
          when fpa_grpitem.nritem = 4 and fpa_itecalcfolha.nreventom = 120 then 12
          else to_number(to_char(fpa_itecalcfolha.dtocorrencia, 'mm'))
        end referencia,
        sum(decode(fpa_incidevento.idincidevento, 'n', 0, '+', fpa_itecalcfolha.vreventofolha, fpa_itecalcfolha.vreventofolha*-1)) valor_total
      from fpa_incidevento
      join fpa_grpitem on fpa_incidevento.nrtipoincide = fpa_grpitem.nrtipoincide
      join fpa_itecalcfolha on fpa_itecalcfolha.nreventom = fpa_incidevento.nreventom
      join gpe_vinculoh vh on vh.nrorg = fpa_itecalcfolha.nrorg
                          and vh.nrvinculom = fpa_itecalcfolha.nrvinculom
      join fpa_calculofolha on fpa_itecalcfolha.nrcalculofolha = fpa_calculofolha.nrcalculofolha
                           and fpa_itecalcfolha.nrorg = fpa_calculofolha.nrorg
     where fpa_grpitem.nrgrpincide = :grupoincide
       and fpa_calculofolha.dtocorrencia between :datainicial and :datafinal
        ".$condestrutlegal."
        and vh.dtmescompetenc = ( select max(dtmescompetenc) from gpe_vinculoh
                                   where nrorg = vh.nrorg
                                     and nrvinculom = vh.nrvinculom
                                     and dtmescompetenc <= fpa_calculofolha.dtocorrencia
                                     and dtmescompetenc >= :datainicial )
       and (fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,3,1)    
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,3,4) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,4,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,4,4)  
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,5,2) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,6,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,7,1)     
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,8,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,9,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,10,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,11,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,12,1) 
            or fpa_calculofolha.nrtipomoviment = decode(fpa_grpitem.nritem,13,1))
       and fpa_calculofolha.nrtpmodalidcal = 1
       and fpa_itecalcfolha.nrvinculom = :nrvinculom
       and fpa_itecalcfolha.nrorg = :nrorg
       and fpa_itecalcfolha.vreventofolha > 0
       and (fpa_incidevento.nrorg = :nrorg or fpa_incidevento.nrorg = 0)
       and ((fpa_grpitem.nritem = 4 and ':mesrescisaovinc' is not null and fpa_itecalcfolha.nreventom not in (110, 125)) or
            (fpa_grpitem.nritem = 4 and ':mesrescisaovinc' is null and fpa_itecalcfolha.nreventom <> 600) or
            fpa_grpitem.nritem <> 4)
     group by
        fpa_grpitem.nritem,
        case
          when fpa_grpitem.nritem = 9 then fpa_itecalcfolha.vrreffolha
          when fpa_grpitem.nritem = 4 and fpa_itecalcfolha.nreventom = 120 then 12
          else to_number(to_char(fpa_itecalcfolha.dtocorrencia, 'mm'))
        end
)
where ((nritem = 3 and referencia < 12) or (nritem = 4 and (referencia = 12 or referencia >= :mesrescisaovinc)) or (nritem <> 3 and nritem <> 4))
group by nritem
");

        $sql->execute($params);
        $result = $sql->fetchAll(); 
        return $result;
    }

}

