<?php

namespace HCM\VAB\Controllers;

use HCM\Util\Messages;
use HCM\VAB\Services\CalculoValeTransporteService;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class CalculoValeTransporteController extends Crud {

    protected $dataSourceName = 'fpa_formvales';
    protected $dataSourceName_get = 'fpa_formvales_get';

    protected $parameterBagColumns = array('P_NRORG', 'P_CDOPERINCLUSAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, CalculoValeTransporteService $calculoValeTransporteService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->calculoValeTransporteService = $calculoValeTransporteService;
        parent::__construct($dataSourceManager);
    }

    public function getFormvales (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName_get);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        
        $rows = $dataSet->getRows();

        foreach($rows as $key => &$row){
            if($row['DSORDEMVALE'] == 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['NRVINCULOM_LIST'];
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = $row['LIST_NMVINCULOM'];
                $row['LIST_NMVINCULOM'] = explode(",", $row['LIST_NMVINCULOM']);
            }    
            
            if($row['DSORDEMVALE'] == 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIOVALE']);
                $row['DSINICIO'] = $row['LIST_NRESTRUTLEGAL'];
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['NMESTRUTLEGAL'] = $row['LIST_NMESTRUTLEGAL'];
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['LIST_NMESTRUTLEGAL']);
                
                if($row['DSFIMVALE']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIMVALE']);
                    $row['DSFIMVALE'] = $row['LIST_NRESTRUTGEREN'];
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    $row['NMESTRUTGEREN'] = $row['LIST_NMESTRUTGEREN'];
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['LIST_NMESTRUTGEREN']);
                }
            }   
            
            if($row['DSORDEMVALE'] === "LISTA_NRESTRUTSIND"){
                if($row['DSFIMVALE']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIMVALE']);
                    $row['DSFIMVALE'] = $row['LIST_NRESTRUTSIND'];
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                    
                    $row['NMESTRUTSIND'] = $row['LIST_NMESTRUTSIND'];
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['LIST_NMESTRUTSIND']);
                }
            }
        }

        $response->addDataSet(new DataSet($this->dataSourceName_get, $rows));
    }

    public function setFormvales (DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('CDOPERADOR', $row)) {
                    if ($row['CDOPERADOR'] === '') {
                        $row['CDOPERADOR'] = $this->hcmParameters->getCdoperador();
                    }
                } else {
                    $row['CDOPERADOR'] = $this->hcmParameters->getCdoperador();
                }
                if (!array_key_exists('NRTIPOLINHAVINC', $row)) {
                    $row['NRTIPOLINHAVINC'] = null;
                }
                if (array_key_exists('NRFORMVALE', $row)) {
                    if ($row['NRFORMVALE'] === '') {
                        $row['NRFORMVALE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FORMVALES', 1, 12, true);
                    }
                } else {
                    $row['NRFORMVALE'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgPadrao(), 'FPA_FORMVALES', 1, 12, true);
                }
                
                 // define os campos de descrição de início e fim com base no critério de seleção passado pela row
                if($row['DSORDEMVALE'] === "LISTA_VINCULO"){
                    if(is_array($row['NRVINCULOM_LIST'])){
                        $row['DSINICIOVALE'] = implode(',',$row['NRVINCULOM_LIST']);
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['LIST_NRESTRUTLEGAL'])){
                        $row['DSINICIOVALE'] = implode(',',$row['LIST_NRESTRUTLEGAL']);
                    }
                    if(isset($row['LIST_NRESTRUTGEREN']) && is_array($row['LIST_NRESTRUTGEREN']) && count($row['LIST_NRESTRUTGEREN']) > 0 && is_array($row['LIST_NRESTRUTGEREN'])){
                        $row['DSFIMVALE'] = implode(',',$row['LIST_NRESTRUTGEREN']);
                    }else{
                        if(is_array($row['DSFIMVALE'])){
                            $row['DSFIMVALE'] = null;
                        }
                    }
                }
                
                if($row['DSORDEMVALE'] === "LISTA_NRESTRUTSIND"){
                    if(count($row['LIST_NRESTRUTSIND']) > 0 && is_array($row['LIST_NRESTRUTSIND'])){
                        $row['DSFIMVALE'] = implode(',',$row['LIST_NRESTRUTSIND']);
                    }else{
                        if(is_array($row['LIST_NRESTRUTSIND'])){
                            $row['DSFIMVALE'] = null;
                        }
                    }
                }
            }
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function calcular(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $idMemoria = $row['IDMEMORIACALC'];
                $dsordem = $row['DSORDEMVALE'];
                $dsiniciovale = $row['DSINICIOVALE'];
                $dsfimvale = $row['DSFIMVALE'];
                $dtiniciovale = $row['DTINICIOVALE'];
                $dtfinalvale = $row['DTFINALVALE'];
                $dtpagamento = $row['DTPAGAMENTOVALE'];
                $nrtipolinhavinc = $row['NRTIPOLINHAVINC'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();
                
                $this->calculoValeTransporteService->calcular($response, $idMemoria, $dsordem, $dsiniciovale, $dsfimvale, $dtiniciovale, $dtfinalvale, $dtpagamento, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    public function retroceder(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                $dsordem = $row['DSORDEMVALE'];
                $dsiniciovale = $row['DSINICIOVALE'];
                $dsfimvale = $row['DSFIMVALE'];
                $nrtipolinhavinc = $row['NRTIPOLINHAVINC'];
                $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
                $nrorg = $this->hcmParameters->getNrorgTrabalho();

                // Por ser retrocesso deve-se gravar o log da operação
                $this->calculoValeTransporteService->persistirLogRetrocesso($row);

                $this->calculoValeTransporteService->retroceder($dsordem, $dsiniciovale, $dsfimvale, $nrtipolinhavinc, $dtmescompetenc, $nrorg, $row);
            }
        } catch (\Exception $e) {
            if ($e->getCode() === Messages::SUCCESS_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_MESSAGE));
            } elseif ($e->getCode() === Messages::ALERT_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_WARNING));
            } elseif ($e->getCode() === Messages::ERROR_MESSAGE_CODE) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($e->getMessage(), DTO\Response\Message::TYPE_ERROR));
            } else {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}