<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class ESocialEventoNaoPerRepository {

    protected  $connection;

    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }

    /*-------------------------- Métodos para retornar os dados do eventos -----------------------------------------------*/
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2190($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2190*/
SELECT PE.NRPESSOA CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,   
       --infoRegPrelim
       REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS CPFTRAB,
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') AS dtNascto,
       TO_CHAR(PH.DTADMISSAOPRE,'YYYY-MM-DD') AS dtAdm  
  FROM GPE_PESSOA PE
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE PE.NRORG = :P_NRORG
   AND PE.NRPESSOA = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2200($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc, $dataEnvioOrig) {
        $sql = <<<SQL
/*getDadosS2200*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --trabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') cpfTrab,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       PH.NMPESSOA nmTrab,
       PH.IDSEXOPESSOA sexo,
       RP.CDESOCIAL racaCor,
       EC.CDESOCIAL estCiv,
       GI.CDESOCIAL grauInstr,
       DECODE(VH.NRTPMOVTRANSFM,1,'S','N') indPriEmpr,
       '' nmSoc,
       --nascimento
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       DECODE(PH.CDPAIS,'0055',MU.CDMUNICIBGE,'') codMunic,
       DECODE(PH.CDPAIS,'0055',PH.SGESTADO,'') uf,
       PA.CDESOCIAL paisNascto,
       PB.CDESOCIAL paisNac,
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 1) nmMae, -- 1=Mae
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 2) nmPai, -- 2=Pai
       --documentos
       --CTPS
       PH.NRCTPSPESSOA nrCtps,
       SUBSTR(PH.NRSERIECTPSPES,1,5) serieCtps,
       PH.SGUFCTPSPES ufCtps,
       --RIC
       --RG
       PH.NRRGPESSOA nrRg,
       PH.CDEXRGPESSOA rg_orgaoEmissor,
       TO_CHAR(PH.DTEXRGPESSOA,'YYYY-MM-DD') rg_dtExped,
       --RNE
       PH.NRDOCUESTRANG nrRne,
       PH.DSEXPDOCESTR rne_orgaoEmissor,
       '' rne_dtExped,
       --OC
       --CNH
       --So traz dados CNH se tiver UF da CNH (mobile)
       DECODE(PH.SGUFHABCARPES, NULL, '', PH.NRCARTHABPES) nrRegCnh,
       DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTHABCARTPES,'YYYY-MM-DD')) cnh_dtExped,
       PH.SGUFHABCARPES ufCnh,
       DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTVALHABCARTPES,'YYYY-MM-DD')) cnh_dtValid,
       '' dtPriHab,
       DECODE(PH.SGUFHABCARPES, NULL, '', PH.DSCATEGHABCART) categoriaCnh,
       --endereco
       --brasil
       NVL(LG.CDESOCIAL,'O') tpLograd,
       EP.DSENDERECO dscLograd,
       EP.NRIMOVELENDERECO nrLograd,
       SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) complemento,
       EP.NMBAIRROENDERECO bairro,
       REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') cep,
       MP.CDMUNICIBGE brasil_codMunic,
       EP.SGESTADO brasil_uf,
       --exterior
       PC.CDESOCIAL paisResid,
       EE.DSENDERECO exterior_dscLograd,
       EE.NRIMOVELENDERECO exterior_nrLograd,
       SUBSTR(EE.DSCOMPLEENDERECO, 0, 30) exterior_complemento,
       EE.NMBAIRROENDERECO exterior_bairro,
       ME.NMMUNICIPIO nmCid,
       REPLACE(REPLACE(REPLACE(EE.NRCEPENDERECO,' '),'.'),'-') codPostal,
       --trabEstrangeiro
       TO_CHAR(PH.DTCHEGADAPAIS,'YYYY-MM-DD') dtChegada,
       NVL(CT.CDESOCIAL,DECODE(PH.IDPOSVISTOPERM,'S','1','')) classTrabEstrang,
       CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA = 4 -- 4=Cônjuge
               AND H.CDPAIS = '0055'),'N') END casadoBr,
       CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA IN (3, 100001) -- 3=Filho
               AND H.CDPAIS = '0055'),'N') END filhosBr,
       --infoDeficiencia
       DECODE(PH.NRCONDFISPES,'2','S','N') defFisica,
       DECODE(PH.NRCONDFISPES,'4','S','N') defVisual,
       DECODE(PH.NRCONDFISPES,'3','S','N') defAuditiva,
       DECODE(PH.NRCONDFISPES,'5','S','N') defMental,
       DECODE(PH.NRCONDFISPES,'8','S','N') defIntelectual,
       DECODE(PH.NRCONDFISPES,'7','S','N') reabReadap,
       NVL(PH.IDCOTAPCD, 'N') infoCota,
       '' observacao,
       --<<dependentes>>-- getDadosS2200dependente
       --aposentadoria
       '' trabAposent, --Facultativo (até 31/12/2017 se {codCateg} = [104]). Não obrigatorio (nas demais situações).
       --contato
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '01') fonePrinc, -- 1-Telefone principal
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU IN ('02','03','04','06')) foneAlternat, -- 2-Celular particular 3-Telefone residencial 4-Telefone comercial 6-Celular comercial
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '05') emailPrinc, -- 5-email
       '' emailAlternat,
       --vinculo
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) matricula,
       '1' tpRegTrab,
       VH.NRTPREGIMEPREV tpRegPrev,
       PH.CDRECIBOADMPRE nrRecInfPrelim,
       CASE WHEN NVL(MO.DTINIMOVIMENT,VM.DTADMISSAOVINC) < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN 'S' ELSE 'N' END cadIni,
       --infoRegimeTrab
       --infoCeletista
       TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD') dtAdm,
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '1', DECODE(SUBSTR(MT.CDCNPJESTRUT,1,8), SUBSTR(PAR.NRINSCRICAOPARC,1,8), '1', '2')), '1') tpAdmissao,
       '1' indAdmissao,
       '1' tpRegJor,
       CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END natAtividade,
       TO_CHAR(ES.DTBASESINDICAL, 'MM') dtBase,
       REPLACE(REPLACE(REPLACE(ES.CDCNPJESTRUT,'.'),'/'),'-') cnpjSindCategProf,
       --FGTS
       DECODE(VM.DTOPCAOFGTS,NULL,'2','1') opcFGTS,
       TO_CHAR(VM.DTOPCAOFGTS,'YYYY-MM-DD') dtOpcFGTS,
       --trabTemporario
       DECODE(CE.CDESOCIAL,'106','2',NULL) hipLeg,
       DECODE(CE.CDESOCIAL,'106','Demada complemetar de serviços',NULL) justContr,
       ----ideTomadorServ
       DECODE(CE.CDESOCIAL,'106',DECODE(DECODE(PNT.NRINSCRICAOPARC,NULL,PN.CDTIPOINSCRICAO,PNT.CDTIPOINSCRICAO),'CNPJ','1','CPF','2',''),NULL) ideTomadorServ_TpInsc,
       DECODE(CE.CDESOCIAL,'106',DECODE(PNT.NRINSCRICAOPARC,NULL,PN.NRINSCRICAOPARC,PNT.NRINSCRICAOPARC),NULL) ideTomadorServ_NrInsc,
       --aprendiz
       --infoEstatutario
       --infoContrato
       VH.NRCARGO codCargo,
       VH.NRFUNCAO codFuncao,
       CE.CDESOCIAL codCateg,
       '' codCarreira,
       '' dtIngrCarr,
       --remuneracao
       ROUND(SL.VRSALARIO, 2) vrSalFx,
       DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') undSalFixo,
       SUBSTR(SL.DSOBSERVACAO,1,90) dscSalVar,
       --duracao
       CASE WHEN VM.NRVINCULOEMPREG IN (2, 3, 10, 11, 12, 13) OR (VM.NRVINCULOEMPREG = 1 AND VM.DTVENCCONTREXPER IS NOT NULL) THEN '2' ELSE '1' END tpContr,
       CASE WHEN VM.NRVINCULOEMPREG IN (2, 3, 10, 11, 12, 13) OR (VM.NRVINCULOEMPREG = 1 AND VM.DTVENCCONTREXPER IS NOT NULL) THEN TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD') ELSE '' END dtTerm,
       CASE WHEN VM.NRVINCULOEMPREG IN (2, 3, 10, 11, 12, 13) OR (VM.NRVINCULOEMPREG = 1 AND VM.DTVENCCONTREXPER IS NOT NULL) THEN 'N' ELSE '' END  clauAsseg, --v_2.3
       CASE WHEN VM.NRVINCULOEMPREG IN (2, 3, 10, 11, 12, 13) OR (VM.NRVINCULOEMPREG = 1 AND VM.DTVENCCONTREXPER IS NOT NULL) THEN 'N' ELSE '' END  clauAssec, --v_2.4.01
       --localTrabalho
       --localTrabGeral
       DECODE(CE.CDESOCIAL,'106',NULL,DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')) localTrabGeral_TpInsc,
       DECODE(CE.CDESOCIAL,'106',NULL,PN.NRINSCRICAOPARC) localTrabGeral_NrInsc,
       DECODE(CE.CDESOCIAL,'106',NULL,PN.NMPRINCIPALPARC) descComp,
       --localTrabDom
       DECODE(CE.CDESOCIAL,'106',NVL(LGD.CDESOCIAL,'O'),NULL) localTrabDom_TpLograd,
       DECODE(CE.CDESOCIAL,'106',EPD.DSENDERECO,NULL) localTrabDom_DscLograd,
       DECODE(CE.CDESOCIAL,'106',EPD.NRIMOVELENDERECO,NULL) localTrabDom_NrLograd,
       DECODE(CE.CDESOCIAL,'106',SUBSTR(EPD.DSCOMPLEENDERECO, 0, 30),NULL) localTrabDom_Complemento,
       DECODE(CE.CDESOCIAL,'106',EPD.NMBAIRROENDERECO,NULL) localTrabDom_Bairro,
       DECODE(CE.CDESOCIAL,'106',REPLACE(REPLACE(REPLACE(EPD.NRCEPENDERECO,' '),'.'),'-'),NULL) localTrabDom_Cep,
       DECODE(CE.CDESOCIAL,'106',MPD.CDMUNICIBGE,NULL) localTrabDom_CodMunic,
       DECODE(CE.CDESOCIAL,'106',EPD.SGESTADO,NULL) localTrabDom_Uf,
       --horContratual
       EH.QTHRSEMESCTRABH qtdHrsSem,
       TJ.CDESOCIAL tpJornada,
       DECODE(TJ.CDESOCIAL,'9',EH.NMESCALATRABH,'') dscTpJorn,
       '0' tmpParc,
       --<<horario>>-- getDadosS2200horario
       --filiacaoSindical
       REPLACE(REPLACE(REPLACE(ES.CDCNPJESTRUT,'.'),'/'),'-') cnpjSindTrab,
       --alvaraJudicial
       --sucessaoVinc
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '', DECODE(MT.CDTIPOINSCRICAO,'CNPJ','1','CPF','2',''))) tpInscAnt,
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '', DECODE(SUBSTR(MT.CDCNPJESTRUT,1,8), SUBSTR(PAR.NRINSCRICAOPARC,1,8), '', REPLACE(REPLACE(REPLACE(MT.CDCNPJESTRUT,'.'),'/'),'-'))), '') cnpjEmpregAnt,
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '', DECODE(SUBSTR(MT.CDCNPJESTRUT,1,8), SUBSTR(PAR.NRINSCRICAOPARC,1,8), '', VM.NRVINCULOM)), '') matricAnt,
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '', DECODE(SUBSTR(MT.CDCNPJESTRUT,1,8), SUBSTR(PAR.NRINSCRICAOPARC,1,8), '', TO_CHAR(MO.DTINIMOVIMENT, 'YYYY-MM-DD'))), '') dtTransf,
       DECODE(MO.NRTPMOVTRANSFM, 8, DECODE(MT.CDCNPJESTRUT, NULL, '', DECODE(SUBSTR(MT.CDCNPJESTRUT,1,8), SUBSTR(PAR.NRINSCRICAOPARC,1,8), '', SUBSTR(MO.DSOBSERVACAO,1,255))), '') sucessaoVinc_Observacao,
       --afastamento
       TO_CHAR(
        CASE
            WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN NULL  /*Situacao Normal*/
            WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN GO.DTINIGOZOFERIAS  /*Em ferias*/
            WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN SF.DTINISITUFUNC  /*Afastado*/
            ELSE NULL
        END,
       'YYYY-MM-DD') dtIniAfast,
       CASE
        WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN ''  /*Situacao Normal*/
        WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN '15'  /*Em ferias*/
        WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN MA.CDESOCIAL  /*Afastado*/
        ELSE ''
       END codMotAfast,
       --desligamento
       CASE WHEN VM.DTRESCISAOVINC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN TO_CHAR(VM.DTRESCISAOVINC,'YYYY-MM-DD') ELSE NULL END dtDeslig,
       VM.NRVINCULOM, PE.NRORG, PE.NRPARCNEGOCIO, EH.NRESCALATRABM, AE.NRTURNO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  JOIN GPE_TIPOVINCULOH VT ON VT.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                          AND (VT.NRORG = VM.NRORG OR VT.NRORG = 0 OR VT.NRORG IS NULL)
                          AND VT.IDVINCULOEMPR = 'S'
                          AND VT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_TIPOVINCULOH
                                                    WHERE GPE_TIPOVINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND GPE_TIPOVINCULOH.NRTIPOVINCULOM = VT.NRTIPOVINCULOM
                                                      AND GPE_TIPOVINCULOH.NRORG = VT.NRORG)
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = VH.NRESTRUTLEGAL
                    AND ET.NRORG = VH.NRORG
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                     AND PN.NRORG = ET.NRORG
  /*Tomador de Serviços*/
  LEFT JOIN TIPOHIERARQORG TH ON (TH.NRORG = VM.NRORG OR TH.NRORG = 0)
                             AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                             AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
  LEFT JOIN HIERARQUIAORG HO ON HO.NRORG = VM.NRORG
                            AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
                            AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
                            AND HO.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA)
                                                      FROM HIERARQUIAORG HI
                                                     WHERE HI.NRORG = HO.NRORG
                                                       AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
                                                       AND HI.NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
                                                       AND HI.DTINIVIGENCIA <= :P_DTMESCOMPETENC)
                            AND (HO.DTFIMVIGENCIA >= LAST_DAY(:P_DTMESCOMPETENC) OR HO.DTFIMVIGENCIA IS NULL)
  LEFT JOIN ESTRUTURAM EMT ON EMT.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                          AND EMT.NRORG = HO.NRORG
  LEFT JOIN ESTRUTURAH EHT ON EHT.NRESTRUTURAM = EMT.NRESTRUTURAM
                          AND EHT.NRORG = EMT.NRORG
                          AND EHT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND ESTRUTURAH.NRESTRUTURAM = EHT.NRESTRUTURAM
                                                       AND ESTRUTURAH.NRORG = EHT.NRORG)
  LEFT JOIN PARCNEGOCIO PNT ON PNT.NRPARCNEGOCIO = EHT.NRPARCNEGOCIO
                           AND PNT.NRORG = EHT.NRORG
  /*Endereço Trabalho Domestico*/
  LEFT JOIN ENDERECOPARC EPD ON EPD.NRPARCNEGOCIO = NVL(PNT.NRPARCNEGOCIO,PN.NRPARCNEGOCIO)
                            AND EPD.NRORG = NVL(PNT.NRORG,PN.NRORG)
                            AND EPD.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MPD ON MPD.CDPAIS = EPD.CDPAIS
                         AND MPD.SGESTADO = EPD.SGESTADO
                         AND MPD.CDMUNICIPIO = EPD.CDMUNICIPIO
  LEFT JOIN LOGRADOURO LGD ON LGD.CDLOGRADOURO = EPD.CDLOGRADOURO
                         AND (LGD.NRORG = EPD.NRORG OR LGD.NRORG = 0)
  /*Dados do Empregado - Dependentes*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Dados do Empregado - Naturalidade*/
  LEFT JOIN PAIS PA ON PA.CDPAIS = PH.CDPAIS
  LEFT JOIN MUNICIPIO MU ON MU.CDPAIS = PH.CDPAIS
                        AND MU.SGESTADO = PH.SGESTADO
                        AND MU.CDMUNICIPIO = PH.CDMUNICIPIO
  LEFT JOIN GPE_NACIONALIDADE NA ON PH.NRNACIONALID = NA.NRNACIONALIDADE
                                AND (PH.NRORG = NA.NRORG OR NA.NRORG = 0)
  LEFT JOIN PAIS PB ON PB.CDPAIS = NA.CDPAIS
  /*Endereço Principal*/
  LEFT JOIN ENDERECOPARC EP ON PE.NRPARCNEGOCIO = EP.NRPARCNEGOCIO
                           AND PE.NRORG = EP.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MP ON MP.CDPAIS = EP.CDPAIS
                        AND MP.SGESTADO = EP.SGESTADO
                        AND MP.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Endereço Exterior*/
  LEFT JOIN ENDERECOPARC EE ON PE.NRPARCNEGOCIO = EE.NRPARCNEGOCIO
                           AND PE.NRORG = EE.NRORG
                           AND EE.CDTIPOENDERECO = 'EXTERIOR'
  LEFT JOIN MUNICIPIO ME ON ME.CDPAIS = EE.CDPAIS
                        AND ME.SGESTADO = EE.SGESTADO
                        AND ME.CDMUNICIPIO = EE.CDMUNICIPIO
  LEFT JOIN PAIS PC ON PC.CDPAIS = EE.CDPAIS
  /*Trabalhador Estrangeiro*/
  LEFT JOIN GPE_CLASSESTRANG CT ON CT.NRCLASSESTRANG = PH.NRCLASSESTRANG
  /*Movimentacao*/
  LEFT JOIN (SELECT MO.NRORG, MO.NRVINCULOM, MO.NRESTRUTURAM, MO.NRTPMOVTRANSFM, MO.DTINIMOVIMENT, MO.DSOBSERVACAO, ET.CDCNPJESTRUT
               FROM GPE_MOVIMENTACAO MO
               JOIN ESTRUTURAH ET ON ET.NRORG = MO.NRORG
                                 AND ET.NRESTRUTURAM = MO.NRESTRUTURAM
                                 AND ET.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = ET.NRORG
                                                             AND H1.NRESTRUTURAM = ET.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC < :P_DTMESCOMPETENC)
              WHERE MO.NRORG = :P_NRORG
                AND MO.NRVINCULOM = :P_NRCHAVE
                AND MO.NRTPMOVTRANSFM = 8
                AND MO.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL) MO ON MO.NRORG = VH.NRORG
                                                                 AND MO.NRVINCULOM = VH.NRVINCULOM
                                                                 AND SUBSTR(MO.CDCNPJESTRUT,1,8) = SUBSTR(PAR.NRINSCRICAOPARC,1,8)
                                                                 AND MO.DTINIMOVIMENT = (SELECT MIN(M1.DTINIMOVIMENT) FROM GPE_MOVIMENTACAO M1
                                                                                           JOIN ESTRUTURAH H1 ON H1.NRORG = M1.NRORG
                                                                                                             AND H1.NRESTRUTURAM = M1.NRESTRUTURAM
                                                                                                             AND H1.DTMESCOMPETENC = (SELECT MAX(H2.DTMESCOMPETENC) FROM ESTRUTURAH H2
                                                                                                                                       WHERE H2.NRORG = H1.NRORG
                                                                                                                                         AND H2.NRESTRUTURAM = H1.NRESTRUTURAM
                                                                                                                                         AND H2.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                                                                                          WHERE M1.NRORG = MO.NRORG
                                                                                            AND M1.NRVINCULOM = MO.NRVINCULOM
                                                                                            AND SUBSTR(H1.CDCNPJESTRUT,1,8) = SUBSTR(PAR.NRINSCRICAOPARC,1,8)
                                                                                            AND M1.NRTPMOVTRANSFM = 8
                                                                                            AND M1.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
                                                                                            AND M1.DTINIMOVIMENT < LAST_DAY(:P_DTMESCOMPETENC))
  /*Movimentacao Anterior*/
  LEFT JOIN (SELECT MT.NRORG, MT.NRVINCULOM, MT.NRESTRUTURAM, MT.DTINIMOVIMENT, ET.CDCNPJESTRUT, ETP.CDTIPOINSCRICAO
               FROM GPE_MOVIMENTACAO MT
               JOIN ESTRUTURAH ET ON ET.NRORG = MT.NRORG
                                 AND ET.NRESTRUTURAM = MT.NRESTRUTURAM
                                 AND ET.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                           WHERE H1.NRORG = ET.NRORG
                                                             AND H1.NRESTRUTURAM = ET.NRESTRUTURAM
                                                             AND H1.DTMESCOMPETENC < :P_DTMESCOMPETENC)
               JOIN PARCNEGOCIO ETP ON ETP.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                                   AND ETP.NRORG = ET.NRORG
              WHERE MT.NRORG = :P_NRORG
                AND MT.NRVINCULOM = :P_NRCHAVE
                AND MT.NRESTRUTURAM <> :P_NRESTRUTURAM
                AND MT.NRTIPOTRANSFER = 1
                AND MT.NRTPMOVTRANSFM <> 9
                AND MT.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL) MT ON MT.NRORG = VH.NRORG
                                                                 AND MT.NRVINCULOM = VH.NRVINCULOM
                                                                 AND SUBSTR(MT.CDCNPJESTRUT,1,8) <> SUBSTR(PAR.NRINSCRICAOPARC,1,8)
                                                                 AND MT.DTINIMOVIMENT = (SELECT MAX(M1.DTINIMOVIMENT) FROM GPE_MOVIMENTACAO M1
                                                                                           JOIN ESTRUTURAH H1 ON H1.NRORG = M1.NRORG
                                                                                                             AND H1.NRESTRUTURAM = M1.NRESTRUTURAM
                                                                                                             AND H1.DTMESCOMPETENC = (SELECT MAX(H2.DTMESCOMPETENC) FROM ESTRUTURAH H2
                                                                                                                                       WHERE H2.NRORG = H1.NRORG
                                                                                                                                         AND H2.NRESTRUTURAM = H1.NRESTRUTURAM
                                                                                                                                         AND H2.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                                                                                          WHERE M1.NRORG = MT.NRORG
                                                                                            AND M1.NRVINCULOM = MT.NRVINCULOM
                                                                                            AND SUBSTR(H1.CDCNPJESTRUT,1,8) <> SUBSTR(PAR.NRINSCRICAOPARC,1,8)
                                                                                            AND M1.NRTIPOTRANSFER = 1
                                                                                            AND M1.NRTPMOVTRANSFM <> 9
                                                                                            AND M1.NRTIPOESTRUTURA = :P_NRTIPOESTRUTLEGAL
                                                                                            AND M1.DTINIMOVIMENT < MO.DTINIMOVIMENT)
  /*Cargo*/
  LEFT JOIN GPE_OCUPACAOH OH ON OH.NROCUPACAOM = VH.NRCARGO
                            AND OH.NRORG = VH.NRORG
                            AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                        AND GPE_OCUPACAOH.NROCUPACAOM = OH.NROCUPACAOM
                                                        AND GPE_OCUPACAOH.NRORG = OH.NRORG)
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  /*Horário*/
  LEFT JOIN GPE_ALTEESCALA AE ON AE.NRORG = VH.NRORG
                             AND AE.NRVINCULOM = VH.NRVINCULOM
                             AND (AE.DTFIMESCALA IS NULL OR AE.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC) OR VM.DTRESCISAOVINC IS NOT NULL)
                             AND AE.DTINIESCALA = (SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                                                    WHERE GPE_ALTEESCALA.DTINIESCALA <= LAST_DAY(:P_DTMESCOMPETENC)
                                                      AND (GPE_ALTEESCALA.DTFIMESCALA IS NULL OR GPE_ALTEESCALA.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC) OR VM.DTRESCISAOVINC IS NOT NULL)
                                                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                                                      AND GPE_ALTEESCALA.NRORG = AE.NRORG)
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = AE.NRESCALATRABM
                              AND EH.NRORG = AE.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= LAST_DAY(:P_DTMESCOMPETENC)
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)
  LEFT JOIN PTO_TIPOJORNADA TJ ON TJ.NRTPJORNADA = EH.NRTPJORNADA
                              AND (TJ.NRORG = EH.NRORG OR TJ.NRORG = 0)
  /*Afastamento - Situacao Funcional*/
  LEFT JOIN (SELECT SF.NRORG, SF.NRVINCULOM, SF.DTINISITUFUNC, SF.DTFIMSITUFUNC, SF.NRMOTIVOAFASTA
               FROM GPE_ALTESITUFUNC SF
               JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                                     AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                           WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                             AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                             AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
              WHERE SF.NRORG = :P_NRORG
                AND SF.NRVINCULOM = :P_NRCHAVE
                AND ((:P_DTENVIOORIG BETWEEN SF.DTINISITUFUNC AND NVL(SF.DTFIMSITUFUNC,:P_DTENVIOORIG)) OR :P_DTENVIOORIG IS NULL) /*P_DTENVIOORIG para casos de retificacao*/
                AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
                     OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
                AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
             ) SF ON SF.NRVINCULOM = VH.NRVINCULOM
                 AND SF.NRORG = VH.NRORG
                 AND SF.DTINISITUFUNC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (SF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR SF.DTFIMSITUFUNC IS NULL)
  /*Motivo Afastamento*/
  LEFT JOIN GPE_MOTIVOAFASTA MA ON MA.NRMOTIVOAFASTA = SF.NRMOTIVOAFASTA
                               AND (MA.NRORG = SF.NRORG OR MA.NRORG = 0)
  /*Afastamento - Ferias*/
  LEFT JOIN (SELECT FE.NRORG, FE.NRVINCULOM, GO.DTINIGOZOFERIAS, GO.DTFIMGOZOFERIAS
               FROM FPA_FERIAS FE
               JOIN FPA_GOZOFERIAS GO ON GO.NRORG = FE.NRORG
                                     AND GO.NRFERIAS = FE.NRFERIAS
              WHERE FE.NRORG = :P_NRORG
                AND FE.NRVINCULOM = :P_NRCHAVE
                AND ((:P_DTENVIOORIG BETWEEN GO.DTINIGOZOFERIAS AND NVL(GO.DTFIMGOZOFERIAS,:P_DTENVIOORIG) ) OR :P_DTENVIOORIG IS NULL) /*P_DTENVIOORIG para casos de retificacao*/
             ) GO ON GO.NRORG = VH.NRORG
                 AND GO.NRVINCULOM = VH.NRVINCULOM
                 AND GO.DTINIGOZOFERIAS < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (GO.DTFIMGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR GO.DTFIMGOZOFERIAS IS NULL)
  /*Sindicato*/
  LEFT JOIN ESTRUTURAH ES ON ES.NRESTRUTURAM = VH.NRESTRUTSIND
                         AND ES.NRORG = VH.NRORG
                         AND ES.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                   WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                     AND ESTRUTURAH.NRESTRUTURAM = ES.NRESTRUTURAM
                                                     AND ESTRUTURAH.NRORG = ES.NRORG)
  /*RaçaCor*/
  LEFT JOIN GPE_RACAPESSOA RP ON RP.NRRACAPESSOA = PH.NRRACAPESSOA
                             AND (RP.NRORG = PH.NRORG OR RP.NRORG = 0)
  /*Estado Civil*/
  LEFT JOIN ESTADOCIVIL EC ON EC.CDESTACIVIL = PH.CDESTACIVIL
                          AND (EC.NRORG = PH.NRORG OR EC.NRORG = 0)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Logradouro*/
  LEFT JOIN LOGRADOURO LG ON LG.CDLOGRADOURO = EP.CDLOGRADOURO
                         AND (LG.NRORG = EP.NRORG OR LG.NRORG = 0)
  /*Parametro do eSocial*/
  LEFT JOIN ESO_PARAMETRO PARAM ON PARAM.NRORG = :P_NRORG
                            AND PARAM.NRESTRUTURAM = :P_NRESTRUTURAM
                            AND PARAM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
                            AND (PARAM.DTFIMVIGENCIA IS NULL OR PARAM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.NRTIPOVINCULOM = 1 --CLT
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTIPOESTRUTLEGAL" => $this->hcmParameters->getTipoEstruturaLegalOrg(),
            "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
            "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
            "P_DTENVIOORIG" => $dataEnvioOrig,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

    /**
     * @param $nrpessoa
     * @return array    Associative array
     */
    public function getDadosS2200dependente($nrorg, $nrparcnegocio, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2200dependente*/
SELECT LPAD(T.CDESOCIAL,2,'0') tpDep,
       H.NMPESSOA nmDep,
       TO_CHAR(H.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       REPLACE(REPLACE(H.NRCPFPESSOA, '.'), '-') cpfDep,
       DECODE((SELECT NVL(COUNT(*),0) FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '2'),0,'N','S') depIRRF,
       DECODE((SELECT NVL(COUNT(*),0) FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '1'),0,'N','S') depSF,
       DECODE(NVL(H.NRCONDFISPES,1),1,'N','S') incTrab
  FROM RELACIONAPARC R
  JOIN TIPORELACIONA T ON (T.NRORG = R.NRORG OR T.NRORG = 0)
                      AND T.NRTIPORELACIONA = R.NRTIPORELACIONA
  JOIN GPE_PESSOA P ON R.NRPARCNEGRELAC = P.NRPARCNEGOCIO
                   AND R.NRORG = P.NRORG
                   AND EXISTS (SELECT 1 FROM FPA_DEPVINCULO D
                                WHERE D.NRPESSOA = P.NRPESSOA
                                  AND D.NRORG = P.NRORG
                                  AND D.NRVINCULOM = :P_NRVINCULOM)
  JOIN GPE_PESSOAH H ON P.NRPESSOA = H.NRPESSOA
                    AND P.NRORG = H.NRORG
                    AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                             WHERE H1.NRPESSOA = P.NRPESSOA
                                               AND H1.NRORG = P.NRORG
                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
 WHERE R.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
   AND R.NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
            "P_NRVINCULOM" => $nrvinculom,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }

    /**
     * @param $nrpessoa
     * @return array    Associative array
     */
    public function getDadosS2200horario($nrorg, $nrescalatrabm, $nrturno) {
        $sql = <<<SQL
/*getDadosS2200horario*/
SELECT DISTINCT dia, codHorContrat
  FROM (SELECT CASE DECODE(HT.IDDIAVARIAVEL,'S','VARIAVEL',TO_CHAR(TU.DTDATABASE + (HT.NRSEQHORARIO-1), 'DAY'))
                    WHEN 'SEGUNDA-FEIRA' THEN 1
                    WHEN 'TERÇA-FEIRA  ' THEN 2
                    WHEN 'QUARTA-FEIRA ' THEN 3
                    WHEN 'QUINTA-FEIRA ' THEN 4
                    WHEN 'SEXTA-FEIRA  ' THEN 5
                    WHEN 'SÁBADO       ' THEN 6
                    WHEN 'DOMINGO      ' THEN 7
                    ELSE 8 END dia,
                   HT.NRHORDIAM codHorContrat
              FROM GPE_TURNO TU
              JOIN GPE_HORARIOTURNO HT ON HT.NRTURNO = TU.NRTURNO
                                      AND HT.NRORG = TU.NRORG
              JOIN GPE_HORDIAH HH ON HH.NRHORDIAM = HT.NRHORDIAM
                                 AND HH.NRORG = HT.NRORG
                                 AND HH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                                FROM GPE_HORDIAH
                                                               WHERE GPE_HORDIAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                 AND GPE_HORDIAH.NRHORDIAM = HH.NRHORDIAM
                                                                 AND GPE_HORDIAH.NRORG = HH.NRORG)
             WHERE TU.NRESCALATRABM = :P_NRESCALATRABM
               AND TU.NRTURNO = :P_NRTURNO
               AND TU.NRORG = :P_NRORG
               AND HH.IDTIPOHORARIO <> 'F'
             ORDER BY HT.NRSEQHORARIO)
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESCALATRABM" => $nrescalatrabm,
            "P_NRTURNO" => $nrturno,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2205($nrenvio, $nrchave, $nrestruturam, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2205*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideTrabalhador
       --alteracao
       REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS CPFTRAB,
       TO_CHAR(DECODE(PE.DTESOCIAL,TO_DATE(:P_DTHISTORICO),PE.DTESOCIAL,PH.DTULTATU), 'YYYY-MM-DD') AS DTALTERACAO,
       --dadosTrabalhador
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       PH.NMPESSOA AS NMTRAB,
       PH.IDSEXOPESSOA AS SEXO,
       RP.CDESOCIAL AS RACACOR,
       EC.CDESOCIAL AS ESTCIV,
       GI.CDESOCIAL AS GRAUINSTR,
       '' AS NMSOC,
       --nascimento
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       DECODE(PH.CDPAIS,'0055',MU.CDMUNICIBGE,'') codMunic,
       DECODE(PH.CDPAIS,'0055',PH.SGESTADO,'') uf,
       PA.CDESOCIAL paisNascto,
       PB.CDESOCIAL paisNac,
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 1) nmMae, -- 1=Mae
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 2) nmPai, -- 2=Pai
       --documentos
       --CTPS
       PH.NRCTPSPESSOA AS nrCtps,
       SUBSTR(PH.NRSERIECTPSPES,1,5) serieCtps,
       PH.SGUFCTPSPES AS ufCtps,
       --RIC
       '' AS NRRIC,
       '' AS ORGAOEMISSOR,
       '' AS DTEXPED,
       --RG
       PH.NRRGPESSOA nrRg,
       PH.CDEXRGPESSOA rg_orgaoEmissor,
       TO_CHAR(PH.DTEXRGPESSOA,'YYYY-MM-DD') rg_dtExped,
       --RNE
       PH.NRDOCUESTRANG nrRne,
       PH.DSEXPDOCESTR rne_orgaoEmissor,
       '' rne_dtExped,
       --OC
        '' AS NROC,
        '' AS OC_ORGAOEMISSOR,
        '' AS OC_DTEXPED,
        '' AS OC_DTVALID,
        --CNH
        --So traz dados CNH se tiver UF da CNH (mobile)
        DECODE(PH.SGUFHABCARPES, NULL, '', PH.NRCARTHABPES) nrRegCnh,
        DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTHABCARTPES,'YYYY-MM-DD')) cnh_dtExped,
        PH.SGUFHABCARPES ufCnh,
        DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTVALHABCARTPES,'YYYY-MM-DD')) cnh_dtValid,
        '' dtPriHab,
        DECODE(PH.SGUFHABCARPES, NULL, '', PH.DSCATEGHABCART) categoriaCnh,
        --endereco
        --brasil
        NVL(LG.CDESOCIAL,'O') AS TPLOGRAD,        
        EP.DSENDERECO AS DSCLOGRAD,
        NVL(EP.NRIMOVELENDERECO, '') AS NRLOGRAD,
        SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) AS COMPLEMENTO,
        EP.NMBAIRROENDERECO AS BAIRRO,
        REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') AS CEP,
        MP.CDMUNICIBGE AS BRASIL_CODMUNIC,
        EP.SGESTADO AS BRASIL_UF,
        --exterior
        PC.CDESOCIAL AS PAISRESID,
        EE.DSENDERECO AS EXTERIOR_DSCLOGRAD,
        NVL(EE.NRIMOVELENDERECO, '') AS EXTERIOR_NRLOGRAD,
        SUBSTR(EE.DSCOMPLEENDERECO, 0, 30) AS EXTERIOR_COMPLEMENTO,
        EE.NMBAIRROENDERECO AS EXTERIOR_BAIRRO,
        ME.NMMUNICIPIO AS NMCID,
        REPLACE(REPLACE(REPLACE(EE.NRCEPENDERECO,' '),'.'),'-') AS CODPOSTAL,
        --trabEstrangeiro
        TO_CHAR(PH.DTCHEGADAPAIS,'YYYY-MM-DD') AS DTCHEGADA,
        NVL(CT.CDESOCIAL,DECODE(PH.IDPOSVISTOPERM,'S','1','')) AS classTrabEstrang,
        CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTHISTORICO
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA = 4 -- 4=Cônjuge
               AND H.CDPAIS = '0055'),'N') END CASADOBR,
        CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTHISTORICO
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA = 3 -- 3=Filho
               AND H.CDPAIS = '0055'),'N') END FILHOSBR,
        --infoDeficiencia
        DECODE(PH.NRCONDFISPES,'2','S','N') AS DEFFISICA,        
        DECODE(PH.NRCONDFISPES,'4','S','N') AS DEFVISUAL,
        DECODE(PH.NRCONDFISPES,'3','S','N') AS DEFAUDITIVA,
        DECODE(PH.NRCONDFISPES,'5','S','N') AS DEFMENTAL,
        DECODE(PH.NRCONDFISPES,'8','S','N') AS DEFINTELECTUAL,
        DECODE(PH.NRCONDFISPES,'7','S','N') AS REABREADAP,        
        DECODE(VM.NRTIPOVINCULOM, 1, NVL(PH.IDCOTAPCD, 'N'), '') AS INFOCOTA,
        '' AS OBSERVACAO,
        --<<dependentes>>-- getDadosS2200dependente
        --aposentadoria
        '' trabAposent, --Facultativo (até 31/12/2017 se {codCateg} = [104]). Não obrigatorio (nas demais situações).
        --contato
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '01') AS FONEPRINC, -- 1-Telefone principal           
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU IN ('02','03','04','06')) AS FONEALTERNAT, -- 2-Celular particular 3-Telefone residencial 4-Telefone comercial 6-Celular comercial           
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '05') AS EMAILPRINC, -- 5-email
       '' AS EMAILALTERNAT,
       VM.NRORG, VM.NRVINCULOM, PN.NRPARCNEGOCIO
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*RaçaCor*/
  LEFT JOIN GPE_RACAPESSOA RP ON RP.NRRACAPESSOA = PH.NRRACAPESSOA
                             AND (RP.NRORG = PH.NRORG OR RP.NRORG = 0)
  /*Estado Civil*/
  LEFT JOIN ESTADOCIVIL EC ON EC.CDESTACIVIL = PH.CDESTACIVIL
                          AND (EC.NRORG = PH.NRORG OR EC.NRORG = 0)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Dados do Empregado - Naturalidade*/
  LEFT JOIN PAIS PA ON PA.CDPAIS = PH.CDPAIS
  LEFT JOIN MUNICIPIO MU ON MU.CDPAIS = PH.CDPAIS
                        AND MU.SGESTADO = PH.SGESTADO
                        AND MU.CDMUNICIPIO = PH.CDMUNICIPIO
  LEFT JOIN GPE_NACIONALIDADE NA ON PH.NRNACIONALID = NA.NRNACIONALIDADE
                                AND (PH.NRORG = NA.NRORG OR NA.NRORG = 0)
  LEFT JOIN PAIS PB ON PB.CDPAIS = NA.CDPAIS
  /*Endereço Principal*/
  LEFT JOIN ENDERECOPARC EP ON EP.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
                           AND EP.NRORG = PN.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MP ON MP.CDPAIS = EP.CDPAIS
                        AND MP.SGESTADO = EP.SGESTADO
                        AND MP.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Endereço Exterior*/
  LEFT JOIN ENDERECOPARC EE ON EE.NRPARCNEGOCIO = PN.NRPARCNEGOCIO
                           AND EE.NRORG = PN.NRORG
                           AND EE.CDTIPOENDERECO = 'EXTERIOR'
  LEFT JOIN MUNICIPIO ME ON ME.CDPAIS = EE.CDPAIS
                        AND ME.SGESTADO = EE.SGESTADO
                        AND ME.CDMUNICIPIO = EE.CDMUNICIPIO
  LEFT JOIN PAIS PC ON PC.CDPAIS = EE.CDPAIS
  /*Logradouro*/
  LEFT JOIN LOGRADOURO LG ON LG.CDLOGRADOURO = EP.CDLOGRADOURO
                         AND (LG.NRORG = EP.NRORG OR LG.NRORG = 0) 
  /*Trabalhador Estrangeiro*/
  LEFT JOIN GPE_CLASSESTRANG CT ON CT.NRCLASSESTRANG = PH.NRCLASSESTRANG
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2206($nrenvio, $nrchave, $nrestruturam, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2206*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,       
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS MATRICULA,
       --altContratual
       TO_CHAR(DECODE(VM.DTESOCIAL,TO_DATE(:P_DTHISTORICO),VM.DTESOCIAL,CASE WHEN TRUNC(SL.DTALTESALARIO,'MONTH') = TRUNC(TO_DATE(:P_DTHISTORICO),'MONTH') THEN SL.DTALTESALARIO ELSE VH.DTULTATU END), 'YYYY-MM-DD') AS DTALTERACAO,
       '' AS DTEF,
       '' AS DSCALT,
       --vinculo
       DECODE(CE.CDESOCIAL,'410',1,VH.NRTPREGIMEPREV) AS TPREGPREV,
       --infoRegimeTrab
       --infoCeletista
       '1' AS TPREGJOR,
       CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END NATATIVIDADE,
       TO_CHAR(ES.DTBASESINDICAL, 'MM') AS DTBASE,
       REPLACE(REPLACE(ES.CDCNPJESTRUT, '.'), '/') AS CNPJSINDCATEGPROF,
       --trabTemp
       '' AS JUSTPRORR,
       --aprend
       '' AS APREND_TPINSC,
       '' AS APREND_NRINSC,
       --infoEstatutario
       '' AS TPPLANRP,
       --infoContrato
       VH.NRCARGO AS CODCARGO,
       VH.NRFUNCAO AS CODFUNCAO,
       CE.CDESOCIAL AS CODCATEG,
       '' AS CODCARREIRA,
       '' AS DTINGRCARR,
       --remuneracao
       ROUND(SL.VRSALARIO, 2) AS VRSALFX,
       DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') AS UNDSALFIXO,
       SUBSTR(SL.DSOBSERVACAO,1,90) AS DSCSALVAR,
       --duracao
       CASE WHEN VM.NRVINCULOEMPREG IN (10, 11, 12, 13) THEN '2' ELSE '1' END tpContr,
       CASE WHEN VM.NRVINCULOEMPREG IN (10, 11, 12, 13) THEN TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD') ELSE '' END dtTerm,
       --localTrabalho
       --localTrabGeral
       DECODE(CE.CDESOCIAL,'106',NULL,DECODE(PNEH.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')) AS LOCALTRABGERAL_TPINSC,
       DECODE(CE.CDESOCIAL,'106',NULL,PNEH.NRINSCRICAOPARC) AS LOCALTRABGERAL_NRINSC,
       DECODE(CE.CDESOCIAL,'106',NULL,PNEH.NMPRINCIPALPARC) AS DESCCOMP,
       --localTrabDom
       DECODE(CE.CDESOCIAL,'106',NVL(LGD.CDESOCIAL,'O'),NULL) localTrabDom_TpLograd,
       DECODE(CE.CDESOCIAL,'106',EPD.DSENDERECO,NULL) localTrabDom_DscLograd,
       DECODE(CE.CDESOCIAL,'106',EPD.NRIMOVELENDERECO,NULL) localTrabDom_NrLograd,
       DECODE(CE.CDESOCIAL,'106',SUBSTR(EPD.DSCOMPLEENDERECO, 0, 30),NULL) localTrabDom_Complemento,
       DECODE(CE.CDESOCIAL,'106',EPD.NMBAIRROENDERECO,NULL) localTrabDom_Bairro,
       DECODE(CE.CDESOCIAL,'106',REPLACE(REPLACE(REPLACE(EPD.NRCEPENDERECO,' '),'.'),'-'),NULL) localTrabDom_Cep,
       DECODE(CE.CDESOCIAL,'106',MPD.CDMUNICIBGE,NULL) localTrabDom_CodMunic,
       DECODE(CE.CDESOCIAL,'106',EPD.SGESTADO,NULL) localTrabDom_Uf,
       --horContratual
       EH.QTHRSEMESCTRABH AS QTDHRSSEM,
       TJ.CDESOCIAL AS TPJORNADA,
       DECODE(TJ.CDESOCIAL,'9',EH.NMESCALATRABH,'') AS DSCTPJORN,
       '0' AS TMPPARC,
       --<<horario>>-- getDadosS2200horario
       --filiacaoSindical
       REPLACE(REPLACE(ES.CDCNPJESTRUT, '.'), '/') AS CNPJSINDTRAB,
       --alvaraJudicial
       '' AS NRPROCJUD,
       --observacoes
       SL.DSOBSERVACAO AS OBSERVACAO,
       --servPubl
       '' AS MTVALTER,
       VM.NRORG, EH.NRESCALATRABM, AE.NRTURNO
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  JOIN GPE_VINCULOEMPREG VE 
    ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE 
    ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG        
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ETH ON ETH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                    AND ETH.NRORG = VH.NRORG
                    AND ETH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ETH.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ETH.NRORG)
  JOIN PARCNEGOCIO PNEH ON PNEH.NRPARCNEGOCIO = ETH.NRPARCNEGOCIO
                     AND PNEH.NRORG = ETH.NRORG
  /*Tomador de Serviços*/
  LEFT JOIN TIPOHIERARQORG TH ON (TH.NRORG = VM.NRORG OR TH.NRORG = 0)
                             AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                             AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
  LEFT JOIN HIERARQUIAORG HO ON HO.NRORG = VM.NRORG
                            AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
                            AND HO.NRESTRUTURAMREL = VH.NRESTRUTGEREN
                            AND HO.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA)
                                                      FROM HIERARQUIAORG HI
                                                     WHERE HI.NRORG = HO.NRORG
                                                       AND HI.NRESTRUTURAMREL = HO.NRESTRUTURAMREL
                                                       AND HI.NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG
                                                       AND HI.DTINIVIGENCIA <= :P_DTMESCOMPETENC)
                            AND (HO.DTFIMVIGENCIA >= LAST_DAY(:P_DTMESCOMPETENC) OR HO.DTFIMVIGENCIA IS NULL)
  LEFT JOIN ESTRUTURAM EMT ON EMT.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                          AND EMT.NRORG = HO.NRORG
  LEFT JOIN ESTRUTURAH EHT ON EHT.NRESTRUTURAM = EMT.NRESTRUTURAM
                          AND EHT.NRORG = EMT.NRORG
                          AND EHT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND ESTRUTURAH.NRESTRUTURAM = EHT.NRESTRUTURAM
                                                       AND ESTRUTURAH.NRORG = EHT.NRORG)
  LEFT JOIN PARCNEGOCIO PNT ON PNT.NRPARCNEGOCIO = EHT.NRPARCNEGOCIO
                           AND PNT.NRORG = EHT.NRORG
  /*Endereço Trabalho Domestico*/
  LEFT JOIN ENDERECOPARC EPD ON EPD.NRPARCNEGOCIO = NVL(PNT.NRPARCNEGOCIO,PNEH.NRPARCNEGOCIO)
                            AND EPD.NRORG = NVL(PNT.NRORG,PNEH.NRORG)
                            AND EPD.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MPD ON MPD.CDPAIS = EPD.CDPAIS
                         AND MPD.SGESTADO = EPD.SGESTADO
                         AND MPD.CDMUNICIPIO = EPD.CDMUNICIPIO
  LEFT JOIN LOGRADOURO LGD ON LGD.CDLOGRADOURO = EPD.CDLOGRADOURO
                         AND (LGD.NRORG = EPD.NRORG OR LGD.NRORG = 0)
  /*Horário*/
  LEFT JOIN GPE_ALTEESCALA AE ON AE.NRORG = VH.NRORG
                             AND AE.NRVINCULOM = VH.NRVINCULOM
                             AND (AE.DTFIMESCALA IS NULL OR AE.DTFIMESCALA > :P_DTMESCOMPETENC)
                             AND AE.DTINIESCALA = (SELECT MAX(DTINIESCALA) FROM GPE_ALTEESCALA
                                                    WHERE GPE_ALTEESCALA.DTINIESCALA <= LAST_DAY(:P_DTMESCOMPETENC)
                                                      AND (GPE_ALTEESCALA.DTFIMESCALA IS NULL OR GPE_ALTEESCALA.DTFIMESCALA > LAST_DAY(:P_DTMESCOMPETENC))
                                                      AND GPE_ALTEESCALA.NRVINCULOM = AE.NRVINCULOM
                                                      AND GPE_ALTEESCALA.NRORG = AE.NRORG)
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = AE.NRESCALATRABM
                              AND EH.NRORG = AE.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)
  LEFT JOIN PTO_TIPOJORNADA TJ ON TJ.NRTPJORNADA = EH.NRTPJORNADA
                              AND (TJ.NRORG = EH.NRORG OR TJ.NRORG = 0)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Sindicato*/
  LEFT JOIN ESTRUTURAH ES ON ES.NRESTRUTURAM = VH.NRESTRUTSIND
                         AND ES.NRORG = VH.NRORG
                         AND ES.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                   WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                     AND ESTRUTURAH.NRESTRUTURAM = ES.NRESTRUTURAM
                                                     AND ESTRUTURAH.NRORG = ES.NRORG)                        
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
            "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2210($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2210*/
SELECT VI.NRVITIMA CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --idRegistrador
       5 AS tpRegistrador,
       DECODE(PNV.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
           CASE WHEN PNV.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PNV.NRINSCRICAOPARC ELSE SUBSTR(PNV.NRINSCRICAOPARC,1,8) END AS nrInsc,  
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') ideEmpregador_tpInsc,
           CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END AS ideEmpregador_nrInsc,           
       --ideTrabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       --cat
       TO_CHAR(OC.DTHROCORRENCIA, 'YYYY-MM-DD') AS dtAcid,       
       TA.CDESOCIAL AS tpAcid,
       TO_CHAR(OC.DTHROCORRENCIA, 'HH24MI') AS hrAcid,
       VI.QTHRATEINICATEND AS hrsTrabAntesAcid,
       VI.IDTIPOCAT AS tpCat,
       DECODE(VI.DTOBITO, NULL, 'N', 'S') AS indCatObito,
       TO_CHAR(VI.DTOBITO, 'YYYY-MM-DD') AS dtObito,
       VI.IDREGPOLICIAL AS indComunPolicia,
       SITG.CDESOCIAL AS codSitGeradora,
       1 AS iniciatCAT,
       OC.DSOBSERVACAO AS observacao,
       OC.DSOBSERVACAO AS obsCAT,
       --localAcidente
       TLC.CDESOCIAL AS tpLocal,
       VI.DSLOCALCAT AS dscLocal,
       EP.CDLOGRADOURO AS tpLograd,
       EP.DSENDERECO AS dscLograd,
       EP.NRIMOVELENDERECO AS nrLograd,
       SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) AS complemento,
       EP.NMBAIRROENDERECO AS bairro,
       EP.NRCEPENDERECO AS cep,
       MU.CDMUNICIBGE AS codMunic,
       EP.SGESTADO AS uf,
       REPLACE(REPLACE(REPLACE(ECAT.CDCNPJESTRUT,'.'),'/'),'-') AS cnpjLocalAcid,
       PA.CDESOCIAL AS pais,
       REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') AS codPostal,
       --ideLocalAcid
       DECODE(LENGTH(REPLACE(REPLACE(REPLACE(ECAT.CDCNPJESTRUT,'.'),'/'),'-')),14,1,11,2,'') AS ideLocalAcid_tpInsc,
       REPLACE(REPLACE(REPLACE(ECAT.CDCNPJESTRUT,'.'),'/'),'-') AS ideLocalAcid_nrInsc,
       --getDadosS2210parteAtingida
       --getDadosS2210agenteCausador       
       --atestado
       DECODE(FM.NRFICHAMEDICA,NULL,'',EHFM.CDFEDERALESTRUT) AS codCNES,
       DECODE(FM.NRFICHAMEDICA,NULL,'',VI.DTHRINICATEND) AS dtAtendimento,
       DECODE(FM.NRFICHAMEDICA,NULL,'',VI.QTHRATENDIMENTO) AS hrAtendimento,
       DECODE(FM.NRFICHAMEDICA,NULL,'',VI.IDINTERNACAO) AS indInternacao,
       DECODE(FM.NRFICHAMEDICA,NULL,'',FM.DTHRINIAFASTAMENTO - DTHRFIMAFASTAMENTO) durTrat,
       DECODE(FM.NRFICHAMEDICA,NULL,'',VI.IDAFASTAMENTO) AS indAfast,  
       DECODE(FM.NRFICHAMEDICA,NULL,'',NL.CDESOCIAL) AS dscLesao,
       DECODE(FM.NRFICHAMEDICA,NULL,'',NL.DSNATUREZALESAO) AS dscCompLesao,
       NULL diagProvavel,
       DECODE(FM.NRFICHAMEDICA,NULL,'',CID.CDCID) AS codCID,
       DECODE(FM.NRFICHAMEDICA,NULL,'',VI.DSOBSERVACAO) AS atestado_observacao,
       --emitente
       DECODE(FM.NRFICHAMEDICA,NULL,'',PHFM.NMPESSOA) AS nmEmit,
       DECODE(FM.NRFICHAMEDICA,NULL,'',DECODE(PHFM.NRCONSELHOREG,3,'1',9,'2',0)) ideOC,
       DECODE(FM.NRFICHAMEDICA,NULL,'',PHFM.NRINSCRCONSREG) AS nrOc,
       DECODE(FM.NRFICHAMEDICA,NULL,'',PHFM.SGUFCONSREG) AS ufOC,
       --catOrigem
       VIANT.DTCAT as dtCatOrig,
       VIANT.NRCAT AS nrCatOrig,
       VIANT.NRCAT AS nrRecCatOrig,
       VI.NRORG, VI.NRVITIMA
  FROM SST_VITIMA VI
  /*Comunicação de Acidente de Trabalho.*/
  JOIN SST_OCORRENCIA OC ON OC.NRORG = VI.NRORG
                        AND OC.NROCORRENCIA = VI.NROCORRENCIA
  JOIN SST_TIPOOCORRE TOC ON TOC.NRORG = OC.NRORG
                         AND TOC.NRTIPOOCORRE = OC.NRTIPOOCORRE  
  LEFT JOIN SST_SITUGERADORA SITG ON (SITG.NRORG = TOC.NRORG OR SITG.NRORG = 0)
                                 AND SITG.NRSITUGERADORA = TOC.NRSITUGERADORA
  LEFT JOIN SST_TIPOLOCALCAT TLC ON (TLC.NRORG = VI.NRORG OR TLC.NRORG = 0)
                                AND TLC.NRTIPOLOCALCAT = VI.NRTIPOLOCALCAT
  LEFT JOIN SST_NATUREZA N ON (N.NRORG = VI.NRORG OR N.NRORG = 0)
                          AND N.NRNATUREZA = VI.NRNATUREZA
  LEFT JOIN ESO_NATUREZALESAO NL ON (NL.NRORG = N.NRORG OR NL.NRORG = 0)
                                AND NL.NRNATUREZALESAO = N.NRNATUREZALESAO
  LEFT JOIN SST_CLASSIFICACAO CL ON CL.NRORG = OC.NRORG 
                                AND CL.NRCLASSIFICACAO = OC.NRCLASSIFICACAO
  LEFT JOIN SST_TIPOACIDENTE TA ON (TA.NRORG = CL.NRORG OR TA.NRORG = 0) 
                               AND TA.NRTIPOACIDENTE = CL.NRTIPOACIDENTE
  /*CAT ORIGEM*/
  LEFT JOIN SST_VITIMA VIANT ON VI.NRORG = VIANT.NRORG
                            AND VI.NRCATORIGEM = VIANT.NRCAT  
  /*LOCAL CAT */
  JOIN ESTRUTURAH ECAT ON ECAT.NRORG = OC.NRORG
                      AND ECAT.NRESTRUTURAM = OC.NRESTRUTURAM
                      AND ECAT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                  WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                    AND NRESTRUTURAM = ECAT.NRESTRUTURAM
                                                    AND NRORG = ECAT.NRORG)
  JOIN PARCNEGOCIO PNCAT ON PNCAT.NRPARCNEGOCIO = ECAT.NRPARCRESPINF
                        AND PNCAT.NRORG = ECAT.NRORG
  LEFT JOIN ENDERECOPARC EP ON PNCAT.NRPARCNEGOCIO = EP.NRPARCNEGOCIO
                           AND PNCAT.NRORG = EP.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN PAIS PA ON PA.CDPAIS = EP.CDPAIS
  LEFT JOIN MUNICIPIO MU ON MU.CDPAIS = EP.CDPAIS
                        AND MU.SGESTADO = EP.SGESTADO
                        AND MU.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Dados do Empregado*/
  JOIN PARCNEGOCIO PNV ON PNV.NRPARCNEGOCIO = VI.NRPARCNEGOCIO
                      AND PNV.NRORG = VI.NRORG   
  JOIN GPE_PESSOA PE ON PNV.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                    AND PNV.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  JOIN GPE_VINCULOM VM ON VM.NRORG = PE.NRORG
                      AND VM.NRPESSOA = PE.NRPESSOA
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  JOIN GPE_TIPOVINCULOH VT ON VT.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                          AND (VT.NRORG = VM.NRORG OR VT.NRORG = 0 OR VT.NRORG IS NULL)
                          AND VT.IDVINCULOEMPR = 'S'
                          AND VT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_TIPOVINCULOH
                                                    WHERE GPE_TIPOVINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND GPE_TIPOVINCULOH.NRTIPOVINCULOM = VT.NRTIPOVINCULOM
                                                      AND GPE_TIPOVINCULOH.NRORG = VT.NRORG)
  /*Ficha Medica*/
  LEFT JOIN SST_FICHAMEDICA FM ON FM.NRORG = VM.NRORG
                              AND FM.NRVINCULOM = VM.NRVINCULOM
                              AND FM.NRCAT = VI.NRCAT
  LEFT JOIN SST_DOENCAFICHAMEDICA DFM ON DFM.NRORG = FM.NRORG
                                     AND DFM.NRFICHAMEDICA = FM.NRFICHAMEDICA
  LEFT JOIN SST_CID CID ON CID.NRCID = DFM.NRCID 
  /*Responsavel Ficha Medica*/
  LEFT JOIN ESTRUTURAH EHFM ON EHFM.NRORG = FM.NRORG
                           AND EHFM.NRESTRUTURAM = FM.NRESTRUTURAM
                           AND EHFM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                       WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                         AND NRESTRUTURAM = EHFM.NRESTRUTURAM
                                                         AND NRORG = EHFM.NRORG)
  LEFT JOIN GPE_PESSOA PFM ON PFM.NRPARCNEGOCIO = FM.NRPARCNEGOCIO
                          AND PFM.NRORG = FM.NRORG
  LEFT JOIN GPE_PESSOAH PHFM ON PFM.NRPESSOA = PHFM.NRPESSOA
                            AND PFM.NRORG    = PHFM.NRORG
                            AND PHFM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                        WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_PESSOAH.NRPESSOA = PHFM.NRPESSOA
                                                          AND GPE_PESSOAH.NRORG = PHFM.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VI.NRORG = :P_NRORG
   AND VI.NRVITIMA = :P_NRCHAVE
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2210parteAtingida($nrorg, $nrvitima) {
        $sql = <<<SQL
/*getDadosS2210parteAtingida*/
SELECT PAE.CDESOCIAL AS codParteAting,
       VPA.LATERALIDADE AS lateralidade
  FROM SST_VITIMAPARTEATINGIDA VPA
  JOIN SST_PARTEATINGIDA PA ON (PA.NRORG = VPA.NRORG OR PA.NRORG = 0)
                           AND PA.NRPARTEATINGIDA = VPA.NRPARTEATINGIDA
  JOIN SST_PARTATINGESOC PAE ON PA.NRPARTATINGESOC = PAE.NRPARTATINGESOC
                            AND (PAE.NRORG = PA.NRORG OR PAE.NRORG = 0)
 WHERE VPA.NRVITIMA = :P_NRVITIMA
   AND VPA.NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRVITIMA" => $nrvitima
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2210agenteCausador($nrorg, $nrvitima) {
        $sql = <<<SQL
/*getDadosS2210agenteCausador*/
SELECT AC.CDESOCIAL codAgntCausador
  FROM SST_VITIMA VI
  LEFT JOIN SST_AGENTCAUSAD AC ON AC.NRAGENTCAUSAD = VI.NRAGENTCAUSAD
                              AND (AC.NRORG = VI.NRORG OR AC.NRORG = 0)
 WHERE VI.NRORG = :P_NRORG
   AND VI.NRVITIMA = :P_NRVITIMA
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRVITIMA" => $nrvitima,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2220($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2220*/
SELECT FM.NRFICHAMEDICA CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       CE.CDESOCIAL codCateg,
       --exMedOcup
       NE.CDESOCIAL AS tpExameOcup,
       --aso
       TO_CHAR(FM.DTHRPREVFICHAMED,'YYYY-MM-DD') AS dtAso,
       NE.CDESOCIAL AS tpAso,
       TRE.IDAPTO AS resAso,   
       --getDadosS2220exame
       --ideServSaude
       EHSSR.CODCNES AS codCNES,       
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = EMSSR.NRPARCNEGOCIO
           AND C.NRORG = EMSSR.NRORG
           AND C.CDFORMACOMU = '01') frmCtt, --Telefone principal,     
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = EMSSR.NRPARCNEGOCIO
           AND C.NRORG = EMSSR.NRORG
           AND C.CDFORMACOMU = '05') email, -- email      
       REPLACE(REPLACE(PHRM.NRCPFPESSOA,'.'),'-') cpfMed,
       REPLACE(REPLACE(PHRM.NRPISPASEPPES,'.'),'-') nisMed,
       PHRM.NMPESSOA AS nmMed,
       PHRM.NRINSCRCONSREG AS nrCRM,
       PHRM.SGUFCONSREG AS ufCRM,
       --respMonit
       REPLACE(REPLACE(PHRM.NRCPFPESSOA,'.'),'-') cpfResp,
       PHRM.NMPESSOA AS nmResp,
       PHRM.NRINSCRCONSREG AS respMonit_nrCRM,
       PHRM.SGUFCONSREG AS respMonit_ufCRM
  FROM SST_FICHAMEDICA FM 
  /*EXAME*/
  LEFT JOIN SST_NATUREZAEXAME NE ON (NE.NRORG = FM.NRORG OR NE.NRORG = 0) AND NE.NRNATUREZAEXAME = FM.NRNATUREZAEXAME
  LEFT JOIN SST_TABRESULTEXAME TRE ON (TRE.NRORG = FM.NRORG OR TRE.NRORG = 0) AND TRE.NRTABRESULTEXAME = FM.NRTABRESULTEXAME
  /*Serviço de Saude Responsavel*/
  LEFT JOIN ESTRUTURAM EMSSR ON EMSSR.NRORG = :P_NRORG AND EMSSR.NRESTRUTURAM = FM.NRESTRUTURAM
  LEFT JOIN ESTRUTURAH EHSSR ON EHSSR.NRORG = :P_NRORG
                     AND EHSSR.NRESTRUTURAM = FM.NRESTRUTURAM
                     AND EHSSR.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EHSSR.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EHSSR.NRORG)
  /*Medico Responsavel*/
  LEFT JOIN GPE_PESSOA PRM ON PRM.NRPARCNEGOCIO = FM.NRPARCNEGOCIO
                    AND PRM.NRORG    = FM.NRORG
  LEFT JOIN GPE_PESSOAH PHRM ON PRM.NRPESSOA = PHRM.NRPESSOA
                     AND PRM.NRORG    = PHRM.NRORG
                     AND PHRM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PHRM.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PHRM.NRORG)

  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON FM.NRVINCULOM = VM.NRVINCULOM AND VM.NRORG = FM.NRORG
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
 WHERE FM.NRORG = :P_NRORG
   AND FM.NRFICHAMEDICA = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2220exame($nrorg, $estrutGerencial, $nrfichamedica) {
        $sql = <<<SQL
/*getDadosS2220exame*/
SELECT TO_CHAR(EFM.DTREALIZACAOEXAME,'YYYY-MM-DD') AS dtExm,
       EFM.CDPROCTUSS AS procRealizado,
       EFM.OBSPROCTUSS AS obsProc,
       EFM.INTERPREXAME AS interprExm,
       DECODE(EFM.IDEXAMEREFSEQ, 'R', 1, 'S', 2) AS ordExame,
       TO_CHAR(EFM.DTREALIZACAOEXAME,'YYYY-MM-DD') AS dtIniMonit,
       NULL AS dtFimMonit,
       DECODE(TRE.IDRESULTEXAME, 'N', 1, 'A', 2,'E', 3, 'G', 4) indResult,
       --respMonit
       PARAMB.NRINSCRICAOPARC AS nisResp,       
       PHAMB.NRINSCRCONSREG AS nrOc,
       PHAMB.SGUFCONSREG AS ufOC   
  FROM SST_EXAMEFICHAMED EFM
  LEFT JOIN SST_TABRESULTEXAME TRE
    ON (TRE.NRORG = EFM.NRORG OR TRE.NRORG = 0)
   AND TRE.NRTABRESULTEXAME = EFM.NRTABRESULTEXAME
  JOIN SST_FICHAMEDICA FM ON FM.NRFICHAMEDICA = EFM.NRFICHAMEDICA AND FM.NRORG = EFM.NRORG
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTGEREN
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  /*Parceiro Biologico*/                                                 
  LEFT JOIN PARCNEGOCIO PARAMB ON PARAMB.NRPARCNEGOCIO = EMH.NRPARCMONBIO
                              AND PARAMB.NRORG = EMH.NRORG
  LEFT JOIN GPE_PESSOA PAMB ON PAMB.NRPARCNEGOCIO = PARAMB.NRPARCNEGOCIO
                           AND PAMB.NRORG    = FM.NRORG
  LEFT JOIN GPE_PESSOAH PHAMB
     ON PHAMB.NRPESSOA = PAMB.NRPESSOA
    AND PHAMB.NRORG    = FM.NRORG
    AND PHAMB.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                              WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                AND GPE_PESSOAH.NRPESSOA = PHAMB.NRPESSOA
                                AND GPE_PESSOAH.NRORG = PHAMB.NRORG)  
 
 

 WHERE EFM.NRORG = :P_NRORG
   AND EFM.NRFICHAMEDICA = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRCHAVE" => $nrfichamedica,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTGEREN" => $estrutGerencial
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2221($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2221*/
SELECT EFM.NREXAMEFICHAMED CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       CE.CDESOCIAL codCateg,
       --toxicologico
       TO_CHAR(NVL(EFM.DTREALIZACAOEXAME,FM.DTCONCLFICHAMED),'YYYY-MM-DD') AS dtExame,
       REPLACE(REPLACE(REPLACE(EHL.CDCNPJESTRUT,'.'),'/'),'-') AS cnpjLab,
       '' AS codSeqExame,
       PHRM.NMPESSOA AS nmMed,
       PHRM.NRINSCRCONSREG AS nrCRM,
       PHRM.SGUFCONSREG AS ufCRM,
       'N' AS indRecusa
  FROM SST_EXAMEFICHAMED EFM
  JOIN SST_FICHAMEDICA FM ON FM.NRORG = EFM.NRORG
                         AND FM.NRFICHAMEDICA = EFM.NRFICHAMEDICA
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON VM.NRORG = FM.NRORG
                      AND VM.NRVINCULOM = FM.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Dados do Empregado - Pessoa*/
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRORG = PE.NRORG
                     AND PH.NRPESSOA = PE.NRPESSOA
                     AND PH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                               WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND H1.NRPESSOA = PH.NRPESSOA
                                                 AND H1.NRORG = PH.NRORG)
  /*Laboratorio*/
  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = FM.NRORG
                          AND EHL.NRESTRUTURAM = FM.NRESTRUTURAM
                          AND EHL.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC) FROM ESTRUTURAH H1
                                                     WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND H1.NRESTRUTURAM = EHL.NRESTRUTURAM
                                                       AND H1.NRORG = EHL.NRORG)
  /*Medico Responsavel*/
  LEFT JOIN GPE_PESSOA PRM ON PRM.NRORG = FM.NRORG
                          AND PRM.NRPARCNEGOCIO = NVL(EFM.NRPARCNEGOCIO,FM.NRPARCNEGOCIO)
  LEFT JOIN GPE_PESSOAH PHRM ON PHRM.NRORG    = PRM.NRORG
                            AND PHRM.NRPESSOA = PRM.NRPESSOA
                            AND PHRM.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                        WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_PESSOAH.NRPESSOA = PHRM.NRPESSOA
                                                          AND GPE_PESSOAH.NRORG = PHRM.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE EFM.NRORG = :P_NRORG
   AND EFM.NREXAMEFICHAMED = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2230_Altesitufunc($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dthistorico) {
        $sql = <<<SQL
/*getDadosS2230_Altesitufunc*/
SELECT ASF.NRALTESITUFUNC CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN NULL ELSE NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) END AS MATRICULA,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN CE.CDESOCIAL ELSE '' END CODCATEG,
       --infoAfastamento
       --iniAfastamento
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC THEN TO_CHAR(ASF.DTINISITUFUNC, 'YYYY-MM-DD') ELSE '' END AS dtIniAfast,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC THEN MA.CDESOCIAL ELSE '' END AS codMotAfast,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC THEN DECODE(MAANT.CDESOCIAL, NULL,'N', 'S') ELSE '' END AS infoMesmoMtv,  
       NULL AS tpAcidTransito,
       CASE WHEN ASF.DTINISITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC THEN ASF.DSOBSALTSITFUNC ELSE '' END AS observacao,    
       -- infoAtestado getDadosS2230infoAtestado           
       --fimAfastamento
       CASE WHEN ASF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) AND ASF.DTFIMSITUFUNC <= LAST_DAY(:P_DTMESCOMPETENC) THEN TO_CHAR(ASF.DTFIMSITUFUNC, 'YYYY-MM-DD') ELSE '' END AS dtTermAfast,
       ASF.NRORG, ASF.NRALTESITUFUNC as NRCHAVE, 'AFASTAMENTO' as DSORIGEM
  FROM GPE_ALTESITUFUNC ASF
  JOIN GPE_VINCULOM VM ON VM.NRORG = ASF.NRORG
                      AND VM.NRVINCULOM = ASF.NRVINCULOM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG      = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = VH.NRESTRUTLEGAL
  
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)      
   /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC  
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRPESSOA = VM.NRPESSOA
                    AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG    = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                     AND PN.NRORG = PE.NRORG  
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Situação Funcional*/
  JOIN GPE_SITUFUNCM SF ON SF.NRSITUFUNCM = ASF.NRSITUFUNCM
  JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                        AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                              WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
  LEFT JOIN GPE_MOTIVOAFASTA MA ON MA.NRMOTIVOAFASTA = ASF.NRMOTIVOAFASTA
                               AND (MA.NRORG = ASF.NRORG OR MA.NRORG = 0)
  LEFT JOIN GPE_ALTESITUFUNC ASFANT ON ASFANT.NRVINCULOM = VM.NRVINCULOM
                                   AND ASFANT.NRORG = VM.NRORG
                                   AND ASFANT.DTFIMSITUFUNC BETWEEN ADD_MONTHS(ASF.DTINISITUFUNC, -2) AND ASF.DTINISITUFUNC
                                   AND ASFANT.DTFIMSITUFUNC = (SELECT MAX(ASFANT1.DTFIMSITUFUNC) FROM GPE_ALTESITUFUNC ASFANT1
                                                                WHERE ASFANT1.NRVINCULOM = ASFANT.NRVINCULOM
                                                                  AND ASFANT1.NRORG = ASFANT.NRORG
                                                                  AND ASFANT1.DTFIMSITUFUNC BETWEEN ADD_MONTHS(ASF.DTINISITUFUNC, -2) AND ASF.DTINISITUFUNC)
  LEFT JOIN GPE_SITUFUNCM SFANT ON SFANT.NRSITUFUNCM = ASFANT.NRSITUFUNCM
  LEFT JOIN GPE_SITUFUNCH SFHANT ON SFHANT.NRSITUFUNCM = SFANT.NRSITUFUNCM
                                AND SFHANT.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                         WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                           AND GPE_SITUFUNCH.NRSITUFUNCM = SFHANT.NRSITUFUNCM
                                                           AND GPE_SITUFUNCH.NRORG = SFHANT.NRORG)
                                AND (SFHANT.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
                                    OR SFANT.NRSITUFUNCM = 12 OR SFANT.NRSITUFUNCM = 100012 OR SFANT.NRSITUFUNCM = 15 OR SFANT.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
                                AND SFANT.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
  LEFT JOIN GPE_MOTIVOAFASTA MAANT ON MAANT.NRMOTIVOAFASTA = ASFANT.NRMOTIVOAFASTA
                                  AND (MAANT.NRORG = ASFANT.NRORG OR MAANT.NRORG = 0)
                                  AND (MAANT.CDESOCIAL = '01' OR MAANT.CDESOCIAL = '03')
                                  AND MAANT.CDESOCIAL = MA.CDESOCIAL
 WHERE ASF.NRORG = :P_NRORG
   AND ASF.NRALTESITUFUNC = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            //"P_NRESTRUTURAM" => $nrestruturam,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2230_GozoFerias($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dthistorico) {
        $sql = <<<SQL
/*getDadosS2230_GozoFerias*/
SELECT GF.NRGOZOFERIAS CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN NULL ELSE NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) END AS MATRICULA,
       CASE WHEN VM.NRTIPOVINCULOM IN (2,3,10) THEN CE.CDESOCIAL ELSE '' END CODCATEG,
       --infoAfastamento
       --iniAfastamento
       CASE WHEN GF.DTINIGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN TO_CHAR(GF.DTINIGOZOFERIAS, 'YYYY-MM-DD') ELSE NULL END AS dtIniAfast,
       CASE WHEN GF.DTINIGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN '15' ELSE '' END AS codMotAfast, /*Gozo de férias ou recesso - Afastamento temporário para o gozo de férias ou recesso*/
       '' AS infoMesmoMtv,  
       '' AS tpAcidTransito,
       '' AS observacao,
       -- infoAtestado getDadosS2230infoAtestado           
       --fimAfastamento
       TO_CHAR(GF.DTFIMGOZOFERIAS, 'YYYY-MM-DD') AS dtTermAfast,
       GF.NRORG, GF.NRGOZOFERIAS as NRCHAVE, 'FERIAS' as DSORIGEM
  FROM FPA_GOZOFERIAS GF
  JOIN FPA_FERIAS FE ON FE.NRORG = GF.NRORG
                    AND FE.NRFERIAS = GF.NRFERIAS
  JOIN GPE_VINCULOM VM ON VM.NRORG = FE.NRORG
                      AND VM.NRVINCULOM = FE.NRVINCULOM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG      = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = VH.NRESTRUTLEGAL
  
  JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC  
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRPESSOA = VM.NRPESSOA
                    AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG    = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                     AND PN.NRORG = PE.NRORG  
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE GF.NRORG = :P_NRORG
   AND GF.NRGOZOFERIAS = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            //"P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2230infoAtestado($nrorg, $nrAlteSituFunc) {
        $sql = <<<SQL
/*getDadosS2230infoAtestado*/
SELECT --infoAtestado
       ASF.CDDIAGNOST AS codCID ,
       NVL(ASF.NRQTDIASITUFUNC, (ASF.DTFIMSITUFUNC - ASF.DTINISITUFUNC + 1)) AS qtdDiasAfast,
       --emitente
       PMH.NMPESSOA AS nmEmit,
       (CASE WHEN PMH.NRCONSELHOREG = 3 THEN '1' WHEN PMH.NRCONSELHOREG = 9 THEN '2' WHEN PMH.NRCONSELHOREG IS NOT NULL AND PMH.NRCONSELHOREG <> 3 AND PMH.NRCONSELHOREG <> 9 THEN '0' ELSE '' END) AS ideOC,
       PMH.NRINSCRCONSREG AS nrOc,
       PMH.SGUFCONSREG AS ufOC
  FROM GPE_ALTESITUFUNC ASF 
  /*Dados Medico Atestado*/
  LEFT JOIN GPE_PESSOA PM
    ON PM.NRPESSOA = ASF.NRPESSOA
   AND PM.NRORG    = ASF.NRORG
  LEFT JOIN GPE_PESSOAH PMH
    ON PMH.NRPESSOA = PM.NRPESSOA
   AND PMH.NRORG    = PM.NRORG
   AND PMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                              WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                AND GPE_PESSOAH.NRPESSOA = PM.NRPESSOA
                                AND GPE_PESSOAH.NRORG = PM.NRORG)
 WHERE ASF.NRALTESITUFUNC = :P_NRCHAVE
   AND ASF.NRORG = :P_NRORG
   AND ASF.DTINISITUFUNC >= :P_DTMESCOMPETENC
   AND (ASF.CDDIAGNOST IS NOT NULL OR ASF.DTFIMSITUFUNC IS NOT NULL OR ASF.NRQTDIASITUFUNC IS NOT NULL)
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRCHAVE" => $nrAlteSituFunc,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia()
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2240($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2240*/
SELECT MO.NRMOVIMENTACAO CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS cpfTrab,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       CE.CDESOCIAL AS codCateg,
       --infoExpRisco
       --iniExpRisco
       MO.DTINIMOVIMENT AS dtIniCondicao,
       --getDadosS2240infoAmb
       --infoAtiv
       MO.DSOBSERVACAO AS dscAtivDes,
       --getDadosS2240ativPericInsal
       --getDadosS2240fatRisco
       --getDadosS2240epc
       --getDadosS2240epi
       --respReg
       EH.DTINIRESAMB AS dtIni,
       EH.DTFIMRESAMB AS dtFim,   
       PARAMB.NRINSCRICAOPARC AS nisResp,       
       PHAMB.NRINSCRCONSREG AS nrOc,
       PHAMB.SGUFCONSREG AS ufOC,
       MO.NRORG, MO.NRESTRUTURAM, MO.NRVINCULOM, VH.NRCARGO, VH.NRFUNCAO
  FROM GPE_MOVIMENTACAO MO    
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOM VM ON VM.NRORG = MO.NRORG
                      AND VM.NRVINCULOM = MO.NRVINCULOM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
   /*Categoria Trabalhador*/    
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG                           
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE ON PE.NRORG = VM.NRORG
                    AND PE.NRPESSOA = VM.NRPESSOA
  JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                     AND PH.NRORG = PE.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PE.NRORG)
  /*Dados do Empregado - Estrutura Movimentada*/
  JOIN ESTRUTURAH EH ON EH.NRORG = MO.NRORG
                    AND EH.NRESTRUTURAM = MO.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = EH.NRORG)                               
  JOIN PARCNEGOCIO PN ON PN.NRORG = PE.NRORG
                     AND PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
  /*Responsavel Ambiente*/ 
  LEFT JOIN PARCNEGOCIO PARAMB ON PARAMB.NRPARCNEGOCIO = EH.NRPARCREGAMB
                              AND PARAMB.NRORG = EH.NRORG
  LEFT JOIN GPE_PESSOA PAMB ON PAMB.NRPARCNEGOCIO = PARAMB.NRPARCNEGOCIO
                           AND PAMB.NRORG = VM.NRORG
  LEFT JOIN GPE_PESSOAH PHAMB ON PHAMB.NRPESSOA = PAMB.NRPESSOA
                             AND PHAMB.NRORG = PAMB.NRORG
                             AND PHAMB.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                          WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                            AND GPE_PESSOAH.NRPESSOA = PHAMB.NRPESSOA
                                                            AND GPE_PESSOAH.NRORG = PHAMB.NRORG)  
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG                 
  
 WHERE MO.NRORG = :P_NRORG
   AND MO.NRMOVIMENTACAO = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2240ativPericInsal($nrorg, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2240ativPericInsal*/
SELECT OM.NROCUPACAOM AS codAtiv,
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH
    ON VH.NRORG = VM.NRORG
   AND VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Dados do Empregado - Ocupação*/                              
  JOIN GPE_OCUPACAOM OM ON OM.NROCUPACAOM = VH.NRCARGO
                       AND OM.NRORG = VH.NRORG  
  JOIN GPE_OCUPACAOH OH ON OH.NRORG = OM.NRORG
                       AND OH.NROCUPACAOM = OM.NROCUPACAOM
                       AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                 WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                   AND GPE_OCUPACAOH.NROCUPACAOM = OH.NROCUPACAOM
                                                   AND GPE_OCUPACAOH.NRORG = OH.NRORG)    
  LEFT JOIN SST_RELACOCUPAPPP ROP ON ROP.NROCUPACAOM = OM.NROCUPACAOM
                                 AND OM.NRORG = ROP.NRORG                                
  LEFT JOIN SST_PERFILPPP PP ON PP.NRPERFILPPP = ROP.NRPERFILPPP
                            AND PP.NRORG = ROP.NRORG
 WHERE VM.NRORG = :P_NRORG 
   AND VM.NRVINCULOM = :P_NRVINCULOM
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRVINCULOM" => $nrvinculom,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2240infoAmb($nrorg, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2240infoAmb*/
SELECT EM.NRESTRUTURAM AS codAmb
  FROM ESTRUTURAM EM
  JOIN ESTRUTURAH EH ON EH.NRORG = :P_NRORG
                    AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = EH.NRORG)
  LEFT JOIN SST_RELACESTRUTPPP REP ON REP.NRESTRUTURAM = EM.NRESTRUTURAM
                                  AND REP.NRORG = EM.NRORG                                
  LEFT JOIN SST_PERFILPPP PP ON PP.NRPERFILPPP = REP.NRPERFILPPP
                            AND PP.NRORG = REP.NRORG
 WHERE EM.NRORG = :P_NRORG 
   AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRVINCULOM" => $nrvinculom,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia()
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2240fatRisco($nrorg, $nrestruturam, $nrcargo, $nrfuncao) {
        $sql = <<<SQL
/*getDadosS2240fatRisco*/
SELECT RA.CDESOCIAL AS codFatRis,
       '' AS tpAval,
       FR.DSINTENSIDADE AS intConc,
       '' AS limTol,
       '' AS unMed,
       FR.DSTECNICA AS tecMedicao,
       FR.IDGERAINSAL AS insalubridade,
       FR.IDGERAPERIC AS periculosidade,
       FR.IDGERAAPOSESPEC AS aposentEsp,
       --epcEpi
       DECODE(RC.NRRISCOEPC, NULL, 1, 2) utilizEPC,
       '' AS eficEpc,
       DECODE(RE.NRRISCOEPI, NULL, 1, 2) utilizEPI,
       FR.NRFATORRISCO
  FROM SST_RELESTRUTRISCO RER
  JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = RER.NRFATORRISCO
                        AND FR.NRORG = RER.NRORG
  JOIN GPE_FATRISCAMB RA ON RA.NRFATRISCAMB = FR.NRFATRISCAMB
                        AND (RA.NRORG = FR.NRORG OR RA.NRORG = 0)
  LEFT JOIN SST_RISCOEPI RE ON RE.NRFATORRISCO = FR.NRFATORRISCO
                           AND RE.NRORG = FR.NRORG 
  LEFT JOIN SST_RISCOEPC RC ON RC.NRFATORRISCO = FR.NRFATORRISCO
                           AND RC.NRORG = FR.NRORG
 WHERE RER.NRORG = :P_NRORG
   AND RER.NRESTRUTURAM = :P_NRESTRUTURAM
UNION ALL
SELECT RA.CDESOCIAL AS codFatRis,
       '' AS tpAval,
       FR.DSINTENSIDADE AS intConc,
       '' AS limTol,
       '' AS unMed,
       FR.DSTECNICA AS tecMedicao,
       FR.IDGERAINSAL AS insalubridade,
       FR.IDGERAPERIC AS periculosidade,
       FR.IDGERAAPOSESPEC AS aposentEsp,
       --epcEpi
       DECODE(RC.NRRISCOEPC, NULL, 1, 2) utilizEPC,
       '' AS eficEpc,
       DECODE(RE.NRRISCOEPI, NULL, 1, 2) utilizEPI,
       FR.NRFATORRISCO
  FROM SST_RELOCUPARISCO ROR
  JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = ROR.NRFATORRISCO
                        AND FR.NRORG = ROR.NRORG
  JOIN GPE_FATRISCAMB RA ON RA.NRFATRISCAMB = FR.NRFATRISCAMB
                        AND (RA.NRORG = FR.NRORG OR RA.NRORG = 0)
  LEFT JOIN SST_RISCOEPI RE ON RE.NRFATORRISCO = FR.NRFATORRISCO
                           AND RE.NRORG = FR.NRORG 
  LEFT JOIN SST_RISCOEPC RC ON RC.NRFATORRISCO = FR.NRFATORRISCO
                           AND RC.NRORG = FR.NRORG
 WHERE ROR.NRORG = :P_NRORG
   AND ROR.NROCUPACAOM = :P_NRCARGO

UNION ALL
SELECT RA.CDESOCIAL AS codFatRis,
       '' AS tpAval,
       FR.DSINTENSIDADE AS intConc,
       '' AS limTol,
       '' AS unMed,
       FR.DSTECNICA AS tecMedicao,
       FR.IDGERAINSAL AS insalubridade,
       FR.IDGERAPERIC AS periculosidade,
       FR.IDGERAAPOSESPEC AS aposentEsp,
       --epcEpi
       DECODE(RC.NRRISCOEPC, NULL, 1, 2) utilizEPC,
       '' AS eficEpc,
       DECODE(RE.NRRISCOEPI, NULL, 1, 2) utilizEPI,
       FR.NRFATORRISCO
  FROM SST_RELOCUPARISCO ROR
  JOIN SST_FATORRISCO FR ON FR.NRFATORRISCO = ROR.NRFATORRISCO
                        AND FR.NRORG = ROR.NRORG
  JOIN GPE_FATRISCAMB RA ON RA.NRFATRISCAMB = FR.NRFATRISCAMB
                        AND (RA.NRORG = FR.NRORG OR RA.NRORG = 0)
  LEFT JOIN SST_RISCOEPI RE ON RE.NRFATORRISCO = FR.NRFATORRISCO
                           AND RE.NRORG = FR.NRORG 
  LEFT JOIN SST_RISCOEPC RC ON RC.NRFATORRISCO = FR.NRFATORRISCO
                           AND RC.NRORG = FR.NRORG
 WHERE ROR.NRORG = :P_NRORG
   AND ROR.NROCUPACAOM = :P_NRFUNCAO
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRCARGO" => $nrcargo,
            "P_NRFUNCAO" => $nrfuncao,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2240epc($nrorg, $nrfatorrisco) {
        $sql = <<<SQL
/*getDadosS2240epc*/
SELECT NMEPC AS dscEpc,
       IDEPCEFICAZ AS eficEpc 
  FROM SST_RISCOEPC
 WHERE NRFATORRISCO = :P_NRFATORRISCO
   AND NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRFATORRISCO" => $nrfatorrisco
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2240epi($nrorg, $nrfatorrisco, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2240epi*/
SELECT E.CERTIAPROV AS caEPI,
       RE.IDEPIEFICAZ AS eficEpi,
       DECODE(E.NRTIPOMEDIDAEPI, NULL, 'N', 'S') AS medProtecao, 
       HE.IDCONDFUNCIO AS condFuncto, 
       HE.IDPRAZOVALID AS przValid, 
       HE.IDPERIODICTROCA AS periodicTroca, 
       /*HE.IDHIGIENIZACAO*/'' AS higienizacao
  FROM SST_RISCOEPI RE
  JOIN SST_EPI E ON E.NRORG  = RE.NRORG
                AND E.NREPI = RE.NREPI
  LEFT JOIN SST_HISTORICOEPI HE ON HE.NREPI  = E.NREPI
                               AND HE.NRORG = RE.NRORG
                               AND HE.NRVINCULOM = :P_NRVINCULOM
 WHERE RE.NRFATORRISCO = :P_NRFATORRISCO
   AND RE.NRORG          = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRFATORRISCO" => $nrfatorrisco,
            "P_NRVINCULOM" => $nrvinculom
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2241($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2241*/
SELECT MO.NRMOVIMENTACAO CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,  
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,       
       --insalPeric
       --iniInsalPeric 
       CASE
         WHEN FR.IDGERAPERIC = 'S' OR FR.IDGERAINSAL = 'S' THEN
           MO.DTINIMOVIMENT
        END AS dtIniCondicao,
       --aposentEsp       
       DECODE(FR.IDGERAAPOSESPEC, 'S',  MO.DTINIMOVIMENT) AS iniAposentEsp_dtIniCondicao
  FROM GPE_MOVIMENTACAO MO    
  JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = MO.NRVINCULOM AND MO.NRORG = VM.NRORG
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Estrutura Movimentada*/
JOIN ESTRUTURAH EH ON EH.NRORG = :P_NRORG
                     AND EH.NRESTRUTURAM = VH.NRESTRUTGEREN
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EH.NRORG)
JOIN SST_RELESTRUTRISCO RER ON RER.NRESTRUTURAM = EH.NRESTRUTURAM AND RER.NRORG = EH.NRORG 
JOIN SST_FATORRISCO FR
ON FR.NRFATORRISCO = RER.NRFATORRISCO
AND FR.NRORG       = RER.NRORG
AND (FR.IDGERAINSAL = 'S' OR FR.IDGERAPERIC = 'S' OR FR.IDGERAAPOSESPEC = 'S')                      
 WHERE VM.NRORG = :P_NRORG
   AND MO.NRMOVIMENTACAO = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2241infoAmb($nrorg, $nrvinculom, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2241infoAmb*/
SELECT EM.NRESTRUTURAM AS codAmb      
FROM ESTRUTURAM EM
JOIN ESTRUTURAH EH ON EH.NRORG = :P_NRORG
                     AND EH.NRESTRUTURAM = EM.NRESTRUTURAM
                     AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EH.NRORG)
WHERE EM.NRORG = :P_NRORG 
AND EM.NRESTRUTURAM = :P_NRESTRUTURAM
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia()
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2241fatRisco($nrorg, $nrestruturaMovimentada) {
        $sql = <<<SQL
/*getDadosS2241fatRisco*/
SELECT RA.CDESOCIAL AS codFatRis
FROM SST_RELESTRUTRISCO RER
JOIN SST_FATORRISCO FR
ON FR.NRFATORRISCO = RER.NRFATORRISCO
AND FR.NRORG       = RER.NRORG
JOIN GPE_FATRISCAMB RA
ON RA.NRFATRISCAMB     = FR.NRFATRISCAMB
AND (RA.NRORG          = FR.NRORG
OR RA.NRORG            = 0)
WHERE RER.NRESTRUTURAM = :P_NRESTRUTURAM
AND RER.NRORG          = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESTRUTURAM" => $nrestruturaMovimentada
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2250($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2250*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR ET.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR ET.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       --infoAvPrevio
       DECODE(VM.DTAVISOPREVIO, NULL, '', TO_CHAR(VM.DTAVISOPREVIO,'YYYY-MM-DD')) AS dtAvPrv,
       DECODE(VM.DTAVISOPREVIO, NULL, '', TO_CHAR(VM.DTRESCISAOVINC,'YYYY-MM-DD')) AS dtPrevDeslig,
       DECODE(VM.DTAVISOPREVIO, NULL, '', TAP.CDESOCIAL) AS tpAvPrevio,
       DECODE(VM.DTAVISOPREVIO, NULL, '', SUBSTR(VM.DSOBSRESCISAO,1,255)) AS observacao,
       --cancAvPrevio
       DECODE(VM.DTCANCAVIPRE, NULL, '', TO_CHAR(VM.DTCANCAVIPRE,'YYYY-MM-DD')) AS dtCancAvPrv,
       DECODE(VM.DTCANCAVIPRE, NULL, '', VM.DSOBSCANCAVIPRE) AS cancAvPrevio_observacao,
       DECODE(VM.DTCANCAVIPRE, NULL, '', VM.NRMOTIVOCANCAP) AS mtvCancAvPrevio
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = VH.NRESTRUTLEGAL
                    AND ET.NRORG = VH.NRORG
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                      AND PAR.NRORG = ET.NRORG
  /*Pessoa*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Tipo de Aviso Prévio*/
  LEFT JOIN GPE_TIPOAVISOPRE TAP ON TAP.NRTPAVIPRE = VM.NRTPAVIPRE
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2260($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2260*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS cpfTrab,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS MATRICULA,
       --infoConvInterm
       VM.CDMATRICULA AS CODCONV, -- verificar com vínculo empregáticio e pedir amália para criar
       TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD') AS dtInicio,
       TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD') dtFim,
       TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD') AS dtPrevPgto,
       --jornada
       '' AS CODHORCONTRAT,
       EH.NMESCALATRABH AS DSCJORNADA,
       --localTrab
       '0' AS indLocal,
       --localTrabInterm
       '' AS tpLograd,
       '' AS dscLograd,
       '' AS nrLograd,
       '' AS complem,
       '' AS bairro,
       '' AS cep,
       '' AS codMunic,
       '' AS uf
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG
  /*Horário*/
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = VH.NRESCALATRABM
                              AND EH.NRORG = VH.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)   
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
     /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2298($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosS2298*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula, 
       --infoReintegr
       VM.IDTPREINTEGRACAO AS tpReint,
       VM.NRPROCJUD AS nrProcJud,
       VM.NRLEIANISTIA AS nrLeiAnistia,       
       TO_CHAR(VM.DTRETORNO,'YYYY-MM-DD')  AS dtEfetRetorno,
       TO_CHAR(VM.DTEFEITOFINANCEIRO,'YYYY-MM-DD') AS dtEfeito,
       VM.IDPAGTOJUIZO AS indPagtoJuizo
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
     /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2299($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2299*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula,
       --infoDeslig
       DECODE(MO.NRMOVIMENTACAO,NULL,TD.CDESOCIAL,'13') AS mtvDeslig,
       TO_CHAR(DECODE(MO.NRMOVIMENTACAO,NULL,VM.DTRESCISAOVINC,MO.DTFIMMOVIMENT),'YYYY-MM-DD') AS dtDeslig,
       DECODE(MO.NRMOVIMENTACAO,NULL,DECODE(VM.DTRESCISAOVINC, VM.DTAVISOPREVIO, 'S', 'N'),'N') indPagtoAPI,       
       DECODE(MO.NRMOVIMENTACAO,NULL,DECODE(VM.DTRESCISAOVINC, VM.DTAVISOPREVIO, TO_CHAR(VM.DTAVISOPREVIO,'YYYY-MM-DD'), NULL),NULL) AS dtProjFimAPI,
       CASE WHEN ITC.VRPENSAO > 0 THEN 2 ELSE 0 END AS pensAlim,
       NULL AS percAliment,
       CASE WHEN ITC.VRPENSAO > 0 THEN ROUND(ITC.VRPENSAO,2) ELSE NULL END AS vrAlim,
       PH.NRCERTOBITO AS nrCertObito,
       VM.NRPROCJUD AS nrProcTrab,
       DECODE(VM.NRTPAVIPRE, 3, 4, 0) indCumprParc,
       DECODE(CE.CDESOCIAL,'111',VM.NRDIASAPURVALES,NULL) qtdDiasInterm,
       --observacoes getDadosS2299observacoes
       --sucessaoVinc
       DECODE(MO.NRMOVIMENTACAO,NULL,NULL,DECODE(MLP.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')) tpInscSuc,
       DECODE(MO.NRMOVIMENTACAO,NULL,NULL,MLH.CDCNPJESTRUT) cnpjSucessora,
       --transfTit
       null cpfSubstituto,
       null dtNascto,
       --verbasResc
       --dmDev getDadosS2299dmDev       
       --ideEstabLot getDadosS2299ideEstabLot       
       --detVerbas getDadosS2299detVerbas       
       --detOper getDadosS2299detOper       
       --detPlano getDadosS2299detPlano       
       --procJudTrab getDadosS2299procJudTrab     
       --infoMV
       --getDadosS2299remunOutrEmpr
       DECODE(OV.VREVENTOFOLHA, NULL, NULL, DECODE(VH.IDMULTVINC, 'PRIMARIO', '1', '3')) indMV,
       null dtFimQuar,
       'N' idConsig,
       '' insConsig,
       '' nrContr,
       VM.NRORG, VM.NRVINCULOM, DECODE(MO.NRMOVIMENTACAO,NULL,VM.DTRESCISAOVINC,MO.DTFIMMOVIMENT) AS DTRESCISAOVINC, VM.NRPESSOA, PE.NRPARCNEGOCIO, VH.NRESTRUTLEGAL, VH.NRESTRUTGEREN, CE.CDESOCIAL CDCATEGTRAB, 
       :P_NRTPAMBIENTE NRTPAMBIENTE, DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR) INIVALIDADEAMB, :P_DTMESCOMPETENC DTMESCOMPETENC,
       VH.NRAGENTENOCIVO
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
  LEFT JOIN ESTRUTURAH EMH ON EMH.NRORG = EMP.NRORG
                     AND EMH.NRESTRUTURAM = EMP.NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Demissao*/
  LEFT JOIN FPA_TPDEMISSAO TD ON VM.NRTPDEMISSAO = TD.NRTPDEMISSAO 
                             AND (TD.NRORG = :P_NRORG OR TD.NRORG = 0)
  /*Pensão*/  
  LEFT JOIN (SELECT ITC.NRORG, ITC.NRVINCULOM, SUM(ITC.VREVENTOFOLHA) VRPENSAO
               FROM FPA_ITECALCFOLHA ITC
              WHERE ITC.NRORG = :P_NRORG
                AND ITC.NRVINCULOM = :P_NRCHAVE
                AND ITC.NRCALCULOFOLHA = (SELECT CF.NRCALCULOFOLHA
                                            FROM FPA_CALCULOFOLHA CF
                                           WHERE CF.NRORG = ITC.NRORG
                                             AND CF.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                             AND CF.NRTPMODALIDCAL = 1
                                             AND CF.NRTIPOMOVIMENT = 1
                                             AND CF.DTOCORRENCIA = (SELECT MAX(C.DTOCORRENCIA)
                                                                      FROM FPA_ITECALCFOLHA I
                                                                      JOIN GPE_VINCULOM V ON V.NRORG = I.NRORG
                                                                                         AND V.NRVINCULOM = I.NRVINCULOM
                                                                      JOIN FPA_CALCULOFOLHA C ON C.NRORG = I.NRORG
                                                                                             AND C.NRCALCULOFOLHA = I.NRCALCULOFOLHA
                                                                                             AND C.NRTIPOMOVIMENT = 1
                                                                                             AND C.NRTPMODALIDCAL = 1
                                                                     WHERE I.NRORG = :P_NRORG
                                                                       AND I.NRVINCULOM = :P_NRCHAVE
                                                                       AND I.NREVENTOM = '3050'
                                                                       AND I.DTOCORRENCIA <= ADD_MONTHS(LAST_DAY(V.DTRESCISAOVINC),1)))
                AND ITC.NREVENTOM IN (SELECT IE.NREVENTOM
                                        FROM FPA_INCIDEVENTO IE
                                       WHERE (IE.NRORG = ITC.NRORG OR IE.NRORG = 0)
                                         AND IE.NRTIPOINCIDE IN (76,132)) /*DIRF - RTPA, INCIDE PENSAO FERIAS*/
              GROUP BY ITC.NRORG, ITC.NRVINCULOM) ITC ON ITC.NRORG = VM.NRORG
                                                     AND ITC.NRVINCULOM = VM.NRVINCULOM
  /*Remuneração Outra Empresa*/
  LEFT JOIN (SELECT OV.NRORG, OV.NRVINCULOM, OV.NRPESSOA, SUM(ITC.VREVENTOFOLHA) VREVENTOFOLHA
               FROM GPE_VINCULOM OV
               JOIN FPA_ITECALCFOLHA ITC ON ITC.NRORG = OV.NRORG
                                        AND ITC.NRVINCULOM = OV.NRVINCULOM
                                        AND ITC.NREVENTOM = 3050
               JOIN FPA_CALCULOFOLHA CF ON CF.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                       AND CF.NRORG = ITC.NRORG
                                       AND CF.NRTIPOMOVIMENT = 1
                                       AND CF.NRTPMODALIDCAL = 1
                                       AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
              WHERE OV.NRORG = :P_NRORG
                --AND OV.NRVINCULOM = :P_NRCHAVE
              GROUP BY OV.NRORG, OV.NRVINCULOM, OV.NRPESSOA) OV ON OV.NRORG = VM.NRORG
                                                               AND OV.NRPESSOA = VM.NRPESSOA
                                                               AND OV.NRVINCULOM <> VM.NRVINCULOM
  /*Caso tenha movimentação de transferencia de saida*/
  LEFT JOIN GPE_MOVIMENTACAO MO ON MO.NRORG = VM.NRORG
                               AND MO.NRVINCULOM = VM.NRVINCULOM
                               AND MO.NRESTRUTURAM = :P_NRESTRUTURAM
                               AND MO.NRTPMOVTRANSFM = 9
                               AND MO.DTFIMMOVIMENT BETWEEN :P_DTMESCOMPETENC AND LAST_DAY(:P_DTMESCOMPETENC)
  /*Movimentação de admissão proveniente de transferencia*/
  LEFT JOIN GPE_MOVIMENTACAO ML ON ML.NRORG = MO.NRORG
                               AND ML.NRVINCULOM = MO.NRVINCULOM
                               AND ML.NRTIPOTRANSFER = MO.NRTIPOTRANSFER
                               AND ML.NRTIPOESTRUTURA = MO.NRTIPOESTRUTURA
                               AND ML.NRTPMOVTRANSFM = 8
                               AND ML.DTINIMOVIMENT > MO.DTFIMMOVIMENT
  LEFT JOIN ESTRUTURAH MLH ON MLH.NRORG = ML.NRORG
                          AND MLH.NRESTRUTURAM = ML.NRESTRUTURAM
                          AND MLH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND ESTRUTURAH.NRESTRUTURAM = MLH.NRESTRUTURAM
                                                       AND ESTRUTURAH.NRORG = MLH.NRORG)
  LEFT JOIN PARCNEGOCIO MLP ON MLP.NRORG = MLH.NRORG
                           AND MLP.NRPARCNEGOCIO = MLH.NRPARCNEGOCIO
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidPeriodProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidPeriodProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidPeriodNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRTPAMBIENTE" => $nrtpambiente,
        );
        
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2299observacoes($nrorg, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2299observacoes*/
SELECT VM.DSOBSRESCISAO AS observacao
  FROM GPE_VINCULOM VM
 WHERE VM.NRVINCULOM = :P_NRVINCULOM
   AND VM.NRORG = :P_NRORG
   AND VM.DSOBSRESCISAO IS NOT NULL
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRVINCULOM" => $nrvinculom,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299dmDev($nrorg, $nrvinculom, $dtrescisao, $nrestrutlegal, $nrestrutgeren, $cdcategtrab, $nrtpambiente, $mtvDeslig, $inivalidadeamb, $nrpessoa, $nrinsc, $dtmescompetenc, $nragentenocivo) {
        $sql = <<<SQL
/*getDadosS2299dmDev*/
SELECT ITC.NRCALCULOFOLHA AS ideDmDev,
       ITC.NRORG, ITC.NRVINCULOM, ITC.NRCALCULOFOLHA, :P_NRESTRUTLEGAL NRESTRUTLEGAL, :P_NRESTRUTGEREN NRESTRUTGEREN, :P_CDCATEGTRAB CDCATEGTRAB, :P_DTMESCOMPETENC DTMESCOMPETENC, :P_NRAGENTENOCIVO NRAGENTENOCIVO
  FROM FPA_ITECALCFOLHA ITC
  JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                           AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                           AND CFL.NRTIPOMOVIMENT IN (1,4,10,19)
                           AND CFL.NRTPMODALIDCAL = 1
 WHERE ITC.NRVINCULOM = :P_NRCHAVE
   AND ITC.NRORG = :P_NRORG
   AND ((CFL.NRTIPOMOVIMENT = 1 AND ITC.NREVENTOM = '3050' AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                                                                     FROM FPA_ITECALCFOLHA IT
                                                                                     JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                                                             AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                                                             AND CF.NRTIPOMOVIMENT = CFL.NRTIPOMOVIMENT
                                                                                                             AND CF.NRTPMODALIDCAL = 1
                                                                                    WHERE IT.NRORG      = ITC.NRORG
                                                                                      AND IT.NRVINCULOM = ITC.NRVINCULOM
                                                                                      AND IT.NREVENTOM = ITC.NREVENTOM
                                                                                      AND CF.DTOCORRENCIA <= FIRST_DAY(:P_DTRESCISAOVINC)))
        OR (CFL.NRTIPOMOVIMENT = 10 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC))
        OR (CFL.NRTIPOMOVIMENT = 19 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC))
        OR (CFL.NRTIPOMOVIMENT = 4 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC) AND SUBSTR(CFL.DTOCORRENCIA, 3, 2) <> '12'))
   AND TO_DATE(:P_DTRESCISAOVINC) >= :P_INIVALIDADEAMB
   AND :P_MTVDESLIG NOT IN ('11','12','13','25','28','29','30','34','36')
   AND ((NOT EXISTS (SELECT 1
                       FROM ESO_ENVIOCADASTROS EC
                       JOIN ESTRUTURAH ESH ON ESH.NRORG = EC.NRORG
                                          AND ESH.NRESTRUTURAM = EC.NRESTRUTURAM
                                          AND SUBSTR(ESH.CDCNPJESTRUT, 1, 8) =  SUBSTR(:P_NRINSC, 1,8)
                                          AND ESH.DTMESCOMPETENC = (SELECT MAX(ESHH.DTMESCOMPETENC)
                                                                      FROM ESTRUTURAH ESHH
                                                                     WHERE ESHH.NRORG = ESH.NRORG
                                                                       AND ESHH.NRESTRUTURAM = ESH.NRESTRUTURAM
                                                                       AND ESHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
                      WHERE EC.NRORG = :P_NRORG
                        AND EC.IDSTATUS IN ('C', 'A')
                        AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
                        AND EC.NRCHAVE = :P_NRPESSOA
                        AND EC.NRTABELA = 4
                        AND TRUNC(TO_DATE(EC.DTMESCOMPETENC), 'YEAR') = TRUNC(TO_DATE(:P_DTMESCOMPETENC), 'YEAR')
                        AND EC.NRENVIOEXCL IS NULL
                        AND EC.IDATIVO = 'S'
                        AND EC.NREVENTO IN (SELECT SE.NREVENTO
                                              FROM ESO_EVENTO SE
                                             WHERE SE.NMEVENTO = 'S-1200')) AND CFL.NRTIPOMOVIMENT = 4) OR CFL.NRTIPOMOVIMENT <> 4)
   AND EXISTS (SELECT 1
                 FROM FPA_ITECALCFOLHA IT
                 JOIN FPA_CALCULOFOLHA CF ON IT.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                                         AND CF.NRORG = IT.NRORG
                 JOIN FPA_EVENTOH EH ON EH.NREVENTOM = IT.NREVENTOM
                                    AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                                    AND EH.DTMESCOMPETENC  = (SELECT MAX(EHH.DTMESCOMPETENC) FROM FPA_EVENTOH EHH
                                                               WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                 AND EHH.NREVENTOM = EH.NREVENTOM
                                                                 AND EHH.NRORG = EH.NRORG)
                 /*Natureza da Rubrica*/
                 JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
                WHERE IT.NRORG = ITC.NRORG
                  AND IT.NRVINCULOM = ITC.NRVINCULOM
                  AND IT.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                  AND IT.VREVENTOFOLHA > 0
                  AND ((IT.IDEVENDEMONCALC = 'N' AND EH.IDEVENTODEMONST = 'N') OR (EH.IDEVENTODEMONST = 'S')) /*SE NÃO DEMONSTRATIVO NO CADASTRO E DEMONSTRATIVO NA FOLHA NÃO ENVIA*/
                  AND (SELECT NVL(MAX(INC.CDESOCIAL),'00')
                         FROM FPA_INCIDEVENTO IE
                         JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
                                               AND INSTR(','||TI.DSNRTIPOMOVIMENT||',',','||CF.NRTIPOMOVIMENT||',') > 0
                         JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
                        WHERE (IE.NRORG = :P_NRORG OR IE.NRORG = 0)
                          AND IE.NREVENTOM = ITC.NREVENTOM
                          AND (IE.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                                     FROM FPA_GRPITEM 
                                                    WHERE NRGRPINCIDE = 47) OR TI.DSNRTIPOMOVIMENT IS NOT NULL)) NOT IN (31, 32, 33, 34, 35, 51, 52, 53, 54, 55, 81, 82, 83) /*codIncIRRF*/)
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRCHAVE" => $nrvinculom,
            "P_DTRESCISAOVINC" => $dtrescisao,
            "P_NRESTRUTLEGAL" => $nrestrutlegal,
            "P_NRESTRUTGEREN" => $nrestrutgeren,
            "P_CDCATEGTRAB" => $cdcategtrab,
            "P_INIVALIDADEAMB" => $inivalidadeamb,
            "P_MTVDESLIG" => $mtvDeslig,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_NRPESSOA" => $nrpessoa,
            "P_NRINSC" => $nrinsc,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRAGENTENOCIVO" => $nragentenocivo,
        );
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299ideEstabLot($nrorg, $nrvinculom, $nrcalculofolha, $nrestrutlegal, $nrestrutgeren, $cdcategtrab, $dtmescompetenc, $nragentenocivo) {
        $sql = <<<SQL
/*getDadosS2299ideEstabLot*/
SELECT DISTINCT
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') AS tpInsc,
       REPLACE(REPLACE(REPLACE(PN.NRINSCRICAOPARC,'.',''),'-',''),'/','') nrInsc,
       NVL(EMSUP.NRESTRUTURAM, EM.NRESTRUTURAM) AS codLotacao,
       DECODE(AN.CDESOCIALNOCIVO,NULL,1,AN.CDESOCIALNOCIVO) AS grauExp,
       DECODE(NVL(CTSUP.CDESOCIAL,CT.CDESOCIAL),'01','1','02','2','03','3','') AS indSimples,
       EM.NRORG, :P_NRVINCULOM NRVINCULOM, :P_NRCALCULOFOLHA NRCALCULOFOLHA,
       (SELECT DECODE(NVL(SUM(IC.VREVENTOFOLHA),0),0,0,1) IDDETOPER
          FROM FPA_ITECALCFOLHA IC
          JOIN FPA_EVENTOH EVH ON (EVH.NRORG = IC.NRORG OR EVH.NRORG = 0)
                              AND EVH.NREVENTOM = IC.NREVENTOM
                              AND EVH.DTMESCOMPETENC  = (SELECT MAX(EHH.DTMESCOMPETENC) FROM FPA_EVENTOH EHH
                                                          WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                            AND EHH.NREVENTOM = EVH.NREVENTOM
                                                            AND EHH.NRORG = EVH.NRORG)
          JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EVH.NRNATURUBR
         WHERE IC.NRORG = :P_NRORG
           AND IC.NRVINCULOM = :P_NRVINCULOM
           AND IC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
           AND NR.CDESOCIAL = '9219') IDDETOPER /*existe desconto de assistencia medica*/,
           :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM ESTRUTURAM EM
  LEFT JOIN ESTRUTURAH EH ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.NRORG = EM.NRORG
                    AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC) FROM ESTRUTURAH EHH
                                              WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND EHH.NRORG = EH.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRORG = EM.NRORG
                     AND PN.NRPARCNEGOCIO = EM.NRPARCNEGOCIO
  LEFT JOIN CLASSTRIBUT CT ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT
  /*Tomador superior da estrutura gerencial*/
  LEFT JOIN TIPOHIERARQORG TH ON (TH.NRORG = EM.NRORG OR TH.NRORG = 0)
                             AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                             AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
  LEFT JOIN HIERARQUIAORG HO ON HO.NRORG = EM.NRORG
                            AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
                            AND HO.NRESTRUTURAMREL = :P_NRESTRUTGEREN
  LEFT JOIN ESTRUTURAM EMSUP ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                            AND EMSUP.NRORG = HO.NRORG
  LEFT JOIN ESTRUTURAH EHSUP ON EHSUP.NRESTRUTURAM = EMSUP.NRESTRUTURAM
                            AND EHSUP.NRORG = EMSUP.NRORG
                            AND EHSUP.DTMESCOMPETENC = (SELECT MAX(EHHSUP.DTMESCOMPETENC) FROM ESTRUTURAH EHHSUP
                                                         WHERE EHHSUP.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND EHHSUP.NRESTRUTURAM = EH.NRESTRUTURAM
                                                           AND EHHSUP.NRORG = EH.NRORG)
  LEFT JOIN PARCNEGOCIO PNSUP ON PNSUP.NRPARCNEGOCIO = EMSUP.NRPARCNEGOCIO
                             AND PNSUP.NRORG = EMSUP.NRORG
  LEFT JOIN CLASSTRIBUT CTSUP ON CTSUP.NRCLASSTRIBUT = EHSUP.NRCLASSTRIBUT
  /*Grau de Exposição de Agente Nocivo*/
  LEFT JOIN GPE_AGENTENOCIVO AN ON AN.NRAGENTENOCIVO = :P_NRAGENTENOCIVO
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRESTRUTURAM = :P_NRESTRUTLEGAL
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
            "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            "P_NRVINCULOM" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
            "P_NRESTRUTLEGAL" => $nrestrutlegal,
            "P_NRESTRUTGEREN" => $nrestrutgeren,
            "P_CDCATEGTRAB" => $cdcategtrab,
            "P_NRAGENTENOCIVO" => $nragentenocivo,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299detVerbas($nrorg, $nrvinculom, $nrcalculofolha, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2299detVerbas*/
SELECT ITC.NREVENTOM AS codRubr,
       CASE WHEN TO_NUMBER(TO_CHAR(CF.DTOCORRENCIA,'MM')) BETWEEN 1 AND 11 AND CF.NRTIPOMOVIMENT = 4 THEN 22 ELSE CF.NRTIPOMOVIMENT END AS ideTabrubr,
       ROUND(DECODE(ITC.VRREFFOLHA, 0, '', ITC.VRREFFOLHA),2) AS qtdRubr,
       ROUND(EH.VRPERCENTUALEVENTO,2) AS fatorRubr,
       ROUND(DECODE(ITC.VRREFFOLHA, 0, '', (ITC.VREVENTOFOLHA / ITC.VRREFFOLHA)),2) AS vrUnit,
       ROUND(ITC.VREVENTOFOLHA,2) AS vrRubr,
       ITC.NRORG, ITC.NRVINCULOM, ITC.NRCALCULOFOLHA, :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM FPA_ITECALCFOLHA ITC
  JOIN FPA_CALCULOFOLHA CF ON ITC.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                          AND CF.NRORG = ITC.NRORG
  JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                     AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                     AND EH.DTMESCOMPETENC  = (SELECT MAX(EHH.DTMESCOMPETENC) FROM FPA_EVENTOH EHH
                                                WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND EHH.NREVENTOM = EH.NREVENTOM
                                                  AND EHH.NRORG = EH.NRORG)
  /*Natureza da Rubrica*/
  JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
 WHERE ITC.NRORG = :P_NRORG
   AND ITC.NRVINCULOM = :P_NRCHAVE
   AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
   AND ITC.VREVENTOFOLHA > 0
   AND ((ITC.IDEVENDEMONCALC = 'N' AND EH.IDEVENTODEMONST = 'N') OR (EH.IDEVENTODEMONST = 'S')) /*SE NÃO DEMONSTRATIVO NO CADASTRO E DEMONSTRATIVO NA FOLHA NÃO ENVIA*/
   AND (SELECT NVL(MAX(INC.CDESOCIAL),'00')
          FROM FPA_INCIDEVENTO IE
          JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
                                AND INSTR(','||TI.DSNRTIPOMOVIMENT||',',','||CF.NRTIPOMOVIMENT||',') > 0
          JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
         WHERE (IE.NRORG = :P_NRORG OR IE.NRORG = 0)
           AND IE.NREVENTOM = ITC.NREVENTOM
           AND (IE.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                      FROM FPA_GRPITEM 
                                     WHERE NRGRPINCIDE = 47) OR TI.DSNRTIPOMOVIMENT IS NOT NULL)) NOT IN (31, 32, 33, 34, 35, 51, 52, 53, 54, 55, 81, 82, 83) /*codIncIRRF*/
 ORDER BY ITC.NREVENTOM
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299detOper($nrorg, $nrvinculom, $nrcalculofolha, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2299detOper*/
SELECT cnpjOper, regANS, SUM(vrPgTit) vrPgTit,
       :P_NRORG NRORG, :P_NRCHAVE NRVINCULOM, :P_NRCALCULOFOLHA NRCALCULOFOLHA, CDCNPJESTRUT, :P_DTMESCOMPETENC DTMESCOMPETENC
FROM (SELECT DISTINCT
             REPLACE(REPLACE(REPLACE(ET.CDCNPJESTRUT,'.'),'/'),'-') AS cnpjOper,
             REPLACE(ET.CDFEDERALESTRUT,'-') AS regANS,
             CASE WHEN ITC.NREVENTOM IN (BH.NREVENTOMENSA, BH.NREVETITULAREMP, BH.NREVETITULARVINC, BH.NREVENTOCOPART) THEN ROUND(ITC.VREVENTOFOLHA,2) ELSE 0 END AS vrPgTit,
             ITC.NREVENTOM,
             ET.CDCNPJESTRUT,
             :P_DTMESCOMPETENC DTMESCOMPETENC
        FROM FPA_ITECALCFOLHA ITC
        JOIN FPA_CALCULOFOLHA CF ON ITC.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                                AND CF.NRORG = ITC.NRORG
        JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                           AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                           AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC)
                                                      FROM FPA_EVENTOH EHH
                                                     WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND EHH.NREVENTOM = EH.NREVENTOM
                                                       AND EHH.NRORG = EH.NRORG)
        JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
        JOIN VAB_BENEFICIARIO BF ON BF.NRVINCULOM = ITC.NRVINCULOM
                                AND BF.NRORG = ITC.NRORG
                                AND BF.IDTPBENEFICIARIO = 'T'
        JOIN VAB_BENEFICIOM BM ON BM.NRBENEFICIOM = BF.NRBENEFICIOM
                              AND BM.NRORG = BF.NRORG
                              AND BM.DTINICIOVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                              AND (BM.DTFINALVIGENCIA IS NULL OR BM.DTFINALVIGENCIA >= :P_DTMESCOMPETENC)
        JOIN VAB_BENEFICIOH BH ON BH.NRBENEFICIOM = BM.NRBENEFICIOM
                              AND BH.NRORG = BM.NRORG
                              AND BH.TPPERIODOBENEF = 'MENSAL'
                              AND ITC.NREVENTOM IN (BH.NREVENTOMENSA, BH.NREVENTOMENSADEP, BH.NREVETITULAREMP, BH.NREVEDEPEMPRESA, BH.NREVETITULARVINC, BH.NREVEDEPEVINC, BH.NREVENTOCOPART, BH.NREVENTOCOPARTDEP)
                              AND BH.DTMESCOMPETENC = (SELECT MAX(BHH.DTMESCOMPETENC)
                                                         FROM VAB_BENEFICIOH BHH
                                                        WHERE BHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND BHH.NRBENEFICIOM = BH.NRBENEFICIOM
                                                          AND BHH.NRORG = BH.NRORG)
        JOIN ESTRUTURAM EM ON EM.NRESTRUTURAM = BH.NRESTRUTURAM
                          AND EM.NRORG = :P_NRORG
        LEFT JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = EM.NRESTRUTURAM
                          AND ET.NRORG = EM.NRORG
                          AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                     FROM ESTRUTURAH
                                                    WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                      AND ESTRUTURAH.NRORG = ET.NRORG)
       WHERE ITC.NRORG = :P_NRORG
         AND ITC.NRVINCULOM = :P_NRCHAVE
         AND ITC.VREVENTOFOLHA > 0
         AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
         AND NR.CDESOCIAL = '9219')
GROUP BY cnpjOper, regANS, CDCNPJESTRUT
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299detPlano($nrorg, $nrvinculom, $nrcalculofolha, $cdcnpjestrut, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2299detPlano*/
SELECT tpDep, cpfDep, nmDep, dtNascto, SUM(vlrPgDep) vlrPgDep
FROM (SELECT DISTINCT
             TR.CDESOCIAL AS tpDep,
             REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS cpfDep,
             PH.NMPESSOA AS nmDep,
             TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
             ROUND(ITC.VREVENTOFOLHA/NVL(DECODE(ITC.VRREFFOLHA,0,1,ITC.VRREFFOLHA),1),2) AS vlrPgDep,
             ITC.NREVENTOM,
             :P_DTMESCOMPETENC DTMESCOMPETENC
        FROM FPA_ITECALCFOLHA ITC
        JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                           AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                           AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC)
                                                      FROM FPA_EVENTOH EHH
                                                     WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND EHH.NREVENTOM = EH.NREVENTOM
                                                       AND EHH.NRORG = EH.NRORG)
        JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
        JOIN VAB_BENEFICIARIO BF ON BF.NRVINCULOM = ITC.NRVINCULOM
                                AND BF.NRORG = ITC.NRORG
                                AND BF.IDTPBENEFICIARIO = 'D'
        JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = BF.NRVINCULOM
                            AND VM.NRORG = BF.NRORG
        JOIN GPE_PESSOA PV ON PV.NRPESSOA = VM.NRPESSOA
                          AND PV.NRORG = VM.NRORG
        JOIN GPE_PESSOA PE ON PE.NRPESSOA = BF.NRPESSOA
                          AND PE.NRORG = BF.NRORG
        JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                           AND PH.NRORG = PE.NRORG
                           AND PH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC)
                                                      FROM GPE_PESSOAH H1
                                                     WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND H1.NRPESSOA = PH.NRPESSOA
                                                       AND H1.NRORG = PH.NRORG)
        JOIN RELACIONAPARC RP ON RP.NRORG = :P_NRORG
                             AND RP.NRPARCNEGOCIO = PV.NRPARCNEGOCIO
                             AND RP.NRPARCNEGRELAC = PE.NRPARCNEGOCIO
        JOIN TIPORELACIONA TR ON TR.NRTIPORELACIONA = RP.NRTIPORELACIONA
        JOIN VAB_BENEFICIOH BH ON BH.NRBENEFICIOM = BF.NRBENEFICIOM
                              AND BH.NRORG = BF.NRORG
                              AND BH.TPPERIODOBENEF = 'MENSAL'
                              AND ITC.NREVENTOM IN (BH.NREVENTOMENSADEP, BH.NREVEDEPEMPRESA, BH.NREVEDEPEVINC, BH.NREVENTOCOPARTDEP)
                              AND BH.DTMESCOMPETENC = (SELECT MAX(BHH.DTMESCOMPETENC)
                                                         FROM VAB_BENEFICIOH BHH
                                                        WHERE BHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND BHH.NRBENEFICIOM = BH.NRBENEFICIOM
                                                          AND BHH.NRORG = BH.NRORG)
        JOIN ESTRUTURAM EM ON EM.NRESTRUTURAM = BH.NRESTRUTURAM
                          AND EM.NRORG = :P_NRORG
        LEFT JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = EM.NRESTRUTURAM
                          AND ET.NRORG = EM.NRORG
                          AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                     FROM ESTRUTURAH
                                                    WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                      AND ESTRUTURAH.NRORG = ET.NRORG)
       WHERE ITC.NRORG = :P_NRORG
         AND ITC.NRVINCULOM = :P_NRCHAVE
         AND ITC.VREVENTOFOLHA > 0
         AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
         AND NR.CDESOCIAL = '9219'
         AND ET.CDCNPJESTRUT = :P_CDCNPJESTRUT)
GROUP BY tpDep, cpfDep, nmDep, dtNascto
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
            "P_CDCNPJESTRUT" => $cdcnpjestrut
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
     public function getDadosS2299procJudTrab($nrorg, $nrestrutura, $nrparcnegocio) {
        $sql = <<<SQL
/*getDadosS2299procJudTrab*/
SELECT AD.CDESOCIAL AS tpTrib,
       PA.NRPROCESSADMJUD AS nrProcJud,
       IPA.NRSUSPEXIGITPROC AS codSusp
  FROM GPE_PROCESSADMJUD PA
  JOIN GPE_ITEMPROCADMJUD IPA ON PA.NRORG = IPA.NRORG
                             AND PA.NRPROCESSADMJUD = IPA.NRPROCESSADMJUD
  LEFT JOIN GPE_ABDECISAO AD ON (AD.NRORG = IPA.NRORG OR AD.NRORG = 0)
                            AND AD.NRABDECISAO = IPA.NRABDECISAO
 WHERE PA.NRORG = :P_NRORG
   AND PA.NRESTRUTURAM = :P_NRESTRUTURAM
   AND PA.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESTRUTURAM" => $nrestrutura,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2299remunOutrEmpr($nrorg, $nrvinculom, $nrPessoa, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2299remunOutrEmpr*/
SELECT DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') AS tpInsc,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END nrInsc,
       CE.CDESOCIAL AS codCateg,
       ITC.VREVENTOFOLHA AS vlrRemunOE,
       :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                 FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Outro Empregador*/
  JOIN ESTRUTURAM EM ON EM.NRORG = VM.NRORG
                    AND EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
  LEFT JOIN ESTRUTURAH EH ON EH.NRESTRUTURAM    = EM.NRESTRUTURAM
                    AND EH.NRORG          = EM.NRORG
                    AND EH.DTMESCOMPETENC =
                      (SELECT MAX(EHH.DTMESCOMPETENC)
                      FROM ESTRUTURAH EHH
                      WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                      AND EHH.NRESTRUTURAM      = EH.NRESTRUTURAM
                      AND EHH.NRORG             = EH.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRORG             = EM.NRORG
                     AND PN.NRPARCNEGOCIO    = EM.NRPARCNEGOCIO
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Remuneração Outra Empresa*/
  JOIN (SELECT ITC.NRORG, ITC.NRVINCULOM, SUM(ITC.VREVENTOFOLHA) VREVENTOFOLHA
          FROM FPA_ITECALCFOLHA ITC
          JOIN FPA_CALCULOFOLHA CF ON CF.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                  AND CF.NRORG = ITC.NRORG
                                  AND CF.NRTIPOMOVIMENT = 1
                                  AND CF.NRTPMODALIDCAL = 1
                                  AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
         WHERE ITC.NRORG = :P_NRORG
           AND ITC.NREVENTOM = 3050
         GROUP BY ITC.NRORG, ITC.NRVINCULOM) ITC ON ITC.NRORG = VM.NRORG
                                                AND ITC.NRVINCULOM = VM.NRVINCULOM
 WHERE VM.NRPESSOA = :P_NRPESSOA
   AND VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM != :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRPESSOA" => $nrPessoa
        );
        return $this->connection->fetchAll($sql, $params);
    }

    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2300($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc, $dataEnvioOrig) {
        $sql = <<<SQL
/*getDadosS2300*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --trabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') cpfTrab,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       PH.NMPESSOA nmTrab,
       PH.IDSEXOPESSOA sexo,
       RP.CDESOCIAL racaCor,
       EC.CDESOCIAL estCiv,
       GI.CDESOCIAL grauInstr,
       '' nmSoc,
       --nascimento
       TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       DECODE(PH.CDPAIS,'0055',MU.CDMUNICIBGE,'') codMunic,
       DECODE(PH.CDPAIS,'0055',PH.SGESTADO,'') uf,
       PA.CDESOCIAL paisNascto,
       PB.CDESOCIAL paisNac,
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 1) nmMae, -- 1=Mae
       (SELECT MAX(PN.NMPRINCIPALPARC)
          FROM RELACIONAPARC RP
          JOIN PARCNEGOCIO PN ON RP.NRPARCNEGRELAC = PN.NRPARCNEGOCIO
                             AND RP.NRORG = PN.NRORG
          WHERE VM.NRORG = RP.NRORG
            AND RP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            AND RP.NRTIPORELACIONA = 2) nmPai, -- 2=Pai
       --documentos
       --CTPS
       PH.NRCTPSPESSOA nrCtps,
       PH.NRSERIECTPSPES serieCtps,
       PH.SGUFCTPSPES ufCtps,
       --RIC
       --RG
       PH.NRRGPESSOA nrRg,
       PH.CDEXRGPESSOA||PH.SGUFRGPESSOA AS rg_orgaoEmissor,
       TO_CHAR(PH.DTEXRGPESSOA,'YYYY-MM-DD') rg_dtExped,
       --RNE
       PH.NRDOCUESTRANG nrRne,
       PH.DSEXPDOCESTR rne_orgaoEmissor,
       '' rne_dtExped,
       --OC
       --CNH
       --So traz dados CNH se tiver UF da CNH (mobile)
       DECODE(PH.SGUFHABCARPES, NULL, '', PH.NRCARTHABPES) nrRegCnh,
       DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTHABCARTPES,'YYYY-MM-DD')) cnh_dtExped,
       PH.SGUFHABCARPES ufCnh,
       DECODE(PH.SGUFHABCARPES, NULL, '', TO_CHAR(PH.DTVALHABCARTPES,'YYYY-MM-DD')) cnh_dtValid,
       '' dtPriHab,
       DECODE(PH.SGUFHABCARPES, NULL, '', PH.DSCATEGHABCART) categoriaCnh,
       --endereco
       --brasil
       NVL(LG.CDESOCIAL,'O') tpLograd,
       EP.DSENDERECO dscLograd,
       EP.NRIMOVELENDERECO nrLograd,
       SUBSTR(EP.DSCOMPLEENDERECO, 0, 30) complemento,
       EP.NMBAIRROENDERECO bairro,
       REPLACE(REPLACE(REPLACE(EP.NRCEPENDERECO,' '),'.'),'-') cep,
       MP.CDMUNICIBGE brasil_codMunic,
       EP.SGESTADO brasil_uf,
       --exterior
       PC.CDESOCIAL paisResid,
       EE.DSENDERECO exterior_dscLograd,
       EE.NRIMOVELENDERECO exterior_nrLograd,
       SUBSTR(EE.DSCOMPLEENDERECO, 0, 30) exterior_complemento,
       EE.NMBAIRROENDERECO exterior_bairro,
       ME.NMMUNICIPIO nmCid,
       REPLACE(REPLACE(REPLACE(EE.NRCEPENDERECO,' '),'.'),'-') codPostal,
       --trabEstrangeiro
       TO_CHAR(PH.DTCHEGADAPAIS,'YYYY-MM-DD') dtChegada,
       NVL(CT.CDESOCIAL,DECODE(PH.IDPOSVISTOPERM,'S','1','')) classTrabEstrang,
       CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA = 4 -- 4=Cônjuge
               AND H.CDPAIS = '0055'),'N') END casadoBr,
       CASE WHEN PH.NRANOCHEGAPAIS IS NOT NULL THEN NVL((SELECT MAX('S')
              FROM RELACIONAPARC R
              JOIN PARCNEGOCIO N ON R.NRPARCNEGRELAC = N.NRPARCNEGOCIO
                                AND R.NRORG = N.NRORG
              JOIN GPE_PESSOA P ON P.NRPARCNEGOCIO = N.NRPARCNEGOCIO
                               AND P.NRORG = N.NRORG
              JOIN GPE_PESSOAH H ON H.NRPESSOA = P.NRPESSOA
                                AND H.NRORG = P.NRORG
                                AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                                         WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND H1.NRPESSOA = H.NRPESSOA
                                                           AND H1.NRORG = H.NRORG)
             WHERE R.NRORG = PE.NRORG
               AND R.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
               AND R.NRTIPORELACIONA IN (3,100001) -- 3=Filho
               AND H.CDPAIS = '0055'),'N') END filhosBr,
       --infoDeficiencia
       DECODE(PH.NRCONDFISPES,'2','S','N') defFisica,
       DECODE(PH.NRCONDFISPES,'4','S','N') defVisual,
       DECODE(PH.NRCONDFISPES,'3','S','N') defAuditiva,
       DECODE(PH.NRCONDFISPES,'5','S','N') defMental,
       DECODE(PH.NRCONDFISPES,'8','S','N') defIntelectual,
       DECODE(PH.NRCONDFISPES,'7','S','N') reabReadap,
       'N' infoCota,
       '' dsObservacao,
       --<<dependentes>>-- getDadosS2200dependente
       --aposentadoria
       '' trabAposent, --Facultativo (até 31/12/2017 se {codCateg} = [104]). Não obrigatorio (nas demais situações).
       --contato
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '01') fonePrinc, -- 1-Telefone principal
       (SELECT REPLACE(REPLACE(REPLACE(REPLACE(REPLACE(MAX(C.CDPREFIXCOMUPARC||C.CDCOMUNICAPARC),' '),'-'),'('),')'),'N/A')
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU IN ('02','03','04','06')) foneAlternat, -- 2-Celular particular 3-Telefone residencial 4-Telefone comercial 6-Celular comercial
       (SELECT MAX(C.CDCOMUNICAPARC)
          FROM COMUNICAPARC C
         WHERE C.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
           AND C.NRORG = PE.NRORG
           AND C.CDFORMACOMU = '05') emailPrinc, -- 5-email
       '' emailAlternat,
       --infoTSVInicio
       CASE WHEN VM.DTADMISSAOVINC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN 'S' ELSE 'N' END cadIni,
       CE.CDESOCIAL codCateg,
       TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD') dtInicio,
       CASE WHEN CE.CDESOCIAL IN ('201','202','401','731','734','738') THEN (CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END) ELSE '' END natAtividade,
       --infoComplementares
       --cargoFuncao
       CASE WHEN CE.CDESOCIAL IN ('201','202','305','701','711','712','721','722','731','734','738') THEN VH.NRCARGO ELSE NULL END codCargo,
       CASE WHEN CE.CDESOCIAL IN ('201','202','305','701','711','712','721','722','731','734','738') THEN VH.NRFUNCAO ELSE NULL END codFuncao,
       --remuneracao
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN ROUND(SL.VRSALARIO, 2) ELSE NULL END vrSalFx,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') ELSE '' END undSalFixo,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN SUBSTR(SL.DSOBSERVACAO,1,90) ELSE '' END dscSalVar,
       --FGTS
       CASE WHEN CE.CDESOCIAL IN ('721') THEN DECODE(VM.DTOPCAOFGTS,NULL,'2','1') ELSE '' END opcFGTS,
       CASE WHEN CE.CDESOCIAL IN ('721') THEN TO_CHAR(VM.DTOPCAOFGTS,'YYYY-MM-DD') ELSE NULL END dtOpcFGTS,
       --infoDirigenteSindical
       DECODE(CE.CDESOCIAL,'401',CE.NRCATEGESOC,'') categOrig,
       DECODE(CE.CDESOCIAL,'401',REPLACE(REPLACE(REPLACE(ES.CDCNPJESTRUT,'.'),'/'),'-'),'') cnpjOrigem,
       DECODE(CE.CDESOCIAL,'401',TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD'),'') dtAdmOrig,
       DECODE(CE.CDESOCIAL,'401',VM.CDMATRICULA,'') matricOrig,
       --infoTrabCedido
       DECODE(CE.CDESOCIAL,'410',CE.NRCATEGESOC,'') infoTrabCedido_categOrig,
       DECODE(CE.CDESOCIAL,'410','__cnpjOrigem__','') cnpjCednt,
       DECODE(CE.CDESOCIAL,'410',VM.CDMATRICULA,'') matricCed,
       DECODE(CE.CDESOCIAL,'410',TO_CHAR(VM.DTADMISSAOVINC,'YYYY-MM-DD'),'') dtAdmCed,
       DECODE(CE.CDESOCIAL,'410','1','') tpRegTrab,
       DECODE(CE.CDESOCIAL,'410',VH.NRTPREGIMEPREV,'') tpRegPrev,
       DECODE(CE.CDESOCIAL,'410','__infOnus__','') infOnus,
       --infoEstagiario
       DECODE(CE.CDESOCIAL,'901','N','') natEstagio,
       DECODE(CE.CDESOCIAL,'901',DECODE(GI.CDESOCIAL,'02','1','03','1','04','1','05','1','06','2','07','2','08','4','09','4','8'),'') nivEstagio,
       DECODE(CE.CDESOCIAL,'901','','') areaAtuacao,
       DECODE(CE.CDESOCIAL,'901','','') nrApol,
       DECODE(CE.CDESOCIAL,'901',ROUND(SL.VRSALARIO, 2),'') vlrBolsa,
       DECODE(CE.CDESOCIAL,'901',TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD'),'') dtPrevTerm,
       --instEnsino
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEH.CDCNPJESTRUT,'.'),'/'),'-'),'') cnpjInstEnsino,
       DECODE(CE.CDESOCIAL,'901',NVL(IEH.NMRAZSOCESTRUT,FO.DSINSTITUICAO),'') nmRazao,
       DECODE(CE.CDESOCIAL,'901',IEE.DSENDERECO,'') instEnsino_dscLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NRIMOVELENDERECO,'') instEnsino_nrLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NMBAIRROENDERECO,'') instEnsino_bairro,
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEE.NRCEPENDERECO,' '),'.'),'-'),'') instEnsino_cep,
       DECODE(CE.CDESOCIAL,'901',MIE.CDMUNICIBGE,'') instEnsino_codMunic,
       DECODE(CE.CDESOCIAL,'901',IEE.SGESTADO,'') instEnsino_uf,
       --ageIntegracao
       --supervisorEstagio
       DECODE(CE.CDESOCIAL,'901',PNS.NRINSCRICAOPARC,'') cpfSupervisor,
       DECODE(CE.CDESOCIAL,'901',PNS.NMPRINCIPALPARC,'') nmSuperv,
       --afastamento
       TO_CHAR(
        CASE
            WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN NULL  /*Situacao Normal*/
            WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN GO.DTINIGOZOFERIAS  /*Em ferias*/
            WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN SF.DTINISITUFUNC  /*Afastado*/
            ELSE NULL   
        END,
       'YYYY-MM-DD') dtIniAfast,
       CASE
        WHEN SF.DTINISITUFUNC IS NULL AND GO.DTINIGOZOFERIAS IS NULL THEN ''  /*Situacao Normal*/
        WHEN GO.DTINIGOZOFERIAS < SF.DTINISITUFUNC OR SF.DTINISITUFUNC IS NULL THEN '15'  /*Em ferias*/
        WHEN SF.DTINISITUFUNC < GO.DTINIGOZOFERIAS OR GO.DTINIGOZOFERIAS IS NULL THEN MA.CDESOCIAL  /*Afastado*/
        ELSE ''
       END codMotAfast,
       --desligamento
       CASE WHEN VM.DTRESCISAOVINC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) THEN TO_CHAR(VM.DTRESCISAOVINC,'YYYY-MM-DD') ELSE NULL END dtTerm,
       VM.NRVINCULOM, PE.NRORG, PE.NRPARCNEGOCIO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  JOIN GPE_TIPOVINCULOH VT ON VT.NRTIPOVINCULOM = VM.NRTIPOVINCULOM
                          AND (VT.NRORG = VM.NRORG OR VT.NRORG = 0 OR VT.NRORG IS NULL)
                          AND VT.IDVINCULOEMPR = 'S'
                          AND VT.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_TIPOVINCULOH
                                                    WHERE GPE_TIPOVINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND GPE_TIPOVINCULOH.NRTIPOVINCULOM = VT.NRTIPOVINCULOM
                                                      AND GPE_TIPOVINCULOH.NRORG = VT.NRORG)
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
                                                  
  /*Estrutura*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = :P_NRORG
                     AND EMP.NRESTRUTURAM = :P_NRESTRUTURAM
                     
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = VH.NRESTRUTGEREN
                    AND ET.NRORG = VH.NRORG
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                     AND PN.NRORG = ET.NRORG
  /*Dados do Empregado - Dependentes*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Dados do Empregado - Naturalidade*/
  LEFT JOIN PAIS PA ON PA.CDPAIS = PH.CDPAIS
  LEFT JOIN MUNICIPIO MU ON MU.CDPAIS = PH.CDPAIS
                        AND MU.SGESTADO = PH.SGESTADO
                        AND MU.CDMUNICIPIO = PH.CDMUNICIPIO
  LEFT JOIN GPE_NACIONALIDADE NA ON PH.NRNACIONALID = NA.NRNACIONALIDADE
                                AND (PH.NRORG = NA.NRORG OR NA.NRORG = 0)
  LEFT JOIN PAIS PB ON PB.CDPAIS = NA.CDPAIS
  /*Endereço Principal*/
  LEFT JOIN ENDERECOPARC EP ON PE.NRPARCNEGOCIO = EP.NRPARCNEGOCIO
                           AND PE.NRORG = EP.NRORG
                           AND EP.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MP ON MP.CDPAIS = EP.CDPAIS
                        AND MP.SGESTADO = EP.SGESTADO
                        AND MP.CDMUNICIPIO = EP.CDMUNICIPIO
  /*Endereço Exterior*/
  LEFT JOIN ENDERECOPARC EE ON PE.NRPARCNEGOCIO = EE.NRPARCNEGOCIO
                           AND PE.NRORG = EE.NRORG
                           AND EE.CDTIPOENDERECO = 'EXTERIOR'
  LEFT JOIN MUNICIPIO ME ON ME.CDPAIS = EE.CDPAIS
                        AND ME.SGESTADO = EE.SGESTADO
                        AND ME.CDMUNICIPIO = EE.CDMUNICIPIO
  LEFT JOIN PAIS PC ON PC.CDPAIS = EE.CDPAIS
  /*Trabalhador Estrangeiro*/
  LEFT JOIN GPE_CLASSESTRANG CT ON CT.NRCLASSESTRANG = PH.NRCLASSESTRANG
  /*Cargo*/
  LEFT JOIN GPE_OCUPACAOH OH ON OH.NROCUPACAOM = VH.NRCARGO
                            AND OH.NRORG = VH.NRORG
                            AND OH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
                                                      WHERE GPE_OCUPACAOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                        AND GPE_OCUPACAOH.NROCUPACAOM = OH.NROCUPACAOM
                                                        AND GPE_OCUPACAOH.NRORG = OH.NRORG)
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  /*Horário*/
  LEFT JOIN GPE_ESCALATRABH EH ON EH.NRESCALATRABM = VH.NRESCALATRABM
                              AND EH.NRORG = VH.NRORG
                              AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_ESCALATRABH
                                                        WHERE GPE_ESCALATRABH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND GPE_ESCALATRABH.NRESCALATRABM = EH.NRESCALATRABM
                                                          AND GPE_ESCALATRABH.NRORG = EH.NRORG)
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)    
  LEFT JOIN PTO_TIPOJORNADA TJ ON TJ.NRTPJORNADA = EH.NRTPJORNADA
                              AND (TJ.NRORG = EH.NRORG OR TJ.NRORG = 0)
  /*Afastamento - Situacao Funcional*/
  LEFT JOIN (SELECT SF.NRORG, SF.NRVINCULOM, SF.DTINISITUFUNC, SF.DTFIMSITUFUNC, SF.NRMOTIVOAFASTA
               FROM GPE_ALTESITUFUNC SF
               JOIN GPE_SITUFUNCH SFH ON SFH.NRSITUFUNCM = SF.NRSITUFUNCM
                                     AND SFH.DTEFETIVA = (SELECT MAX(DTEFETIVA) FROM GPE_SITUFUNCH
                                                           WHERE GPE_SITUFUNCH.DTEFETIVA <= :P_DTMESCOMPETENC
                                                             AND GPE_SITUFUNCH.NRSITUFUNCM = SFH.NRSITUFUNCM
                                                             AND GPE_SITUFUNCH.NRORG = SFH.NRORG)
              WHERE SF.NRORG = :P_NRORG
                AND SF.NRVINCULOM = :P_NRCHAVE
                AND NVL(:P_DTENVIOORIG,SYSDATE) BETWEEN SF.DTINISITUFUNC AND NVL(SF.DTFIMSITUFUNC,NVL(:P_DTENVIOORIG,SYSDATE)) /*P_DTENVIOORIG para casos de retificacao*/
                AND (SFH.IDAFASTSITUFUNC = 'S' /*Situacao funcional que gere afastamento*/
                     OR SF.NRSITUFUNCM = 12 OR SF.NRSITUFUNCM = 100012 OR SF.NRSITUFUNCM = 15 OR SF.NRSITUFUNCM = 16) /*Atestados médico e odontológico, Afastamentos 15 dias empregador*/
                AND SF.NRSITUFUNCM <> 13 /*Demitito eh parametrizado como gera afastamento mas nao gera o evento*/
             ) SF ON SF.NRVINCULOM = VH.NRVINCULOM
                 AND SF.NRORG = VH.NRORG
                 AND SF.DTINISITUFUNC < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (SF.DTFIMSITUFUNC >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR SF.DTFIMSITUFUNC IS NULL)
  /*Motivo Afastamento*/
  LEFT JOIN GPE_MOTIVOAFASTA MA ON MA.NRMOTIVOAFASTA = SF.NRMOTIVOAFASTA
                               AND (MA.NRORG = SF.NRORG OR MA.NRORG = 0)
  /*Afastamento - Ferias*/
  LEFT JOIN (SELECT FE.NRORG, FE.NRVINCULOM, GO.DTINIGOZOFERIAS, GO.DTFIMGOZOFERIAS
               FROM FPA_GOZOFERIAS GO
               JOIN FPA_FERIAS FE ON FE.NRORG = GO.NRORG
                                 AND FE.NRFERIAS = GO.NRFERIAS
              WHERE GO.NRORG = :P_NRORG
                AND NVL(:P_DTENVIOORIG,SYSDATE) BETWEEN GO.DTINIGOZOFERIAS AND NVL(GO.DTFIMGOZOFERIAS,NVL(:P_DTENVIOORIG,SYSDATE)) /*P_DTENVIOORIG para casos de retificacao*/
             ) GO ON GO.NRORG = VH.NRORG
                 AND GO.NRVINCULOM = VH.NRVINCULOM
                 AND GO.DTINIGOZOFERIAS < TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR))
                 AND (GO.DTFIMGOZOFERIAS >= TO_DATE(DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR)) OR GO.DTFIMGOZOFERIAS IS NULL)
  /*Sindicato*/
  LEFT JOIN ESTRUTURAH ES ON ES.NRESTRUTURAM = VH.NRESTRUTSIND
                         AND ES.NRORG = VH.NRORG
                         AND ES.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                   WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                     AND ESTRUTURAH.NRESTRUTURAM = ES.NRESTRUTURAM
                                                     AND ESTRUTURAH.NRORG = ES.NRORG)
  /*RaçaCor*/
  LEFT JOIN GPE_RACAPESSOA RP ON RP.NRRACAPESSOA = PH.NRRACAPESSOA
                             AND (RP.NRORG = PH.NRORG OR RP.NRORG = 0)
  /*Estado Civil*/
  LEFT JOIN ESTADOCIVIL EC ON EC.CDESTACIVIL = PH.CDESTACIVIL
                          AND (EC.NRORG = PH.NRORG OR EC.NRORG = 0)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Logradouro*/
  LEFT JOIN LOGRADOURO LG ON LG.CDLOGRADOURO = EP.CDLOGRADOURO
                         AND (LG.NRORG = EP.NRORG OR LG.NRORG = 0)
  /*Instituição Ensino*/
  LEFT JOIN DES_FORMACAO FO ON FO.NRORG = PE.NRORG
                           AND FO.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                           AND FO.DTINICIO = (SELECT MAX(DTINICIO) FROM DES_FORMACAO
                                               WHERE NRORG = PE.NRORG
                                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                                 AND DTINICIO <= :P_DTMESCOMPETENC)
  LEFT JOIN ESTRUTURAH IEH ON IEH.NRORG = FO.NRORG
                          AND IEH.NRESTRUTURAM = FO.NRESTRUTURAM
                          AND IEH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND NRESTRUTURAM = IEH.NRESTRUTURAM
                                                       AND NRORG = IEH.NRORG)
  LEFT JOIN ENDERECOPARC IEE ON IEE.NRPARCNEGOCIO = IEH.NRPARCNEGOCIO
                            AND IEE.NRORG = IEH.NRORG
                            AND IEE.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MIE ON MIE.CDPAIS = IEE.CDPAIS
                         AND MIE.SGESTADO = IEE.SGESTADO
                         AND MIE.CDMUNICIPIO = IEE.CDMUNICIPIO
  LEFT JOIN PAIS PIE ON PIE.CDPAIS = IEE.CDPAIS
  /*Supervisor Estagio*/
  LEFT JOIN PARCNEGOCIO PNS ON PNS.NRPARCNEGOCIO = IEH.NRPARCRESPINF
                           AND PNS.NRORG = IEH.NRORG
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*Parametro do eSocial*/
  LEFT JOIN ESO_PARAMETRO PARAM ON PARAM.NRORG = :P_NRORG
                            AND PARAM.NRESTRUTURAM = :P_NRESTRUTURAM
                            AND PARAM.DTINIVIGENCIA <= :P_DTMESCOMPETENC
                            AND (PARAM.DTFIMVIGENCIA IS NULL OR PARAM.DTFIMVIGENCIA >= :P_DTMESCOMPETENC)
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidNaoPerProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidNaoPerProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidNaoPerNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
            "P_DTENVIOORIG" => $dataEnvioOrig,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

    /**
     * @param $nrpessoa
     * @return array    Associative array
     */
    public function getDadosS2300dependente($nrorg, $nrparcnegocio, $nrvinculom) {
        $sql = <<<SQL
/*getDadosS2300dependente*/
SELECT DISTINCT LPAD(T.CDESOCIAL,2,'0') tpDep,
       H.NMPESSOA nmDep,
       TO_CHAR(H.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
       REPLACE(REPLACE(H.NRCPFPESSOA, '.'), '-') cpfDep,
       NVL((SELECT 'S' FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '2'),'N') depIRRF,
       NVL((SELECT 'S' FROM FPA_DEPVINCULO WHERE NRVINCULOM = :P_NRVINCULOM AND NRPESSOA = P.NRPESSOA AND NRORG = P.NRORG AND NRTIPODEPENDE = '1'),'N') depSF,
       DECODE(NVL(H.NRCONDFISPES,1),1,'N','S') incTrab
  FROM RELACIONAPARC R
  JOIN TIPORELACIONA T ON (T.NRORG = R.NRORG OR T.NRORG = 0)
                      AND T.NRTIPORELACIONA = R.NRTIPORELACIONA
  JOIN GPE_PESSOA P ON R.NRPARCNEGRELAC = P.NRPARCNEGOCIO
                   AND R.NRORG = P.NRORG
  JOIN GPE_PESSOAH H ON P.NRPESSOA = H.NRPESSOA
                    AND P.NRORG = H.NRORG
                    AND H.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH H1
                                             WHERE H1.NRPESSOA = P.NRPESSOA
                                               AND H1.NRORG = P.NRORG
                                               AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  JOIN FPA_DEPVINCULO D ON D.NRPESSOA = P.NRPESSOA
                       AND D.NRORG = P.NRORG
                       AND D.NRVINCULOM = :P_NRVINCULOM
 WHERE R.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
   AND R.NRORG = :P_NRORG
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
            "P_NRVINCULOM" => $nrvinculom,
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
     /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2306($nrenvio, $nrchave, $nrestruturam, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2306*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,           
       --ideTrabSemVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       CE.CDESOCIAL codCateg,       
       --infoTSVAlteracao
       TO_CHAR(VH.DTULTATU,'YYYY-MM-DD') dtAlteracao,
       CASE WHEN CE.CDESOCIAL IN ('201','202','401','731','734','738') THEN (CASE WHEN VM.NRVINCULOEMPREG IN ('3','4','12','13') THEN '2' ELSE '1' END) ELSE '' END natAtividade,
       --infoComplementares
       --cargoFuncao
       CASE WHEN CE.CDESOCIAL IN ('201','202','305','701','711','712','721','722','731','734','738') THEN VH.NRCARGO ELSE NULL END codCargo,
       CASE WHEN CE.CDESOCIAL IN ('201','202','305','701','711','712','721','722','731','734','738') THEN VH.NRFUNCAO ELSE NULL END codFuncao,
       --remuneracao
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN ROUND(SL.VRSALARIO, 2) ELSE NULL END vrSalFx,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN DECODE(SL.IDTPSALARIO,'HORA','1','DIARIO','2','SEMANAL','3','QUINZENAL','4','MENSAL','5','TAREFA','6','7') ELSE '' END undSalFixo,
       CASE WHEN CE.CDESOCIAL IN ('701','711','712','721','722') THEN SUBSTR(SL.DSOBSERVACAO,1,90) ELSE '' END dscSalVar,
       --infoEstagiario
       DECODE(CE.CDESOCIAL,'901','N','') natEstagio,
       DECODE(CE.CDESOCIAL,'901',DECODE(GI.CDESOCIAL,'02','1','03','1','04','1','05','1','06','2','07','2','08','4','09','4','8'),'') nivEstagio,
       DECODE(CE.CDESOCIAL,'901','','') areaAtuacao,
       DECODE(CE.CDESOCIAL,'901','','') nrApol,
       DECODE(CE.CDESOCIAL,'901',ROUND(SL.VRSALARIO, 2),'') vlrBolsa,
       DECODE(CE.CDESOCIAL,'901',TO_CHAR(VH.DTFIMCONTRDETERMIN,'YYYY-MM-DD'),'') dtPrevTerm,
       --instEnsino
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEH.CDCNPJESTRUT,'.'),'/'),'-'),'') cnpjInstEnsino,
       DECODE(CE.CDESOCIAL,'901',NVL(IEH.NMRAZSOCESTRUT,FO.DSINSTITUICAO),'') nmRazao,
       DECODE(CE.CDESOCIAL,'901',IEE.DSENDERECO,'') instEnsino_dscLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NRIMOVELENDERECO,'') instEnsino_nrLograd,
       DECODE(CE.CDESOCIAL,'901',IEE.NMBAIRROENDERECO,'') instEnsino_bairro,
       DECODE(CE.CDESOCIAL,'901',REPLACE(REPLACE(REPLACE(IEE.NRCEPENDERECO,' '),'.'),'-'),'') instEnsino_cep,
       DECODE(CE.CDESOCIAL,'901',MIE.CDMUNICIBGE,'') instEnsino_codMunic,
       DECODE(CE.CDESOCIAL,'901',IEE.SGESTADO,'') instEnsino_uf,
       --ageIntegracao
       --supervisorEstagio
       DECODE(CE.CDESOCIAL,'901',PNS.NRINSCRICAOPARC,'') cpfSupervisor,
       DECODE(CE.CDESOCIAL,'901',PNS.NMPRINCIPALPARC,'') nmSuperv,
       VM.NRVINCULOM, PE.NRORG, PE.NRPARCNEGOCIO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTHISTORICO),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Local de Trabalho*/
  JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = VH.NRESTRUTGEREN
                    AND ET.NRORG = VH.NRORG
                    AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                              WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                AND ESTRUTURAH.NRORG = ET.NRORG)
  /*Supervisor Estagio*/
  LEFT JOIN PARCNEGOCIO PNS ON PNS.NRPARCNEGOCIO = ET.NRPARCRESPINF
                           AND PNS.NRORG = ET.NRORG
  /*Dados do Empregado - Dependentes*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTHISTORICO
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Salário*/
  LEFT JOIN GPE_ALTESALARIO SL ON SL.NRVINCULOM = VM.NRVINCULOM
                              AND SL.NRORG = VM.NRORG
                              AND SL.DTALTESALARIO = (SELECT MAX(DTALTESALARIO) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO <= LAST_DAY(:P_DTMESCOMPETENC)
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
                              AND SL.NROCORRENCIA =  (SELECT MAX(NROCORRENCIA) FROM GPE_ALTESALARIO
                                                       WHERE GPE_ALTESALARIO.DTALTESALARIO = SL.DTALTESALARIO
                                                         AND GPE_ALTESALARIO.NRVINCULOM = SL.NRVINCULOM
                                                         AND GPE_ALTESALARIO.NRORG = SL.NRORG)
  /*Grau Instrução*/
  LEFT JOIN GPE_GRAUINSTR GI ON GI.NRGRAUINSTR = PH.NRGRAUINSTR
                            AND (GI.NRORG = PH.NRORG OR GI.NRORG = 0)
  /*Instituição Ensino*/
  LEFT JOIN DES_FORMACAO FO ON FO.NRORG = PE.NRORG
                           AND FO.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                           AND FO.DTINICIO = (SELECT MAX(DTINICIO) FROM DES_FORMACAO
                                               WHERE NRORG = PE.NRORG
                                                 AND NRPARCNEGOCIO = PE.NRPARCNEGOCIO
                                                 AND DTINICIO <= :P_DTMESCOMPETENC)
  LEFT JOIN ESTRUTURAH IEH ON IEH.NRORG = FO.NRORG
                          AND IEH.NRESTRUTURAM = FO.NRESTRUTURAM
                          AND IEH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                     WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND NRESTRUTURAM = IEH.NRESTRUTURAM
                                                       AND NRORG = IEH.NRORG)
  LEFT JOIN ENDERECOPARC IEE ON IEE.NRPARCNEGOCIO = IEH.NRPARCNEGOCIO
                            AND IEE.NRORG = IEH.NRORG
                            AND IEE.CDTIPOENDERECO = 'PRINCIPAL'
  LEFT JOIN MUNICIPIO MIE ON MIE.CDPAIS = IEE.CDPAIS
                         AND MIE.SGESTADO = IEE.SGESTADO
                         AND MIE.CDMUNICIPIO = IEE.CDMUNICIPIO
  LEFT JOIN PAIS PIE ON PIE.CDPAIS = IEE.CDPAIS
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                WHERE DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND NRESTRUTURAM = EMH.NRESTRUTURAM
                                                  AND NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_DTHISTORICO" => $dtmescompetenc,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
     /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS2399($nrenvio, $nrchave, $nrestruturam, $nrtpambiente, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2399*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR ET.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) id,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') tpInsc,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR ET.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END nrInsc,
       --ideTrabSemVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-'), NVL(REPLACE(REPLACE(PH.NRNITPESSOA,'.'),'-'), PH.NRSUSPESSOA)) nisTrab,
       CE.CDESOCIAL codCateg,
       --infoTSVTermino
       TO_CHAR(VM.DTRESCISAOVINC,'YYYY-MM-DD') dtTerm,
       DECODE(CE.CDESOCIAL,'721',TD.CDESOCIAL,'') mtvDesligTSV,
       '' pensAlim,
       '' percAliment,
       '' vrAlim,
       --dmDev getDadosS2399dmDev       
       --ideEstabLot getDadosS2399ideEstabLot       
       --detVerbas getDadosS2399detVerbas       
       --detOper getDadosS2399detOper       
       --detPlano getDadosS2399detPlano       
       --procJudTrab getDadosS2399procJudTrab       
       --infoMV
       --getDadosS2399remunOutrEmpr
       DECODE(OV.VREVENTOFOLHA, NULL, '', DECODE(VH.IDMULTVINC, 'PRIMARIO', '1', '3')) indMV,
       null dtFimQuar,
       VM.NRORG, VM.NRVINCULOM, VM.DTRESCISAOVINC, VM.NRPESSOA, PE.NRPARCNEGOCIO, VH.NRESTRUTLEGAL, VH.NRESTRUTGEREN, CE.CDESOCIAL CDCATEGTRAB, 
       DECODE(:P_NRTPAMBIENTE, 1, DECODE(EMP.NRGRUPOESOCIAL, 3, :P_INIVALIDADEAMBNS, :P_INIVALIDADEAMBP), 2, :P_INIVALIDADEAMBPR) INIVALIDADEAMB, :P_DTMESCOMPETENC DTMESCOMPETENC,
       VH.NRAGENTENOCIVO
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Dados do Empregado - Pessoa*/
  JOIN GPE_PESSOA PE ON VM.NRPESSOA = PE.NRPESSOA
                    AND VM.NRORG    = PE.NRORG
  JOIN GPE_PESSOAH PH ON PE.NRPESSOA = PH.NRPESSOA
                     AND PE.NRORG    = PH.NRORG
                     AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                               WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                 AND GPE_PESSOAH.NRORG = PH.NRORG)
  /*Tipo Demissao*/                   
  JOIN FPA_TPDEMISSAO TD ON VM.NRTPDEMISSAO = TD.NRTPDEMISSAO
                        AND (TD.NRORG = :P_NRORG OR TD.NRORG = 0)
  /*Empregador*/
  JOIN ESTRUTURAM EMP ON EMP.NRORG = VH.NRORG
                     AND EMP.NRESTRUTURAM = VH.NRESTRUTLEGAL
  LEFT JOIN ESTRUTURAH ET ON ET.NRORG = EMP.NRORG
                         AND ET.NRESTRUTURAM = EMP.NRESTRUTURAM
                         AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                   WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                     AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                     AND ESTRUTURAH.NRORG = ET.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = ET.NRPARCNEGOCIO
                      AND PAR.NRORG = ET.NRORG
  /*Remuneração Outra Empresa*/
  LEFT JOIN (SELECT OV.NRORG, OV.NRVINCULOM, OV.NRPESSOA, SUM(ITC.VREVENTOFOLHA) VREVENTOFOLHA
               FROM GPE_VINCULOM OV
               JOIN FPA_ITECALCFOLHA ITC ON ITC.NRORG = OV.NRORG
                                        AND ITC.NRVINCULOM = OV.NRVINCULOM
                                        AND ITC.NREVENTOM = 3050
               JOIN FPA_CALCULOFOLHA CF ON CF.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                       AND CF.NRORG = ITC.NRORG
                                       AND CF.NRTIPOMOVIMENT = 1
                                       AND CF.NRTPMODALIDCAL = 1
                                       AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
              WHERE OV.NRORG = :P_NRORG
                --AND OV.NRVINCULOM = P_NRCHAVE
              GROUP BY OV.NRORG, OV.NRVINCULOM, OV.NRPESSOA) OV ON OV.NRORG = VM.NRORG
                                                               AND OV.NRPESSOA = VM.NRPESSOA
                                                               AND OV.NRVINCULOM <> VM.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM   
   AND VM.NRTIPOVINCULOM IN (2,3,10) --Estagiario, Socio, Autonomo
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRTPAMBIENTE" => $nrtpambiente,
            "P_INIVALIDADEAMBP" => $this->hcmParameters->getEsoIniValidPeriodProducao(),
            "P_INIVALIDADEAMBPR" => $this->hcmParameters->getEsoIniValidPeriodProdRest(),
            "P_INIVALIDADEAMBNS" => $this->hcmParameters->getEsoIniValidPeriodNacioSimples(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
        );
        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getDadosS2399dmDev($nrorg, $nrvinculom, $dtrescisao, $nrestrutlegal, $nrestrutgeren, $cdcategtrab, $inivalidadeamb, $dtmescompetenc, $nragentenocivo) {
        $sql = <<<SQL
/*getDadosS2399dmDev*/
SELECT ITC.NRCALCULOFOLHA AS ideDmDev,
       ITC.NRORG, ITC.NRVINCULOM, ITC.NRCALCULOFOLHA, :P_NRESTRUTLEGAL NRESTRUTLEGAL, :P_NRESTRUTGEREN NRESTRUTGEREN, :P_CDCATEGTRAB CDCATEGTRAB, :P_DTMESCOMPETENC DTMESCOMPETENC, :P_NRAGENTENOCIVO NRAGENTENOCIVO
  FROM FPA_ITECALCFOLHA ITC
  JOIN FPA_CALCULOFOLHA CFL ON CFL.NRORG = ITC.NRORG
                           AND CFL.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                           AND CFL.NRTIPOMOVIMENT IN (1,4,10,19)
                           AND CFL.NRTPMODALIDCAL = 1
 WHERE ITC.NRVINCULOM = :P_NRCHAVE
   AND ITC.NRORG = :P_NRORG
   AND ((CFL.NRTIPOMOVIMENT = 1 AND ITC.NREVENTOM = '3050' AND CFL.DTOCORRENCIA = (SELECT MAX(CF.DTOCORRENCIA)
                                                                                     FROM FPA_ITECALCFOLHA IT
                                                                                     JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = IT.NRORG
                                                                                                             AND CF.NRCALCULOFOLHA = IT.NRCALCULOFOLHA
                                                                                                             AND CF.NRTIPOMOVIMENT = CFL.NRTIPOMOVIMENT
                                                                                                             AND CF.NRTPMODALIDCAL = 1
                                                                                    WHERE IT.NRORG      = ITC.NRORG
                                                                                      AND IT.NRVINCULOM = ITC.NRVINCULOM
                                                                                      AND IT.NREVENTOM = ITC.NREVENTOM
                                                                                      AND CF.DTOCORRENCIA <= FIRST_DAY(:P_DTRESCISAOVINC)))
        OR (CFL.NRTIPOMOVIMENT = 10 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC))
        OR (CFL.NRTIPOMOVIMENT = 19 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC))
        OR (CFL.NRTIPOMOVIMENT = 4 AND ITC.NREVENTOM = '3030' AND CFL.DTOCORRENCIA = FIRST_DAY(:P_DTRESCISAOVINC) AND SUBSTR(CFL.DTOCORRENCIA, 3, 2) <> '12'))
   AND TO_DATE(:P_DTRESCISAOVINC) >= :P_INIVALIDADEAMB
   AND EXISTS (/*getDadosS2399detVerbas*/
               SELECT 1
                 FROM FPA_ITECALCFOLHA IT
                 JOIN FPA_CALCULOFOLHA CF ON IT.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                                         AND IT.NRORG = CF.NRORG
                 JOIN FPA_EVENTOH EH ON EH.NREVENTOM = IT.NREVENTOM
                                    AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                                    AND EH.DTMESCOMPETENC  = (SELECT MAX(EHH.DTMESCOMPETENC) FROM FPA_EVENTOH EHH
                                                               WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                                 AND EHH.NREVENTOM = EH.NREVENTOM
                                                                 AND EHH.NRORG = EH.NRORG)
                 /*Natureza da Rubrica*/
                 JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
                WHERE IT.NRORG = ITC.NRORG
                  AND IT.NRVINCULOM = ITC.NRVINCULOM
                  AND IT.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                  AND IT.VREVENTOFOLHA > 0
                  AND ((IT.IDEVENDEMONCALC = 'N' AND EH.IDEVENTODEMONST = 'N') OR (EH.IDEVENTODEMONST = 'S')) /*SE NÃO DEMONSTRATIVO NO CADASTRO E DEMONSTRATIVO NA FOLHA NÃO ENVIA*/
                  AND (SELECT NVL(MAX(INC.CDESOCIAL),'00')
                         FROM FPA_INCIDEVENTO IE
                         JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
                                               AND INSTR(','||TI.DSNRTIPOMOVIMENT||',',','||CF.NRTIPOMOVIMENT||',') > 0
                         JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
                        WHERE (IE.NRORG = :P_NRORG OR IE.NRORG = 0)
                          AND IE.NREVENTOM = IT.NREVENTOM
                          AND (IE.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                                     FROM FPA_GRPITEM 
                                                    WHERE NRGRPINCIDE = 47) OR TI.DSNRTIPOMOVIMENT IS NOT NULL)) NOT IN (31, 32, 33, 34, 35, 51, 52, 53, 54, 55, 81, 82, 83) /*codIncIRRF*/)
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_DTRESCISAOVINC" => $dtrescisao,
            "P_NRESTRUTLEGAL" => $nrestrutlegal,
            "P_NRESTRUTGEREN" => $nrestrutgeren,
            "P_CDCATEGTRAB" => $cdcategtrab,
            "P_INIVALIDADEAMB" => $inivalidadeamb,
            "P_NRAGENTENOCIVO" => $nragentenocivo,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2399ideEstabLot($nrorg, $nrvinculom, $nrcalculofolha, $nrestrutlegal, $nrestrutgeren, $cdcategtrab, $dtmescompetenc, $nragentenocivo) {
        $sql = <<<SQL
/*getDadosS2399ideEstabLot*/
SELECT DISTINCT
       DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') AS tpInsc,
       REPLACE(REPLACE(REPLACE(PN.NRINSCRICAOPARC,'.',''),'-',''),'/','') nrInsc,
       NVL(EMSUP.NRESTRUTURAM, EM.NRESTRUTURAM) AS codLotacao,
       CASE
         WHEN :P_CDCATEGTRAB IN ('101','102','103','104','105','106','111','201','202','301','302','303','305','306','307','308','309','731','734','738')
         THEN DECODE(AN.CDESOCIALNOCIVO,NULL,1,AN.CDESOCIALNOCIVO)
         ELSE NULL
       END AS grauExp,
       DECODE(NVL(CTSUP.CDESOCIAL,CT.CDESOCIAL),'01','1','02','2','03','3','') AS indSimples,
       EM.NRORG, :P_NRVINCULOM NRVINCULOM, :P_NRCALCULOFOLHA NRCALCULOFOLHA,
       (SELECT DECODE(NVL(SUM(IC.VREVENTOFOLHA),0),0,0,1) IDDETOPER
          FROM FPA_ITECALCFOLHA IC
          JOIN FPA_EVENTOH EVH ON (EVH.NRORG = IC.NRORG OR EVH.NRORG = 0)
                              AND EVH.NREVENTOM = IC.NREVENTOM
          JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EVH.NRNATURUBR
         WHERE IC.NRORG = :P_NRORG
           AND IC.NRVINCULOM = :P_NRVINCULOM
           AND IC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
           AND NR.CDESOCIAL = '9219') IDDETOPER /*existe desconto de assistencia medica*/,
           :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM ESTRUTURAM EM
  LEFT JOIN ESTRUTURAH EH ON EH.NRESTRUTURAM = EM.NRESTRUTURAM
                    AND EH.NRORG = EM.NRORG
                    AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC) FROM ESTRUTURAH EHH
                                              WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                AND EHH.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND EHH.NRORG = EH.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRORG = EM.NRORG
                     AND PN.NRPARCNEGOCIO = EM.NRPARCNEGOCIO
  LEFT JOIN CLASSTRIBUT CT ON CT.NRCLASSTRIBUT = EH.NRCLASSTRIBUT
  /*Tomador superior da estrutura gerencial*/
  LEFT JOIN TIPOHIERARQORG TH ON (TH.NRORG = EM.NRORG OR TH.NRORG = 0)
                             AND TH.NRTIPOESTRUTREL = :P_NRTIPOESTRUTGEREN
                             AND TH.NRTIPOESTRUTSUP = NVL(:P_NRTIPOESTRUTOMADOR, 7)
  LEFT JOIN HIERARQUIAORG HO ON HO.NRORG = EM.NRORG
                            AND HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG
                            AND HO.NRESTRUTURAMREL = :P_NRESTRUTGEREN
  LEFT JOIN ESTRUTURAM EMSUP ON EMSUP.NRESTRUTURAM = HO.NRESTRUTURAMSUP
                            AND EMSUP.NRORG = HO.NRORG
  LEFT JOIN ESTRUTURAH EHSUP ON EHSUP.NRESTRUTURAM = EMSUP.NRESTRUTURAM
                            AND EHSUP.NRORG = EMSUP.NRORG
                            AND EHSUP.DTMESCOMPETENC = (SELECT MAX(EHHSUP.DTMESCOMPETENC) FROM ESTRUTURAH EHHSUP
                                                         WHERE EHHSUP.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                           AND EHHSUP.NRESTRUTURAM = EH.NRESTRUTURAM
                                                           AND EHHSUP.NRORG = EH.NRORG)
  LEFT JOIN PARCNEGOCIO PNSUP ON PNSUP.NRPARCNEGOCIO = EMSUP.NRPARCNEGOCIO
                             AND PNSUP.NRORG = EMSUP.NRORG
  LEFT JOIN CLASSTRIBUT CTSUP ON CTSUP.NRCLASSTRIBUT = EHSUP.NRCLASSTRIBUT
  /*Grau de Exposição de Agente Nocivo*/
  LEFT JOIN GPE_AGENTENOCIVO AN ON AN.NRAGENTENOCIVO = :P_NRAGENTENOCIVO
 WHERE EM.NRORG = :P_NRORG
   AND EM.NRESTRUTURAM = :P_NRESTRUTLEGAL
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRTIPOESTRUTGEREN" => $this->hcmParameters->getTipoEstruturaGerencialOrg(),
            "P_NRTIPOESTRUTOMADOR" => $this->hcmParameters->getTipoEstruturaTomadorOrg(),
            "P_NRVINCULOM" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
            "P_NRESTRUTLEGAL" => $nrestrutlegal,
            "P_NRESTRUTGEREN" => $nrestrutgeren,
            "P_CDCATEGTRAB" => $cdcategtrab,
            "P_NRAGENTENOCIVO" => $nragentenocivo,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2399detVerbas($nrorg, $nrvinculom, $nrcalculofolha, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2399detVerbas*/
SELECT ITC.NREVENTOM AS codRubr,
       CASE WHEN TO_NUMBER(TO_CHAR(CF.DTOCORRENCIA,'MM')) BETWEEN 1 AND 11 AND CF.NRTIPOMOVIMENT = 4 THEN 22 ELSE CF.NRTIPOMOVIMENT END AS ideTabrubr,
       ROUND(DECODE(ITC.VRREFFOLHA, 0, '', ITC.VRREFFOLHA),2) AS qtdRubr,
       ROUND(EH.VRPERCENTUALEVENTO,2) AS fatorRubr,
       ROUND(DECODE(ITC.VRREFFOLHA, 0, '', (ITC.VREVENTOFOLHA / ITC.VRREFFOLHA)),2) AS vrUnit,
       ROUND(ITC.VREVENTOFOLHA,2) AS vrRubr,
       ITC.NRORG, ITC.NRVINCULOM, ITC.NRCALCULOFOLHA, :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM FPA_ITECALCFOLHA ITC
  JOIN FPA_CALCULOFOLHA CF ON ITC.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                          AND CF.NRORG = ITC.NRORG
  JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                     AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                     AND EH.DTMESCOMPETENC  = (SELECT MAX(EHH.DTMESCOMPETENC) FROM FPA_EVENTOH EHH
                                                WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND EHH.NREVENTOM = EH.NREVENTOM
                                                  AND EHH.NRORG = EH.NRORG)
  /*Natureza da Rubrica*/
  JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
 WHERE ITC.NRORG = :P_NRORG
   AND ITC.NRVINCULOM = :P_NRCHAVE
   AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
   AND ITC.VREVENTOFOLHA > 0
   AND ((ITC.IDEVENDEMONCALC = 'N' AND EH.IDEVENTODEMONST = 'N') OR (EH.IDEVENTODEMONST = 'S')) /*SE NÃO DEMONSTRATIVO NO CADASTRO E DEMONSTRATIVO NA FOLHA NÃO ENVIA*/
   AND (SELECT NVL(MAX(INC.CDESOCIAL),'00')
          FROM FPA_INCIDEVENTO IE
          JOIN FPA_TIPOINCIDE TI ON TI.NRTIPOINCIDE = IE.NRTIPOINCIDE
                                AND INSTR(','||TI.DSNRTIPOMOVIMENT||',',','||CF.NRTIPOMOVIMENT||',') > 0
          JOIN INCIDENCIAIRRF INC ON INSTR(','||INC.DSNRTIPOINCIDE||',',','||TI.NRTIPOINCIDE||',') > 0
         WHERE (IE.NRORG = :P_NRORG OR IE.NRORG = 0)
           AND IE.NREVENTOM = ITC.NREVENTOM
           AND (IE.NRTIPOINCIDE IN (SELECT NRTIPOINCIDE 
                                      FROM FPA_GRPITEM 
                                     WHERE NRGRPINCIDE = 47) OR TI.DSNRTIPOMOVIMENT IS NOT NULL)) NOT IN (31, 32, 33, 34, 35, 51, 52, 53, 54, 55, 81, 82, 83) /*codIncIRRF*/
 ORDER BY ITC.NREVENTOM
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2399detOper($nrorg, $nrvinculom, $nrcalculofolha, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2399detOper*/
SELECT cnpjOper, regANS, SUM(vrPgTit) vrPgTit,
       :P_NRORG NRORG, :P_NRCHAVE NRVINCULOM, :P_NRCALCULOFOLHA NRCALCULOFOLHA, CDCNPJESTRUT, :P_DTMESCOMPETENC DTMESCOMPETENC
FROM (SELECT DISTINCT
             REPLACE(REPLACE(REPLACE(ET.CDCNPJESTRUT,'.'),'/'),'-') AS cnpjOper,
             REPLACE(ET.CDFEDERALESTRUT,'-') AS regANS,
             ROUND(ITC.VREVENTOFOLHA,2) AS vrPgTit,
             ITC.NREVENTOM,
             ET.CDCNPJESTRUT
        FROM FPA_ITECALCFOLHA ITC
        JOIN FPA_CALCULOFOLHA CF ON ITC.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                                AND CF.NRORG = ITC.NRORG
        JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                           AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                           AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC)
                                                      FROM FPA_EVENTOH EHH
                                                     WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND EHH.NREVENTOM = EH.NREVENTOM
                                                       AND EHH.NRORG = EH.NRORG)
        JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
        JOIN VAB_BENEFICIARIO BF ON BF.NRVINCULOM = ITC.NRVINCULOM
                                AND BF.NRORG = ITC.NRORG
                                AND BF.IDTPBENEFICIARIO = 'T'
        JOIN VAB_BENEFICIOM BM ON BM.NRBENEFICIOM = BF.NRBENEFICIOM
                              AND BM.NRORG = BF.NRORG
                              AND BM.DTINICIOVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC)
                              AND (BM.DTFINALVIGENCIA IS NULL OR BM.DTFINALVIGENCIA >= :P_DTMESCOMPETENC)
        JOIN VAB_BENEFICIOH BH ON BH.NRBENEFICIOM = BM.NRBENEFICIOM
                              AND BH.NRORG = BM.NRORG
                              AND BH.TPPERIODOBENEF = 'MENSAL'
                              AND ITC.NREVENTOM IN (BH.NREVENTOMENSA, BH.NREVETITULARVINC, BH.NREVENTOCOPART, BH.NREVENTOCOPARTDEP)
                              AND BH.DTMESCOMPETENC = (SELECT MAX(BHH.DTMESCOMPETENC)
                                                         FROM VAB_BENEFICIOH BHH
                                                        WHERE BHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND BHH.NRBENEFICIOM = BH.NRBENEFICIOM
                                                          AND BHH.NRORG = BH.NRORG)
        JOIN ESTRUTURAM EM ON EM.NRESTRUTURAM = BH.NRESTRUTURAM
                          AND EM.NRORG = :P_NRORG
        LEFT JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = EM.NRESTRUTURAM
                          AND ET.NRORG = EM.NRORG
                          AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                     FROM ESTRUTURAH
                                                    WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                      AND ESTRUTURAH.NRORG = ET.NRORG)
       WHERE ITC.NRORG = :P_NRORG
         AND ITC.NRVINCULOM = :P_NRCHAVE
         AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
         AND NR.CDESOCIAL = '9219')
GROUP BY cnpjOper, regANS, CDCNPJESTRUT
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2399detPlano($nrorg, $nrvinculom, $nrcalculofolha, $cdcnpjestrut, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2399detPlano*/
SELECT tpDep, cpfDep, nmDep, dtNascto, SUM(vlrPgDep) vlrPgDep
FROM (SELECT DISTINCT
             TR.CDESOCIAL AS tpDep,
             REPLACE(REPLACE(PH.NRCPFPESSOA, '.'), '-') AS cpfDep,
             PH.NMPESSOA AS nmDep,
             TO_CHAR(PH.DTNASCPESSOA,'YYYY-MM-DD') dtNascto,
             ROUND(ITC.VREVENTOFOLHA/NVL(DECODE(ITC.VRREFFOLHA,0,1,ITC.VRREFFOLHA),1),2) AS vlrPgDep,
             ITC.NREVENTOM,
             :P_DTMESCOMPETENC DTMESCOMPETENC
        FROM FPA_ITECALCFOLHA ITC
        JOIN FPA_EVENTOH EH ON EH.NREVENTOM = ITC.NREVENTOM
                           AND (EH.NRORG = :P_NRORG OR EH.NRORG = 0)
                           AND EH.DTMESCOMPETENC = (SELECT MAX(EHH.DTMESCOMPETENC)
                                                      FROM FPA_EVENTOH EHH
                                                     WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND EHH.NREVENTOM = EH.NREVENTOM
                                                       AND EHH.NRORG = EH.NRORG)
        JOIN NATUREZARUBRICA NR ON NR.NRNATURUBR = EH.NRNATURUBR
        JOIN VAB_BENEFICIARIO BF ON BF.NRVINCULOM = ITC.NRVINCULOM
                                AND BF.NRORG = ITC.NRORG
                                AND BF.IDTPBENEFICIARIO = 'D'
                                AND BF.IDATIVO = 'S'
        JOIN GPE_VINCULOM VM ON VM.NRVINCULOM = BF.NRVINCULOM
                            AND VM.NRORG = BF.NRORG
        JOIN GPE_PESSOA PV ON PV.NRPESSOA = VM.NRPESSOA
                          AND PV.NRORG = VM.NRORG
        JOIN GPE_PESSOA PE ON PE.NRPESSOA = BF.NRPESSOA
                          AND PE.NRORG = BF.NRORG
        JOIN GPE_PESSOAH PH ON PH.NRPESSOA = PE.NRPESSOA
                           AND PH.NRORG = PE.NRORG
                           AND PH.DTMESCOMPETENC = (SELECT MAX(H1.DTMESCOMPETENC)
                                                      FROM GPE_PESSOAH H1
                                                     WHERE H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                       AND H1.NRPESSOA = PH.NRPESSOA
                                                       AND H1.NRORG = PH.NRORG)
        JOIN RELACIONAPARC RP ON RP.NRORG = :P_NRORG
                             AND RP.NRPARCNEGOCIO = PV.NRPARCNEGOCIO
                             AND RP.NRPARCNEGRELAC = PE.NRPARCNEGOCIO
        JOIN TIPORELACIONA TR ON TR.NRTIPORELACIONA = RP.NRTIPORELACIONA
        JOIN VAB_BENEFICIOH BH ON BH.NRBENEFICIOM = BF.NRBENEFICIOM
                              AND BH.NRORG = BF.NRORG
                              AND BH.TPPERIODOBENEF = 'MENSAL'
                              AND ITC.NREVENTOM IN (BH.NREVENTOMENSA, BH.NREVETITULARVINC, BH.NREVENTOCOPARTDEP)
                              AND BH.DTMESCOMPETENC = (SELECT MAX(BHH.DTMESCOMPETENC)
                                                         FROM VAB_BENEFICIOH BHH
                                                        WHERE BHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                          AND BHH.NRBENEFICIOM = BH.NRBENEFICIOM
                                                          AND BHH.NRORG = BH.NRORG)
        JOIN ESTRUTURAM EM ON EM.NRESTRUTURAM = BH.NRESTRUTURAM
                          AND EM.NRORG = :P_NRORG
        LEFT JOIN ESTRUTURAH ET ON ET.NRESTRUTURAM = EM.NRESTRUTURAM
                          AND ET.NRORG = EM.NRORG
                          AND ET.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                     FROM ESTRUTURAH
                                                    WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                      AND ESTRUTURAH.NRESTRUTURAM = ET.NRESTRUTURAM
                                                      AND ESTRUTURAH.NRORG = ET.NRORG)
       WHERE ITC.NRORG = :P_NRORG
         AND ITC.NRVINCULOM = :P_NRCHAVE
         AND ITC.NRCALCULOFOLHA = :P_NRCALCULOFOLHA
         AND NR.CDESOCIAL = '9219'
         AND ET.CDCNPJESTRUT = :P_CDCNPJESTRUT)
GROUP BY tpDep, cpfDep, nmDep, dtNascto
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRCALCULOFOLHA" => $nrcalculofolha,
            "P_CDCNPJESTRUT" => $cdcnpjestrut
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
     public function getDadosS2399procJudTrab($nrorg, $nrestrutura, $nrparcnegocio) {
        $sql = <<<SQL
/*getDadosS2399procJudTrab*/
SELECT AD.CDESOCIAL AS tpTrib,
       PA.NRPROCESSADMJUD AS nrProcJud,
       IPA.NRSUSPEXIGITPROC AS codSusp
  FROM GPE_PROCESSADMJUD PA
  JOIN GPE_ITEMPROCADMJUD IPA ON PA.NRORG = IPA.NRORG
                             AND PA.NRPROCESSADMJUD = IPA.NRPROCESSADMJUD
  LEFT JOIN GPE_ABDECISAO AD ON (AD.NRORG = IPA.NRORG OR AD.NRORG = 0)
                            AND AD.NRABDECISAO = IPA.NRABDECISAO
 WHERE PA.NRORG = :P_NRORG
   AND PA.NRESTRUTURAM = :P_NRESTRUTURAM
   AND PA.NRPARCNEGOCIO = :P_NRPARCNEGOCIO
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_NRESTRUTURAM" => $nrestrutura,
            "P_NRPARCNEGOCIO" => $nrparcnegocio,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2399remunOutrEmpr($nrorg, $nrvinculom, $nrPessoa, $dtmescompetenc) {
        $sql = <<<SQL
/*getDadosS2399remunOutrEmpr*/
SELECT DECODE(PN.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') AS tpInsc,
       CASE WHEN PN.CDTIPOINSCRICAO = 'CPF' OR EH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PN.NRINSCRICAOPARC ELSE SUBSTR(PN.NRINSCRICAOPARC,1,8) END nrInsc,
       CE.CDESOCIAL AS codCateg,
       ITC.VREVENTOFOLHA AS vlrRemunOE,
       :P_DTMESCOMPETENC DTMESCOMPETENC
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.NRORG = VM.NRORG
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)
                                                 FROM GPE_VINCULOH
                                                WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                  AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                                                  AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                  AND GPE_VINCULOH.NRORG = VH.NRORG)
  /*Outro Empregador*/
  JOIN ESTRUTURAM EM ON EM.NRORG = VM.NRORG
                    AND EM.NRESTRUTURAM = VH.NRESTRUTLEGAL
  LEFT JOIN ESTRUTURAH EH ON EH.NRESTRUTURAM    = EM.NRESTRUTURAM
                    AND EH.NRORG          = EM.NRORG
                    AND EH.DTMESCOMPETENC =
                      (SELECT MAX(EHH.DTMESCOMPETENC)
                      FROM ESTRUTURAH EHH
                      WHERE EHH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                      AND EHH.NRESTRUTURAM      = EH.NRESTRUTURAM
                      AND EHH.NRORG             = EH.NRORG)
  JOIN PARCNEGOCIO PN ON PN.NRORG             = EM.NRORG
                     AND PN.NRPARCNEGOCIO    = EM.NRPARCNEGOCIO
  /*Categoria Trabalhador*/
  JOIN GPE_VINCULOEMPREG VE ON VE.NRVINCULOEMPREG = VM.NRVINCULOEMPREG
  LEFT JOIN GPE_CATEGTRABESOC CE ON CE.NRCATEGESOC = VE.NRCATEGESOC
  /*Remuneração Outra Empresa*/
  JOIN (SELECT ITC.NRORG, ITC.NRVINCULOM, SUM(ITC.VREVENTOFOLHA) VREVENTOFOLHA
          FROM FPA_ITECALCFOLHA ITC
          JOIN FPA_CALCULOFOLHA CF ON CF.NRCALCULOFOLHA = ITC.NRCALCULOFOLHA
                                  AND CF.NRORG = ITC.NRORG
                                  AND CF.NRTIPOMOVIMENT = 1
                                  AND CF.NRTPMODALIDCAL = 1
                                  AND CF.DTOCORRENCIA = :P_DTMESCOMPETENC
         WHERE ITC.NRORG = :P_NRORG
           AND ITC.NREVENTOM = 3050
         GROUP BY ITC.NRORG, ITC.NRVINCULOM) ITC ON ITC.NRORG = VM.NRORG
                                                AND ITC.NRVINCULOM = VM.NRVINCULOM
 WHERE VM.NRPESSOA = :P_NRPESSOA
   AND VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM != :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $nrorg,
            "P_DTMESCOMPETENC" => $dtmescompetenc,
            "P_NRCHAVE" => $nrvinculom,
            "P_NRPESSOA" => $nrPessoa
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    /**
     * @param $nrenvio
     * @param $nrchave
     * @param $nrestruturam
     * @return array    Associative array
     */
    public function getDadosS3000($nrenvio, $nrchave, $nrestruturam, $nrenvioorig, $nrIdeTrabalhador, $nrIdeFolhaPgto) {
        $sql = <<<SQL
/*getDadosS3000*/
SELECT EC.NRCHAVE CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,   
       --infoExclusao
       EV.NMEVENTO tpEvento,
       EC.NRRECIBO2 nrRecEvt,
       --ideTrabalhador
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') cpfTrab,
       CASE WHEN EV.NMEVENTO IN ('S-1210', 'S-2190') THEN '' ELSE REPLACE(REPLACE(PH.NRPISPASEPPES,'.'),'-') END nisTrab,
       --ideFolhaPagto
       DECODE(EV.NMEVENTO, 'S-1210', DECODE(EC.NRTABELA,4,'2','1'), DECODE(CF.NRTIPOMOVIMENT,1,'1',2,'1',4,DECODE(TO_CHAR(CF.DTOCORRENCIA,'MM'),'12','2','1'),19,'1','')) indApuracao,
       DECODE(EV.NMEVENTO, 'S-1210', TO_CHAR(EC.DTMESCOMPETENC,'YYYY-MM'), TO_CHAR(DECODE(EV.NMEVENTO,'S-1210',IC.DTOCORRENCIA,CF.DTOCORRENCIA), DECODE(CF.NRTIPOMOVIMENT,4,DECODE(TO_CHAR(CF.DTOCORRENCIA,'MM'),'12','YYYY','YYYY-MM'),'YYYY-MM'))) perApur
  FROM ESO_ENVIOCADASTROS EC
  JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
  /*ideTrabalhador*/
  LEFT JOIN GPE_PESSOAH PH ON PH.NRORG = EC.NRORG
                          AND PH.NRPESSOA = :P_NRIDETRABALHADOR
                          AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                    WHERE GPE_PESSOAH.NRORG = PH.NRORG
                                                      AND GPE_PESSOAH.NRPESSOA = PH.NRPESSOA
                                                      AND GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  LEFT JOIN GPE_VINCULOM VM ON VM.NRORG = PH.NRORG
                           AND VM.NRPESSOA = PH.NRPESSOA
  LEFT JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                           AND VH.NRVINCULOM = VM.NRVINCULOM
                           AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                    WHERE GPE_VINCULOH.NRORG = VH.NRORG
                                                      AND GPE_VINCULOH.NRVINCULOM = VH.NRVINCULOM
                                                      AND GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  LEFT JOIN ESTRUTURAH EH ON EH.NRORG = VH.NRORG
                    AND EH.NRESTRUTURAM = VH.NRESTRUTLEGAL
                    AND EH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH H1
                                              WHERE H1.NRESTRUTURAM = EH.NRESTRUTURAM
                                                AND H1.NRORG = EH.NRORG
                                                AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*ideFolhaPagto*/
  LEFT JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = EC.NRORG
                               AND CF.NRCALCULOFOLHA = :P_NRIDEFOLHAPAGTO
  LEFT JOIN FPA_ITECALCFOLHA IC ON IC.NRORG = CF.NRORG
                               AND IC.NRCALCULOFOLHA = CF.NRCALCULOFOLHA
                               AND IC.NRVINCULOM = VM.NRVINCULOM
                               AND IC.NREVENTOM IN (3030,3050)
 WHERE EC.NRORG = :P_NRORG
   AND EC.NRENVIO = :P_NRENVIOORIG
   AND (((SUBSTR(EH.CDCNPJESTRUT,1,8) = SUBSTR(PAR.NRINSCRICAOPARC,1,8) OR EV.NMEVENTO = 'S-2190') AND :P_NRIDETRABALHADOR IS NOT NULL) OR (:P_NRIDETRABALHADOR IS NULL))
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam,
            "P_NRENVIOORIG" => $nrenvioorig,
            "P_NRIDETRABALHADOR" => $nrIdeTrabalhador,
            "P_NRIDEFOLHAPAGTO" => $nrIdeFolhaPgto,
        );
        
        return $this->connection->fetchAssoc($sql, $params);
    }

}